/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header:
 *
 * Purpose:
 *    Initial C routing code for the boot. This runs after the
 *    assembler boot code has finished. 
 *
 * Public Procedures:
 *    boot                    Main entry for the C part of the boot.
 *
 * Private Procedures:
 *    clock_enable            enable 88E6218 clocks needed to run the boot.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================
#include "88E6218.h"
#include "xmodem.h"
//#include "uart.h"
#include "test.h"
#include "gpio.h"
#include "board.h"
#include "mem_init.h"
#include "boot.h"

//=============================================================================
//                   PRIVATE PROCEDURES (ANSI Prototypes)
//=============================================================================
static void board_init( void );
static void clock_enable( void );

//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================

/******************************************************************************
 *
 * Name: boot
 *
 * Description:
 *   boot.s calls this to get the ThumbC part of the bootrom going. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *    Arg1 (u32 ): u32ramBist	- I&DSRAM bist result. (raw ARM mrc format).
 *    Arg2 (u32 ): u32tagBist	- I&DTAG  bist result.         "
 *    Arg2 (u32 ): u32cacheBist	- I&DCACHE  bist result.       "
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
#define RD_88E6208

#if 0
extern void
boot(UINT32 u32ramBist, UINT32 u32tagBist, UINT32 u32cacheBist)
{
 /*   extern void Thumb2ARM(void (void) );*/
    extern void bootMonitor(void);
    
	#ifdef RAM_BOOT_TEST 
	test_boot(u32ramBist, u32tagBist, u32cacheBist);
	#else

   /* board_init();*/
    
    bootMonitor();
    #endif /*RAM_BOOT_TEST*/
}
#endif
 
/*

//=============================================================================
//                         CODED PRIVATE PROCEDURES
//=============================================================================

 ******************************************************************************
 *
 * Name: board_init
 *
 * Description:
 *   This function initialize the following peripherals:
 		Memory controller
 		UART 
 *
 * Conditions For Use:
 *   Specific to 88E6218 Evaluation board. 
 *
 * Arguments:
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
 
extern void
board_init()
{
	#ifndef RD_88E6208
	UINT32 	success;
	SDRAM_DATA	sdram_data;
	#endif /* RD_88E6208 */
	
    clock_enable();
    
    /* UART initialization */
	//setup_uart();

/*	I2C_init(); - not implemented yet */

	/* Memory Interface Initialization */ 
	#ifdef RD_88E6208
	memoryInitDefualt();
	#else /* DB_88E6218 */
	success = sdramDetect(&sdram_data);
	if (success == true)
		memoryInit(&sdram_data);
	else /* RD_88E6208*/
		memoryInitDefualt();
	#endif
	
	/* QuoterDeck initialization */
/* 	QuoterdeckInit(); - not inplemented yet */
	
	
}



/******************************************************************************
 *
 * Name: clock_enable
 *
 * Description:
 *   Read modify write on the SCLKEN register to enable GPIO and SIU
 *   (serial). Also set the logic up for the 44Mhz->8Mhz derived
 *   clock. Supposedly this is to be the default value but that is
 *   not what was found in testing, thus the explicit set here.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
static void
clock_enable( void )
{
    GT_REG_WRITE(SCLKEN,(-1));		     // Enable all clocks.
    GT_REG_WRITE(SI_DIV,2);		     // how 8mhz is derived
                                             // from 44mhz. No doc on
                                             // this apparantly...
}	
