/*
    gtEnv.c - Environment handling for evaluation board parameters
*/

#include "MvCore.h"
//#include "gtFlash.h"
#include "gtMemory.h"
#include "gtEnv.h"
#include <string.h>

#define storeLastSectorBeforeDelete
/* 2KByte of environment parameters */
char environment[ENVSIZE];

/* Static variables */
#ifdef storeLastSectorBeforeDelete
#define STORETOADDR _16M
static unsigned int lastsector;
#endif
static DEVICE_WINDOW eeprom;
static bool eeprom_found= true;

static unsigned int origAddr , origWidth;
unsigned int envParam (const char *line) {
    unsigned int temp = 0;
    while ((line[temp] != ' ') && (line[temp] != 0)) {
        temp ++;
    }
    return(temp + 1);
}

char *envValue (char *line) {
    unsigned int temp = 0;
    while (line[temp] != ' ') {
        temp ++;
    }
    while (line[temp+1] == ' ') {
        temp ++;
    }
    return(&line[temp+1]);
}


char *envGetValue (const char *key) {
    char *temp;
    temp = envGet (key);
    if (temp == NULL) {
        return NULL;
    }
    return envValue (temp);
}

char *envGet (const char *key) {
    unsigned int temp;
    unsigned int len = strlen (key);
    for (temp = 0 ; temp < (ENVSIZE-len) ; temp ++) {
        if (environment[temp] == 0) {
            unsigned int len_orig = 1; 
            if (environment[temp+1] == 0) {
                break;
            }
            while (environment[temp+len_orig] != ' ') {
                len_orig ++;
            }
            len_orig --;
            if (len_orig < len) {
                len_orig = len;
            }
            if (!strncmp (&environment[temp+1] , key , len_orig)) {
                return &environment[temp+1];
            }
        }
    }
    return NULL;
}



bool envAdd (const char *param_orig) {
    unsigned int temp , param_len;
    bool sink;
    char param_temp[256]; /* We assume that the parameter name is less than 256 characters */
    char param[512]; /* We assume that the whole line is less than 512 characters */
    unsigned int point1 , point2;
    strcpy (param , param_orig);
    param_len = envParam (param);
    memcpy (param_temp , param , param_len);
    param_temp[param_len] = 0;
    /* If param already defined, delete it at first */
    if (envGet (param_temp) != NULL) {
        sink = envDel (param_temp);
    }
    for (temp = 0 ; temp < (ENVSIZE-1) ; temp ++) {
        if ((environment[temp] == 0 ) && (environment[temp+1] == 0)) {
            break;
        }
    }
    if((temp + strlen(param_orig)) > ENVSIZE)
        return false;
    temp ++;
      
    
    point1=0;
    point2=temp;
    while (param[point1] != 0) {
        if ((param[point1] == ' ') && ((param[point1 + 1] == '.') ||
                                      (param[point1 + 1] == '/') ||
                                      (param[point1 + 1] == ':') ||
                                      (param[point1 + 1] == '='))) {
             point1++;
             continue;
        }
        environment[point2] = param[point1];
        point2 ++;
        if ((param[point1+1] == ' ') && ((param[point1] == '.') ||
                                        (param[point1] == '/') ||
                                        (param[point1] == ':') ||
                                        (param[point1] == '='))) {
            point1++;
        }
    point1++;
    }
    point1 = 0;
    environment[point2] = 0;
    return true;
}

bool envDel (const char *param) {
    unsigned int offset , len;
    char *pointer;
    pointer = envGet(param);
    if (pointer != NULL) {
        offset = (unsigned int) pointer - (unsigned int) environment;
        len = strlen (pointer);
        memcpy (pointer , (char *) ((unsigned int) pointer + len + 1) , ENVSIZE-offset-len-1);
        return true;
    }
    return false;
}
#if 0
unsigned int envDump (char *buff) {
    unsigned int temp = 1;
    unsigned int elements = 0;
    unsigned int pointer = 0;
    while (1) {
        if (environment[temp] == 0) {
            break;
        }
        pointer += gtSprintf (&buff[pointer] , "%s\n",&environment[temp]);
        elements ++;
        do {
            temp ++;
        } while (environment[temp]);
        temp ++;
    }
    return elements;
}
#endif
static int getdec(char **ptr)
{
    char *p = *ptr;
    int ret=0;
    if ((*p < '0') || (*p > '9')) return(-1);
    while ((*p >= '0') && (*p <= '9')) {
        ret = ret*10 + (*p - '0');
        p++;
    }
    *ptr = p;
    return(ret);
}

unsigned int ipToUint(char *str) {
    unsigned int i;
    unsigned int ip = 0;
    unsigned int temp , point = 0;
    char p_alloc[256];
    char *p;
    int val;
    /* Remove the spaces from the IP address before conversion */
    p = p_alloc;
    for (temp = 0 ; temp < strlen(str) ; temp ++) {
        if (str[temp] != ' ') {
            p[point] = str[temp];
            point++;
        }
    }
    p[point] = 0;
    if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
    if (*p != '.') return(0);
    p++;
    ip = val;
    if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
    if (*p != '.') return(0);
    p++;
    ip = (ip << 8) | val;
    if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
    if (*p != '.') return(0);
    p++;
    ip = (ip << 8) | val;
    if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
    return((ip << 8) | val);
}


