/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtMemory.h - Memory mappings and remapping functions declarations 
*
* DESCRIPTION:
*       None.
*
* DEPENDENCIES:
*       None.
*
*******************************************************************************/

#ifndef __INCgtMemoryh
#define __INCgtMemoryh

/* includes */
#include "MvCore.h"
#include "EagleReg.h"

/* defines */
#define MEM_DONT_MODIFY       0xffffffff
#define MEM_PARITY_SUPPORT    0x40000000

#define _8BIT                 0x00000000
#define _16BIT                0x00100000
#define _32BIT                0x00200000
#define _64BIT                0x00300000

/* typedefs */
typedef struct _deviceParam 
{                             /* boundary values */
    unsigned int turnOff;     /* 0x0 - 0xf       */
    unsigned int acc2First;   /* 0x0 - 0x1f      */ 
    unsigned int acc2Next;    /* 0x0 - 0x1f      */
    unsigned int ale2Wr;      /* 0x0 - 0xf       */
    unsigned int wrLow;       /* 0x0 - 0xf       */
    unsigned int wrHigh;      /* 0x0 - 0xf       */
    unsigned int deviceWidth; /* in Bytes        */
} DEVICE_PARAM;  

typedef enum _memoryBank{MEM_BANK0, MEM_BANK1, MEM_BANK2, MEM_BANK3
                        } MEMORY_BANK;

typedef enum _pciWindow{PCI_0_IO, PCI_0_MEM0, PCI_0_MEM1, PCI_0_MEM2, PCI_0_MEM3
#ifdef INCLUDE_PCI_1
                       ,PCI_1_IO, PCI_1_MEM0, PCI_1_MEM1, PCI_1_MEM2, PCI_1_MEM3
#endif /* INCLUDE_PCI_1 */
                       } PCI_WINDOW;

typedef enum _deviceWindow{DEVICE0, DEVICE1, DEVICE2, DEVICE3, BOOT_DEVICE
                          } DEVICE_WINDOW;

typedef enum _memoryProtectRegion{MEM_REGION0, MEM_REGION1, MEM_REGION2,       \
                                  MEM_REGION3, MEM_REGION4, MEM_REGION5,       \
                                  MEM_REGION6, MEM_REGION7                     \
                                 } MEMORY_PROTECT_REGION;

/* When defining a protected window , this paramter indicates whether it
   is accessible or not */
typedef enum _memoryAccess       {MEM_ACCESS_ALLOWED, MEM_ACCESS_FORBIDDEN
                                 } MEMORY_ACCESS;

/* When defining a protected window , this paramter indicates whether it
   is accessible for writing or not */
typedef enum _memoryAccessWrite  {MEM_WRITE_ALLOWED, MEM_WRITE_FORBIDDEN
                                 } MEMORY_ACCESS_WRITE;

/* When defining a protected window , this paramter indicates whether caching
   is alowed or forbidden */
typedef enum _memoryCacheProtect {MEM_CACHE_ALLOWED, MEM_CACHE_FORBIDDEN
                                 } MEMORY_CACHE_PROTECT;
#ifdef INCLUDE_SNOOP_SUPPORT
typedef enum _memorySnoopType    {MEM_NO_SNOOP, MEM_SNOOP_WT, MEM_SNOOP_WB     \
                                 } MEMORY_SNOOP_TYPE;

typedef enum _memorySnoopRegion  {MEM_SNOOP_REGION0, MEM_SNOOP_REGION1,        \
                                  MEM_SNOOP_REGION2, MEM_SNOOP_REGION3         \
                                 } MEMORY_SNOOP_REGION;

#endif /* INCLUDE_SNOOP_SUPPORT */

/* Functions */    

/* Returns base address and size for a given memory bank, device or PCI memory 
   window. */ 
unsigned int gtMemoryGetMemoryBankBaseAddr(MEMORY_BANK bank);
unsigned int gtMemoryGetPciWindowBaseAddr (PCI_WINDOW window); 
unsigned int gtMemoryGetDeviceWindowBaseAddr(DEVICE_WINDOW device);
unsigned int gtMemoryGetMemoryBankSize  (MEMORY_BANK bank);
unsigned int gtMemoryGetPciWindowSize(PCI_WINDOW window); 
unsigned int gtMemoryGetDeviceWindowSize(DEVICE_WINDOW device);

/* Returns the given device's width. */
unsigned int gtMemoryGetDeviceWidth(DEVICE_WINDOW device);

/* Set new base and size for DRAM banks (SCS 0,1,2 or 3), the devices (CS 0,1,
   2,3 or BOOT CS) or the PCI memory windows. */
void gtMemorySetMemoryBank(MEMORY_BANK bank, unsigned int bankBase, 
                           unsigned int bankSize);

void gtMemorySetPciWindow(PCI_WINDOW window,unsigned int pciWindowBase,
                          unsigned int pciWindowSize);    

void gtMemorySetDeviceWindow(DEVICE_WINDOW device, unsigned int deviceBase,
                             unsigned int deviceSize); 

/* Change the Internal Register Base Address to a new given Address. */
void gtMemoryMapInternalRegistersSpace(unsigned int internalRegBase);

/* Returns internal Register Space Base Address. */
unsigned int gtMemoryGetInternalRegistersSpace(void);

/* Writes a new remap value to the remap register */
unsigned int gtMemorySetPciRemapValue(PCI_WINDOW memoryWindow,
                                      unsigned int remapValueHigh,
                                      unsigned int remapValueLow);

/* Configurate the protection region feature for a given region. */
bool gtMemorySetProtectRegion(MEMORY_PROTECT_REGION region, 
                              MEMORY_ACCESS memoryAccess,
                              MEMORY_ACCESS_WRITE memoryWrite,
                              MEMORY_CACHE_PROTECT cacheProtection,
                              unsigned int baseAddress,
                              unsigned int regionSize);

/* Extarcts the device's parameters */
bool gtMemoryGetDeviceParam(DEVICE_PARAM *deviceParam, DEVICE_WINDOW device);

/* Sets new parameters for a device */
bool gtMemorySetDeviceParam(DEVICE_PARAM *deviceParam, DEVICE_WINDOW device);

#ifdef INCLUDE_SNOOP_SUPPORT
/* Configurate the snoop feature to a given space. */
bool gtMemorySetRegionSnoopMode(MEMORY_SNOOP_REGION region,
                                MEMORY_SNOOP_TYPE snoopType,
                                unsigned int baseAddress,
                                unsigned int regionSize);
#endif /* INCLUDE_SNOOP_SUPPORT */

#endif /* __INCgtMemoryh */

