/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gSdma.h 
*
* DESCRIPTION:
*       Sdma functions header file. 
*
* DEPENDENCIES:
*       gtMpsc.c/.h
*       gtDesc.c/.h
*
*******************************************************************************/

#ifndef _INCgtsdmah 
#define _INCgtsdmah

/* includes */             
#include "MvCore.h"
#include "EagleReg.h"


/* typedefs */
typedef  enum  _sdmaGroup {SDMA_GROUP0
#ifdef INCLUDE_SDMA_GROUP_1
                          ,SDMA_GROUP1
#endif /* INCLUDE_SDMA_GROUP_1 */
                          } SDMA_GROUP;

typedef enum _sdmaChannel {SDMA0 ,SDMA1
#ifdef INCLUDE_SDMA_CHANNELS_2_7
                          ,SDMA2 ,SDMA3
                          ,SDMA4 ,SDMA5
                          ,SDMA6 ,SDMA7
#endif /* INCLUDE_SDMA_CHANNELS_2_7 */
                          } SDMA_CHANNEL;

/* defines */
#ifdef INCLUDE_SDMA_GROUP_1
    /* Write SDMA Group Register (SGC) */
    #define GT_SDMA_SGC_CONFIG(SgcConfig)								  	\
			    GT_REG_WRITE(SDMA_GROUP_CONFIG_REG, SgcConfig)
#endif /* INCLUDE_SDMA_GROUP_1 */

/* SDMA command register - SDCMx */            
#define SDMA_ABORT_RX_BIT                       BIT15
#define SDMA_STOP_TX_BIT                        BIT16
#define SDMA_ENABLE_RX                          BIT7
#define SDMA_TX_DEMAND                          BIT23
#define SDMA_ABORT_TX_BIT                       BIT31

/* SDMA configuration register - SDCx */           
#define SDMA_CUNFIGURATION_DEFAULT              0xfc /* RC=0xf,BLMT=1,BLMR=1 */
#define SDMA_RECEIVE_FIFO_THRESHOLD_8_BYTES     0
#define SDMA_RECEIVE_FIFO_THRESHOLD_HALF_FIFO   BIT0
#define SDMA_MULTI_FRAME_MODE                   0
#define SDMA_SINGLE_FRAME_MODE                  BIT1
#define SDMA_BLMR                               BIT6
#define SDMA_BLMT                               BIT7
#define SDMA_PCI_OVERRIDE                       BIT8
#define SDMA_RIFB                               BIT9
#define SDMA_BURST_64B                          (BIT12|BIT13)
#define SDMA_BURST_32B                          BIT13
#define SDMA_BURST_16B                          BIT12
#define SDMA_BURST_8B                           0

/* Abort SDMA RX immediately */
#define GT_SDMA_ABORT_RX(group, channel)			   	   					\
			GT_REG_WRITE(SDMA_COMM_REG(group, channel), SDMA_ABORT_RX_BIT)

/* Abort SDMA TX immediately */
#define GT_SDMA_ABORT_TX(group, channel)								   	\
			GT_REG_WRITE(SDMA_COMM_REG(group, channel), SDMA_ABORT_TX_BIT)

/* stop SDMA TX at the end of frame */
#define GT_SDMA_STOP_TX(group, channel)										\
			GT_REG_WRITE(SDMA_COMM_REG(group, channel), SDMA_STOP_TX_BIT)

/* Write to SDMA configuration reg */
#define GT_SDMA_SDC_CONFIG(group, channel, sdcConfig)					   	\
            GT_REG_WRITE(SDMA_CONFIG_REG(group, channel), sdcConfig)
			 
/* functions */
void gtSdmaStartTransmit(SDMA_GROUP group, SDMA_CHANNEL channel, 
                         unsigned int firstTxDescAddr);
void gtSdmaEnableReception(SDMA_GROUP group, SDMA_CHANNEL channel,  
                           unsigned int firstRxDescAddr);
void gtAbortSdma(SDMA_GROUP group, SDMA_CHANNEL channel);
void gtSdmaTxDemand(SDMA_GROUP group, SDMA_CHANNEL channel);
void gtSdmaRxEnable(SDMA_GROUP group, SDMA_CHANNEL channel);

#endif  /* _INCgtsdmah */
