/**************************************************************************
 *ETHERBOOT -  BOOTP/TFTP Bootstrap Program
 *Author: Martin Renters
 *Date: Dec/93
 *
 *Literature dealing with the network protocols:
 *ARP - RFC826
 *RARP - RFC903                                                                               
 *UDP - RFC768
 *BOOTP - RFC951, RFC2132 (vendor extensions)
 *DHCP - RFC2131, RFC2132 (options)
 *TFTP - RFC1350, RFC2347 (options), RFC2348 (blocksize), RFC2349 (tsize)
 *RPC - RFC1831, RFC1832 (XDR), RFC1833 (rpcbind/portmapper)
 *NFS - RFC1094, RFC1813 (v3, useful for clarifications, not implemented)
 *
 **************************************************************************/

#include "gtTftp.h"
#include "gtEnv.h"
#include "gtEther.h"
#include "gtDesc.h"
#include "gtDma.h"
#include "gtMemory.h"
//#include "gtStdio.h" /* Rabeeh */
#include <string.h> 
//#include "gtFlash.h"
//#include "ev64260.h"
#include <setjmp.h>
#ifdef ETHER_SWITCH
extern void etherswitch_init(void);
extern void etherSwitch_Init(void);
#endif
extern jmp_buf out_of_here;
int gwait = 0;

#if 0
#define ___swab16(x) \
({ \
        unsigned short __x = (x); \
        ((unsigned short)( \
                (((unsigned short)(__x) & (unsigned short)0x00ffU) << 8) | \
                (((unsigned short)(__x) & (unsigned short)0xff00U) >> 8) )); \
})

#define ___swab32(x) \
({ \
        unsigned int __x = (x); \
        ((unsigned int)( \
                (((unsigned int)(__x) & (unsigned int)0x000000ffUL) << 24) | \
                (((unsigned int)(__x) & (unsigned int)0x0000ff00UL) <<  8) | \
                (((unsigned int)(__x) & (unsigned int)0x00ff0000UL) >>  8) | \
                (((unsigned int)(__x) & (unsigned int)0xff000000UL) >> 24) )); \
})
#else
unsigned ___swab32(unsigned x)
{
    return ((    (x >> 24) & 0xff) \
	    |  ((x >>  8) & 0xff00) \
	    |  ((x <<  8) & 0xff0000L) \
	    |  ((x << 24) & 0xff000000L));
}
unsigned short ___swab16(unsigned short x)
{
    return (((x >>  8) & 0xff) \
	    |  ((x <<  8) & 0xff00L));
}
#endif

#define ntohl(x) ___swab32(x)
#define htonl(x) ___swab32(x)
#define ntohs(x) ___swab16(x)
#define htons(x) ___swab16(x)

#define EAGLE_MAC_READ(x,y) *y = *(volatile unsigned int *) (0x80008000 + x)
#define EAGLE_MAC_WRITE(x,y) *(volatile unsigned int *) (0x80008000 + x) = y;

unsigned long CurrRxDesc; /* Rx current Descriptor Number*/
      

unsigned int originalSDCR;
int portNum = 0;
struct arptable_t arptable[MAX_ARP];
static char packet[ETH_MAX_PACKET];
int reached_block = 1;
char kernel[100]={};
char * copy_to_addr = 0;
DMA_ENGINE dmaEng;
unsigned int dmaCommand;
char         mac[6];
unsigned int macLow, macHigh;

TX_DESC *eth0_tx_desc;             //pointer to desc in tx descs memory space
RX_DESC *eth0_rx_desc;             //pointer to desc in rx descs memory space
char *eth0_tx_buffer;              //pointer to tx buffer in memory space (not really needed...)
char *eth0_rx_buffer[NR];          //pointers to allinged rx bufs in rx bufs memory space 
unsigned long   fileSize=0;
unsigned int hashTableBase;

static const unsigned char broadcast[] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

struct nic  nic =
{
    0,  /* reset */
    0,              /* poll */
    0,      /* transmit */
    0,  /* disable */

/* bcc has problems with complicated casts */
    0,          /* no aui */
    arptable[ARP_CLIENT].node,  /* node_addr */
    packet,         /* packet */
    0,          /* packetlen */
    0           /* priv_data */
};


unsigned int currticks(void) 
{
    return 0;
}

/**
 * strlen - Find the length of a string
 * @s: The string to be sized
 */
size_t strlen(const char * s)
{
        const char *sc;

        for (sc = s; *sc != '\0'; ++sc)
                /* nothing */;
        return sc - s;
}

/**
 * strncmp - Compare two length-limited strings
 * @cs: One string
 * @ct: Another string
 * @count: The maximum number of bytes to compare
 */
int strncmp(const char * cs,const char * ct,size_t count)
{
        register signed char __res = 0;

        while (count) {
                if ((__res = *cs - *ct++) != 0 || !*cs++)
                        break;
                count--;
        }

        return __res;
}

/**
 * strcat - Append one %NUL-terminated string to another
 * @dest: The string to be appended to
 * @src: The string to append to it
 */
char * strcat(char * dest, const char * src)
{
        char *tmp = dest;

        while (*dest)
                dest++;
        while ((*dest++ = *src++) != '\0')
                ;

        return tmp;
}
/**
 * strchr - Find the first occurrence of a character in a string
 * @s: The string to be searched
 * @c: The character to search for
 */
char * strchr(const char * s, int c)
{
        for(; *s != (char) c; ++s)
                if (*s == '\0')
                        return NULL;
        return (char *) s;
}
/**
 * strncpy - Copy a length-limited, %NUL-terminated string
 * @dest: Where to copy the string to
 * @src: Where to copy the string from
 * @count: The maximum number of bytes to copy
 *
 * Note that unlike userspace strncpy, this does not %NUL-pad the buffer.
 * However, the result is not %NUL-terminated if the source exceeds
 * @count bytes.
 */
char * strncpy(char * dest,const char *src,size_t count)
{
        char *tmp = dest;

        while (count-- && (*dest++ = *src++) != '\0')
                /* nothing */;

        return tmp;
}
/**
 * strcpy - Copy a %NUL terminated string
 * @dest: Where to copy the string to
 * @src: Where to copy the string from
 */
char * strcpy(char * dest,const char *src)
{
        char *tmp = dest;

        while ((*dest++ = *src++) != '\0')
                /* nothing */;
        return tmp;
}
void * memcpy(void * to, const void * from, unsigned int n)
{
    char *addr;
    for (addr = (char *) 0 ; addr < (char *) n ; addr = (char *) ((unsigned long)addr + (unsigned long) 1)) 
    {
        * (char *) ((unsigned long)addr+(unsigned long)to) = * (char *) ((unsigned long)addr + (unsigned long)from);
    }


    return(to);
}
#if 0
void * memset(void * s, char c,unsigned int count)
{
    unsigned int counter;
    for (counter = 0 ;counter < count ; counter ++)
    {
        *(char *) ((unsigned int) s + counter) = c;
    }
    return s;
}

int memcmp (char *first , char *second , unsigned int number) 
{
    unsigned int temp;
    for (temp = 0 ; temp < number ; temp++)
    {
        if (first[temp] > second[temp]) return 1;
        if (first[temp] < second[temp]) return 1;
    }
    return 0;
}

/* Dummies - should be removed - Rabeeh */
void atof (unsigned int dummy) {}
extern size_t strcspn (__const char *__s, __const char *__reject) {
  return 0;
}
/**
 * strcmp - Compare two strings
 * @cs: One string
 * @ct: Another string
 */
int strcmp(const char * cs,const char * ct)
{
        register signed char __res;

        while (1) {
                if ((__res = *cs - *ct++) != 0 || !*cs++)
                        break;
        }

        return __res;
}
#endif
int getdec(char **ptr)
{
    char *p = *ptr;
    int ret=0;
    if ((*p < '0') || (*p > '9')) return(-1);
    while ((*p >= '0') && (*p <= '9')) 
    {
        ret = ret*10 + (*p - '0');
        p++;
    }
    *ptr = p;
    return(ret);
}

int strcasecmp(const char *a, const char *b)
{
    while (*a && *b && (*a & ~0x20) == (*b & ~0x20)) 
    {
        a++; b++;
    }
    return((*a & ~0x20) - (*b & ~0x20));
}

 
unsigned int strtohex (char ch) 
{
    if ((ch >= '0') && (ch <= '9')) 
    {
        return(ch - '0');
    }
    if ((ch >= 'a') && (ch <= 'f')) 
    {
        return(ch - 'a');
    }
    if ((ch >= 'A') && (ch <= 'F')) 
    {
        return(ch - 'A');
    }
    return 0;
}

/**************************************************************************
 *tftp_load_network - Network Loading Kick Off Routine
 *Inputs :
 *Outputs :
 **************************************************************************/
bool tftp_load_network(eth_initialization ethInitVals)
{
    int i;
    char arp_host[256];
    unsigned int temp , point = 0;
    unsigned int* macLowPtr = (unsigned int*)(mac+2);
    unsigned int* macHighPtr = (unsigned int*)(mac);
    
    portNum = ethInitVals.portNumber;
//    armond_printf("\nBased on Etherboot/32\n");
    /* Reset the ARP Table*/
    for (i = 0 ;i < MAX_ARP ; i ++) 
    {
        arptable[i].node[0] = 0x0;
        arptable[i].node[1] = 0x0;
        arptable[i].node[2] = 0x0;
        arptable[i].node[3] = 0x0;
        arptable[i].node[4] = 0x0;
        arptable[i].node[5] = 0x0;
    }

    /* Remove the spaces from the MAC address before conversion */
    for (temp = 0 ; temp < strlen (ethInitVals.mac) ; temp ++) 
    {
        if (ethInitVals.mac[temp] != ' ') 
        {
            arp_host[point] = ethInitVals.mac[temp];
            point++;
        }
    } 
    mac[0] = (strtohex(arp_host[0]) << 4) + strtohex(arp_host[1]);
    mac[1] = (strtohex(arp_host[3]) << 4) + strtohex(arp_host[4]);
    mac[2] = (strtohex(arp_host[6]) << 4) + strtohex(arp_host[7]);
    mac[3] = (strtohex(arp_host[9]) << 4) + strtohex(arp_host[10]);
    mac[4] = (strtohex(arp_host[12]) << 4) + strtohex(arp_host[13]);
    mac[5] = (strtohex(arp_host[15]) << 4) + strtohex(arp_host[16]);
    macLow = *macLowPtr;
    macHigh = *macHighPtr;
    macHigh = macHigh >> 16;
    kernel[0]=0;
    strcpy(kernel, ethInitVals.fileName);
    atob (&copy_to_addr , ethInitVals.loadAddr , 16);
    arptable[ARP_CLIENT].node[0] = (strtohex(arp_host[0]) << 4) + strtohex(arp_host[1]);
    arptable[ARP_CLIENT].node[1] = (strtohex(arp_host[3]) << 4) + strtohex(arp_host[4]);
    arptable[ARP_CLIENT].node[2] = (strtohex(arp_host[6]) << 4) + strtohex(arp_host[7]);
    arptable[ARP_CLIENT].node[3] = (strtohex(arp_host[9]) << 4) + strtohex(arp_host[10]);
    arptable[ARP_CLIENT].node[4] = (strtohex(arp_host[12]) << 4) + strtohex(arp_host[13]);
    arptable[ARP_CLIENT].node[5] = (strtohex(arp_host[15]) << 4) + strtohex(arp_host[16]);
    //    arptable[ARP_CLIENT].ipaddr.s_addr = ipToUint (envGetValue ("tftpHostIP")); Rabeeh
    arptable[ARP_CLIENT].ipaddr.s_addr = htonl(ipToUint ("192.168.0.2"));
    arptable[ARP_SERVER].ipaddr.s_addr = htonl(ipToUint ("192.168.0.1"));

    /* Do not use GateWay because of static IP of host / user. If you want to use
       the GateWay then declare your server as the gateway*/
    arptable[ARP_GATEWAY].ipaddr.s_addr = 0;

    eth0_tx_desc = (TX_DESC*)ethInitVals.txDescBaseAddr;
    eth0_rx_desc = (RX_DESC*)ethInitVals.rxDescBaseAddr; 
    eth0_tx_buffer = (char*)ethInitVals.txDataBuffBaseAddr;
    eth0_rx_buffer[0] = (char*)ethInitVals.rxDataBuffBaseAddr;
    dmaEng = ethInitVals.dmaEngine;
    dmaCommand = ethInitVals.dmaCommand;
    hashTableBase = ethInitVals.hashTableBaseAddr;
    /* Init FE port */
    eth_init(portNum); 
    if (tftp(kernel,downloadkernel))
    {
//        armond_printf("\nFinished downloading %s  size=%d bytes.\n",ethInitVals.fileName,fileSize);
        return true;
    }
    return false;
}


/**************************************************************************
 *DEFAULT_NETMASK - Return default netmask for IP address
 *
 *Input :
 *Output : Netmask of network (calculated using client IP)
 **************************************************************************/
unsigned int default_netmask(void)
{
    int net = ntohl(arptable[ARP_CLIENT].ipaddr.s_addr) >> 24;
    if (net <= 127)
        return(htonl(0xff000000));
    else if (net < 192)
        return(htonl(0xffff0000));
    else
        return(htonl(0xffffff00));
}

/**************************************************************************
 *UDP_TRANSMIT - Send a UDP datagram
 *
 *Inputs :
 *destip - IP of destination machine
 *srcsock - source socket of client
 *destsock - destination socket of server
 *len - length of the udp datagram
 *buf - pointer to the datagram
 *
 *Outputs : success/failure
 **************************************************************************/
int udp_transmit(unsigned long destip, unsigned int srcsock,
                 unsigned int destsock, int len, const void *buf)
{
    struct iphdr *ip;
    struct udphdr *udp;
    struct arprequest arpreq;
    int arpentry, i;
    int retry;
    ip = (struct iphdr *)buf;
    udp = (struct udphdr *)((long)buf + sizeof(struct iphdr));
    ip->verhdrlen = 0x45;
    ip->service = 0;
    ip->len = htons(len);
    ip->ident = htons(0x1843);//0; // Changed by Rabeeh
    ip->frags = htons(0x4000);
    ip->ttl = 255;
    ip->protocol = IP_UDP;
    ip->chksum = 0;
    ip->src.s_addr = arptable[ARP_CLIENT].ipaddr.s_addr;
    ip->dest.s_addr = destip;
    ip->chksum = ipchksum((unsigned short *)buf, sizeof(struct iphdr));
    udp->src = htons(srcsock);
    udp->dest = htons(destsock);
    udp->len = htons(len - sizeof(struct iphdr));
    udp->chksum = 0;
    if (destip == IP_BROADCAST) 
    {
        eth_transmit(portNum, (char*)broadcast, IP, len, (char*)buf);
    } 
    else 
    {
        for (arpentry = 0; arpentry<MAX_ARP; arpentry++)
            if (arptable[arpentry].ipaddr.s_addr == destip) break;
        if (arpentry == MAX_ARP) 
        {
//            armond_printf("%I is not in my arp table!\n", destip);
            return(false);
        }
        for (i = 0; i<ETHER_ADDR_SIZE; i++)
            if (arptable[arpentry].node[i]) break;
        if (i == ETHER_ADDR_SIZE) 
        { 
            /* Need to do arp request */
            arpreq.hwtype = htons(1);
            arpreq.protocol = htons(IP);
            arpreq.hwlen = ETHER_ADDR_SIZE;
            arpreq.protolen = 4;
            arpreq.opcode = htons(ARP_REQUEST);
            memcpy(arpreq.shwaddr, arptable[ARP_CLIENT].node, ETHER_ADDR_SIZE); 
            memcpy(arpreq.sipaddr, &arptable[ARP_CLIENT].ipaddr, sizeof(in_addr));
            memset(arpreq.thwaddr, 0, ETHER_ADDR_SIZE);
            memcpy(arpreq.tipaddr, &destip, sizeof(in_addr));
            for (retry = 1; retry <= MAX_ARP_RETRIES; retry++) 
            {
                eth_transmit(portNum, (char*)broadcast, ARP, sizeof(arpreq),
                             (char*)&arpreq);
                if (await_reply(AWAIT_ARP, arpentry,
                                arpreq.tipaddr, TIMEOUT)) goto xmit;
                if (await_reply(AWAIT_ARP, arpentry,
                                arpreq.tipaddr, 0)) goto xmit;
            }
            return(false);
        }
        xmit:
        eth_transmit(portNum, (char*)arptable[arpentry].node, IP, len, (char*)buf);
    }
    return(true);
}

//extern unsigned char ascii_to_bin(unsigned char c);


/**************************************************************************
 *DOWNLOADKERNEL - In packet reception, this function is called to analyze
 *                 it and store packet's relevant data to gtMemory.
 *                 In case we reached end of file we jump to the entry
 *                 address of the file just downloaded
 *
 *Inputs :
 *data - pointer to the packets info
 *block - block sequence number in the TFTP protocol
 *len - length of packet
 *eof - flag regarding end of file (if eof = 1, then we reached end of file)
 *
 *Outputs - returns -1 always (didn't reach end of file yet)
**************************************************************************/
int downloadkernel(unsigned char *data, int block, int len, int eof)
{
    register int address;
    int i;
    static int last_chars=0;
    char * pCopy_to_add;
    char buffer[2048];
    
    /* Download Kernel in Binary Format*/
    if (block == 1) 
    {
        reached_block = 1;
    }
    if ((len != 0) && (reached_block == block)) 
    {
        /*** copy to permanent place ****/
        // yotam //while(gtDmaIsChannelActive(dmaEng)!=DMA_CHANNEL_IDLE);
        /* yotam
        gtDmaTransfer(dmaEng, (unsigned int)data, 
                      (unsigned int)copy_to_addr, len, 
                      dmaCommand, NULL);
                      */
      //      armond_printf ("copy_to_addr = %08x\n",copy_to_addr);
      memcpy (copy_to_addr, data, len); /* Rabeeh */
      /*      {
	unsigned int j;
	for (j = 0 ; j < len ; j +=4) {
//	  armond_printf ("%08x %08x : %08x %08x\n",j+copy_to_addr, * (unsigned int *)(copy_to_addr + j), data + j, * (unsigned int *)(data + j));
	}
	}*/
        copy_to_addr = (char *) ((unsigned long) copy_to_addr + len);
        reached_block ++;
    } 
    else 
    {
//        armond_printf ("Received Error Block (length %d, Block %d Actual Block %d)\n",len,block,reached_block);
    }
    fileSize+= len;
    if (eof) 
    {
        cleanup_net();
    }
    return(-1) ; /* there is more data */

}


/**************************************************************************
 *TFTP - Download kernel image using TFTP protocol
 *
 *Inputs :
 *name - pointer to the desired full path of the file to be downloaded
 *fnc - function to be called after getting TFTP blocks
 *
 *Outputs -
 **************************************************************************/
int tftp(const char *name, int (*fnc)(unsigned char *, int, int, int))
{
    int             retry = 0;
    static unsigned short iport = 2000;
    unsigned short  oport;
    unsigned short  temp , len, block = 0, prevblock = 0;
    int       bcounter = 0;
    struct tftp_t  *tr;
    struct tftp_t   tp;
    int       rc;
    int       packetsize = TFTP_DEFAULTSIZE_PACKET;

    /* Clear out the Rx queue first.  It contains nothing of interest,
     * except possibly ARP requests from the DHCP/TFTP server.  We use
     * polling throughout Etherboot, so some time may have passed since we
     * last polled the receive queue, which may now be filled with
     * broadcast packets.  This will cause the reply to the packets we are
     * about to send to be lost immediately.  Not very clever.  */
    reached_block = 1;
    tp.opcode = htons(TFTP_RRQ);
    len = (gtSprintf((char *)tp.u.rrq, "%s octet blksize %d ",
                     "img1mb", TFTP_MAX_PACKET));
    for (temp = 0 ; temp < len ; temp ++) 
    {
        if (((char *)tp.u.rrq)[temp] == ' ') ((char *)tp.u.rrq)[temp] = 0;
    }

    len = len + ((unsigned int) &tp.u.rrq - (unsigned int)&tp);
    if (!udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr, ++iport,
                      TFTP_PORT, len, &tp))
        return(0);
    for (;;) 
    {
        if (!await_reply(AWAIT_TFTP, iport, NULL, TIMEOUT)) 
        {
            if (!block && retry++ < MAX_TFTP_RETRIES) 
            {   
                /* maybe initial request was lost */
                if (!udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
                                  ++iport, TFTP_PORT, len, &tp))
                    return(0);
                continue;
            }
            break;    /* timeout */
        }
        tr = (struct tftp_t *)&nic.packet[ETHER_HDR_SIZE];
        if (tr->opcode == ntohs(TFTP_ERROR)) 
        {
//            armond_printf("TFTP error %d (%s)\n",
//                       ntohs(tr->u.err.errcode),
//                       tr->u.err.errmsg);
            break;
        }

        if (tr->opcode == ntohs(TFTP_OACK)) 
        {
            char *p = tr->u.oack.data, *e;

            if (prevblock)      /* shouldn't happen */
                continue; /* ignore it */
            len = ntohs(tr->udp.len) - sizeof(struct udphdr) - 2;
            if (len > TFTP_MAX_PACKET)
                goto noak;
            e = p + len;
            while (*p != '\000' && p < e) 
            {
                if (!strcasecmp("blksize", p)) 
                {
                    p += 8;
                    if ((packetsize = getdec(&p)) < TFTP_DEFAULTSIZE_PACKET)
                        goto noak;
                    while (p < e && *p) p++;
                    if (p < e)
                        p++;
                } 
                else 
                {
                    noak:
                    tp.opcode = htons(TFTP_ERROR);
                    tp.u.err.errcode = 8;
                    len += (gtSprintf((char *)tp.u.err.errmsg,
                                      "RFC1782 error"));//- ((char *)&tp)) + 1;*/
                    udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr,
                                 iport, ntohs(tr->udp.src),
                                 len, &tp);
                    return(0);
                }
            }
            if (p > e)
                goto noak;
            block = tp.u.ack.block = 0; /* this ensures, that */
            /* the packet does not get */
            /* processed as data! */
        } 
        else if (tr->opcode == ntohs(TFTP_DATA)) 
        {
            len = ntohs(tr->udp.len) - sizeof(struct udphdr) - 4;
            if (len > packetsize)   /* shouldn't happen */
                continue; /* ignore it */
            block = ntohs(tp.u.ack.block = tr->u.data.block);
        } 
        else /* neither TFTP_OACK nor TFTP_DATA */
            break;

        if ((block || bcounter) && (block != prevblock+1)) 
        {
            /* Block order should be continuous */
            tp.u.ack.block = htons(block = prevblock);
        }
        tp.opcode = htons(TFTP_ACK);
        oport = ntohs(tr->udp.src);
        udp_transmit(arptable[ARP_SERVER].ipaddr.s_addr, iport,
                     oport, TFTP_MIN_PACKET, &tp);    /* ack */

        if ((unsigned short)(block-prevblock) != 1) 
        {
            /* Retransmission or OACK, don't process via callback
             * and don't change the value of prevblock.  */
            continue;
        }
        prevblock = block;
        retry = 0;    /* It's the right place to zero the timer? */
        if ((rc = fnc((unsigned char *)tr->u.data.download,
                      ++bcounter, len, len < packetsize)) >= 0) 
        {

            return(rc);
        }
        if (len < packetsize) 
        {    
            /* End of data */
            return(1);
        }
    }
    return(0);
}


/**************************************************************************
 *AWAIT_REPLY - Wait until we get a response for our request
 *
 *Inputs :
 *type - the request type (AWAIT_ARP , AWAIT_TFTP)
 *ival - the IP value for the waitened reply
 *ptr - a pointer 
 *timeout - time out
 *
 *Outpus : Success/Failure flag
 **************************************************************************/
int await_reply(int type, int ival, void *ptr, int timeout)
{
    unsigned long time;
    struct    iphdr *ip;
    struct    udphdr *udp;
    struct    arprequest *arpreply;
    struct    bootp_t *bootpreply;
    unsigned short ptype;

    unsigned int protohdrlen = ETHER_HDR_SIZE + sizeof(struct iphdr) +
                               sizeof(struct udphdr);
    /* TimeOut mechanism is not implemented*/
    time = timeout + currticks();
    /* The timeout check is done below.  The timeout is only checked if
     * there is no packet in the Rx queue.  This assumes that eth_poll()
     * needs a negligible amount of time.  */
    for (;;) 
    {
        if (eth_poll(portNum)) 
        {   
            /* We have something! */
            /* Check for ARP - No IP hdr */
//	  	  armond_printf ("Got packet (length %d) - waiting for type %d\n",nic.packetlen,type);
	  if (nic.packetlen >= ETHER_HDR_SIZE) 
            {
                ptype = ((unsigned short) nic.packet[12]) << 8
                        | ((unsigned short) nic.packet[13]);
            } 
            else continue; /* what else could we do with it? */
	    //	    armond_printf ("Checking if ARP ?!?\n");
            if ((nic.packetlen >= ETHER_HDR_SIZE +
                 sizeof(struct arprequest)) &&
                (ptype == ARP) ) 
            {
                unsigned long tmp;
                arpreply = (struct arprequest *)
                           &nic.packet[ETHER_HDR_SIZE];
		//		armond_printf ("AWAIT_ARP = %d, ip %x ptr %02x%02x%02x%02x\n", (type == AWAIT_ARP) ? 1 : 0, arpreply->sipaddr,((char *) ptr)[0], ((char *)ptr)[1], ((char *)ptr)[2], ((char *)ptr)[3]);
                if ((arpreply->opcode == ntohs(ARP_REPLY)) &&
                    !memcmp(arpreply->sipaddr, ptr, sizeof(in_addr)) &&
                    (type == AWAIT_ARP)) 
                {
                    memcpy(arptable[ival].node, arpreply->shwaddr, ETHER_ADDR_SIZE);
                    return(true);
                }

                memcpy(&tmp, arpreply->tipaddr, sizeof(in_addr));

                if ((arpreply->opcode == ntohs(ARP_REQUEST)) &&
                   (tmp == arptable[ARP_CLIENT].ipaddr.s_addr)) 
                {
                    arpreply->opcode = htons(ARP_REPLY);
                    memcpy(arpreply->tipaddr, arpreply->sipaddr, sizeof(in_addr));
                    memcpy(arpreply->thwaddr, arpreply->shwaddr, ETHER_ADDR_SIZE);
                    memcpy(arpreply->sipaddr, &arptable[ARP_CLIENT].ipaddr, sizeof(in_addr));
                    memcpy(arpreply->shwaddr, arptable[ARP_CLIENT].node, ETHER_ADDR_SIZE);
                    eth_transmit(portNum, (char *)arpreply->thwaddr, ARP,
                                 sizeof(struct  arprequest), (char*)arpreply);
/*void eth_transmit(int portNum, const char *destAddr,  unsigned int packetType,         
                  unsigned int dataSize, const char *dataPtr);   */       
                }
                continue;
            }

            if (type == AWAIT_QDRAIN) 
            {
                continue;
            }
            /* Anything else has IP header */
            if ((nic.packetlen < protohdrlen) ||
                (ptype != IP) ) continue;
	    
            ip = (struct iphdr *)&nic.packet[ETHER_HDR_SIZE];
            if ((ip->verhdrlen != 0x45) ||
                ipchksum((unsigned short *)ip, sizeof(struct iphdr)) ||
                (ip->protocol != IP_UDP)) continue;
           udp = (struct udphdr *)&nic.packet[ETHER_HDR_SIZE +
                                               sizeof(struct iphdr)];

            /* BOOTP ? */
            bootpreply = (struct bootp_t *)&nic.packet[ETHER_HDR_SIZE];
            /* TFTP ? */
            if ((type == AWAIT_TFTP) && (ntohs(udp->dest) == ival)) 
            {
                return(true);
            }

        } 
        else 
        {
            /* Check for abort key only if the Rx queue is empty -
             * as long as we have something to process, don't
             * assume that something failed.  It is unlikely that
             * we have no processing time left between packets.  */
            /* Do the timeout after at least a full queue walk.  */
#if 0
            if ((timeout == 0) || (currticks() > time)) 
            {
                break;
            }
#else
	    break; // Wait is in the eth_poll()
#endif
        }
    }

    return(false);
}


/**************************************************************************
 *IPCHKSUM - Calculate Checksum IP Header
 *
 *Inputs - 
 *ip - a pointer to the IP header
 *len - length of the IP header
 *
 *Outpus -
 *The checksum of the IP header
 **************************************************************************/
unsigned short ipchksum(register unsigned short *ip, register int len)
{
    unsigned long sum = 0;
    len >>= 1;
    while (len--) 
    {
        sum += *(ip++);
        if (sum > 0xFFFF)
            sum -= 0xFFFF;
    }
    return((~sum) & 0x0000FFFF);
}

/**************************************************************************
 *CLEANUP_NET - shut down networking
 *
 *Inputs -
 *Outputs -
 **************************************************************************/
void cleanup_net(void)
{
    eth_disable(portNum);
}

/**************************************************************************
eth_poll: Wait for a frame, by polling the current waiting rx desc (desc no. 
          CurrRxDesc). When a frame arrives it initializes again the rx
          descriptor, increments CurrRxDesc and sets Enable Rx bit again.
***************************************************************************/
bool eth_poll(int portNum)
{
    long wait = 0;
    /* return true if there's an ethernet packet ready to read */
    /* nic->packet should contain data on return */
    /* nic->packetlen should contain length of data */  

    /* wait for a frame */
//    armond_printf ("Check if we have a packet arrived... %d - %08x",CurrRxDesc, &eth0_rx_desc[CurrRxDesc].cmd_sts);
  while (eth0_rx_desc[CurrRxDesc].cmd_sts & DESC_OWNED_BY_GT) 
    {
	wait++;
	if(wait > 0x10000) {
	    longjmp(out_of_here, wait);
	}
    }
  if(wait > gwait) {
      gwait = wait;
  }
//    armond_printf ("Yep\n");
    /* set nic */
  nic.packetlen = eth0_rx_desc[CurrRxDesc].bytecnt;
  nic.packet = (char *) eth0_rx_buffer[CurrRxDesc];
  /*  {
    unsigned int i;
//    armond_printf ("Packet at %08x (length %d) %08x %08x %08x %08x %08x\n",nic.packet,nic.packetlen,&eth0_rx_desc[CurrRxDesc].bytecnt,
		   * (unsigned int *) ((unsigned int)&eth0_rx_desc[CurrRxDesc].cmd_sts + 0),
		   * (unsigned int *) ((unsigned int)&eth0_rx_desc[CurrRxDesc].cmd_sts + 4),
		   * (unsigned int *) ((unsigned int)&eth0_rx_desc[CurrRxDesc].cmd_sts + 8),
		   * (unsigned int *) ((unsigned int)&eth0_rx_desc[CurrRxDesc].cmd_sts + 12));
    for (i = 0 ; i < 64 ; i++) {
//      armond_printf ("%x : %x\n",i, nic.packet[i]);
    }
  }
  */

    //    armond_printf ("Packet is at %08x by descriptor\n",eth0_rx_desc[CurrRxDesc].buf_ptr);
    /* sets the receiving desc to initial state */
    eth0_rx_desc[CurrRxDesc].bufsize = 1536;
    eth0_rx_desc[CurrRxDesc].cmd_sts = DESC_OWNED_BY_GT | DESC_ENABLE_INTERRUPT; 

    /* increment rx descs counter */
    CurrRxDesc ++;
    if (CurrRxDesc == NR) CurrRxDesc = 0;
    
    /* ENABLE Rx - in case it was aborted */
    EAGLE_MAC_WRITE (ETH_SDMA_COMMAND_REG(0) + (portNum << 10), ETH_ENABLE_RX); 
    //nic.packetlen = 0x600; /* Seems a bug in the HW ??!! */
    return true;
}

/**************************************************************************
eth_transmit: receives destAddr and data to transmit. Builds ethernet header 
              to frame adds data and transmits frame.
***************************************************************************/
void eth_transmit(int portNum, char *destAddr,  unsigned int packetType,         
                  unsigned int dataSize, char *dataPtr)          
{
    unsigned int bufSize;
    /* wait for last transmission to end */
    while (eth0_tx_desc[0].cmd_sts & DESC_OWNED_BY_GT);
    /* Build Ethernet Header - 14 bytes: DA,SA,Type (of upper layer)*/
    /* Destination MAC is broadcast */
    eth0_tx_buffer[0] = destAddr[0];
    eth0_tx_buffer[1] = destAddr[1];
    eth0_tx_buffer[2] = destAddr[2];
    eth0_tx_buffer[3] = destAddr[3];
    eth0_tx_buffer[4] = destAddr[4];
    eth0_tx_buffer[5] = destAddr[5];
    /* Source MAC is as defined in eeprom parameters */
    eth0_tx_buffer[6] =  arptable[ARP_CLIENT].node[0];
    eth0_tx_buffer[7] =  arptable[ARP_CLIENT].node[1];
    eth0_tx_buffer[8] =  arptable[ARP_CLIENT].node[2];
    eth0_tx_buffer[9] =  arptable[ARP_CLIENT].node[3];
    eth0_tx_buffer[10] =  arptable[ARP_CLIENT].node[4];
    eth0_tx_buffer[11] =  arptable[ARP_CLIENT].node[5];  
    /* Type is IP or ARP packet */
    eth0_tx_buffer[12] = (packetType & 0xff00) >> 8;
    eth0_tx_buffer[13] = packetType & 0xff;

    /* copy packet data into tx buffer, after eth header */
    memcpy ((char*)((unsigned long)eth0_tx_buffer +14), dataPtr, dataSize);

    /* set size to min eth packet of 60 (+4B for CRC)*/
    if (dataSize < 60-14) bufSize = 60;
    else bufSize = dataSize + 14;
    /* initialize 1 tx desc */
    gtDescInitTxChain(1, (unsigned int)eth0_tx_desc,
                      (unsigned int) eth0_tx_buffer, bufSize, DESC_OWNED_BY_GT | 
                      DESC_GENERATE_ERROR_CODE_TX | DESC_ADD_PADDING 
                      , NO_CYCLIC_CHAIN, 1); 
    gtEtherStartTransmit(portNum, (unsigned int) eth0_tx_desc, ETH_TX_PRIO0);
}

/**************************************************************************
eth_disable: Turn off ethernet interface
***************************************************************************/
void eth_disable(int portNum)
{
    gtEtherAbortTx(portNum);
    gtEtherAbortRx(portNum); 
    EAGLE_MAC_WRITE (ETH_SDMA_CONFIG_REG(0) + (portNum << 10), originalSDCR);
}

/**************************************************************************
eth_init: initializes eth data structures and registers for future receptions
           and transmissions.  
***************************************************************************/
void os_TaskDelay(int x) {
#if 1
  int i;
  //  armond_printf("TaskDelay... ");
  for (i = 0 ; i < x * 10000 ; i++) {}
  //  armond_printf("Done\n");
#else
  return;
#endif
}
int WritePhy(unsigned int offset, unsigned int data)
{
  unsigned int rdata;
  EAGLE_MAC_WRITE(0x10, (offset <<21) | (0 <<16) | (data &0xffff));
  EAGLE_MAC_READ(0x10, &rdata);
  while( (rdata >>28) & 1)
    {
      os_TaskDelay(2);
      EAGLE_MAC_READ(0x10, &rdata);
    }
  return 0;
}
int ReadPhy(unsigned int offset)
{
  unsigned int rdata;
  EAGLE_MAC_WRITE(0x10, (offset <<21) | (1<<26)| (0 <<16) );
  EAGLE_MAC_READ(0x10, &rdata);
  while( (rdata >>28) & 1)
    {
      os_TaskDelay(2);
      EAGLE_MAC_READ(0x10, &rdata);
    }
  return( rdata & 0xffff);
}

#define W81_WRITE_BYTE(x,y) * (unsigned char *) x = y;
#define W81_WRITE_HWORD(x,y) * (unsigned short *) x = y;
#define W81_WRITE_WORD(x,y) * (unsigned int *) x = y;

extern int flash_boot;
void eth_init(int portNum)
{
    unsigned int temp;
    HASH_TABLE_ENTRY_DATA hashData;
    unsigned int phydata;

//    armond_printf ("Restarting FEU macro\n");
    // Enable clocks going to all modules
    W81_WRITE_HWORD(0x8000200c, 0xffff);
//    os_TaskDelay(100);
    
    W81_WRITE_WORD(0x8000604c, 0xe00156);
//    os_TaskDelay(100);
    W81_WRITE_WORD(0x8000604c, 0xe00357);
    
//   W81_WRITE_BYTE(0x8000c000, 1); // preset
//    W81_WRITE_BYTE(0x8000c004, 0);
//    os_TaskDelay(100);
    W81_WRITE_WORD(0x80006050,0x7fe87a00);
    
//    os_TaskDelay(100);
    W81_WRITE_WORD(0x80006050,0x7fe83080); //FEU analog reset
//    os_TaskDelay(100);
    os_TaskDelay(10);
   
    W81_WRITE_WORD(0x80006050,0x7fe8b080); // FEU reset
//    os_TaskDelay(100);
    os_TaskDelay(10);
    /* initialize registers */  
    // WritePhy(0,0x100);   /* power up in 10Mbit mode FDX */
    WritePhy(0,0x00);   /* power up in 10Mbit mode HDX */
    phydata = ReadPhy(0);



#ifndef ETHER_SWITCH
    EAGLE_MAC_READ(ETH_SDMA_CONFIG_REG(0) + (portNum << 10), &originalSDCR);
    EAGLE_MAC_WRITE (ETH_SDMA_CONFIG_REG(0) + (portNum << 10), 
                  ETH_BURST_SIZE_32B | 0x3c | BIT6 | BIT7); //rc = 0xf
    EAGLE_MAC_READ(ETH_PORT_CONFIG_REG(0) + (portNum << 10), &temp);
    temp = (temp) | ETH_HASH_FUNCTION_0 | ETH_HASH_SIZE_16KB | ETH_PORT_ENABLE | BIT0;
    EAGLE_MAC_WRITE (ETH_PORT_CONFIG_REG(0) + (portNum << 10), temp);
    EAGLE_MAC_WRITE (ETH_PORT_CONFIG_EXTEND_REG(0) + (portNum << 10), 
		     ETH_OVERRIDE_PRIORITY_FOR_RX_PACKETS | 
		     ETH_MAX_FRAME_LENGTH_1536B | ETH_RMII_ENABLE |
		     BIT19 /* Work in 10MBps only !!! */ | 
		     BIT20);
#else
    //W81_WRITE_WORD(0x8000200c, 0xffffffff);

    //W81_WRITE_WORD(0x80002030, 0x7109);   /* 44Mhz clock */
    //os_TaskDelay(100);
    //W81_WRITE_WORD(0x8000604c, 0xe00357);

    //os_TaskDelay(100);

    //W81_WRITE_WORD(0x80006050,0x7fe87a00);

    //os_TaskDelay(100);
    //W81_WRITE_WORD(0x80006050,0x7fe83080); //FEU analog reset
    //os_TaskDelay(100);


    //W81_WRITE_WORD(0x80006050,0x2072b080); //FEU reset and incr strength of sdram clk
    //W81_WRITE_WORD(0x80002044, ((0xffffffe7) | 0x8));   /* Enable FEU TBG 50 Mhz output clock */ 
     //W81_WRITE_WORD( 0x8000d004, 0x7fff);  /* enables gpio 15 output mode */
     //W81_WRITE_WORD(0x8000d00c, 0x7fff);   /* set gpio 15 low , reset peripherals */

     //os_TaskDelay(100);

     //W81_WRITE_WORD(0x8000d00c, 0xffff);   /* set gpio 15 high, enable peripherals */
	 //os_TaskDelay (100);
    
     etherswitch_init ();
     W81_WRITE_WORD (0x80008000, 0x00);
     W81_WRITE_WORD (0x80008440, 0x32FC);
     W81_WRITE_WORD (0x80008400, 0x81);
     W81_WRITE_WORD (0x80008408, 0x00085600);
#endif

    CurrRxDesc=0;
    eth0_tx_desc[0].cmd_sts = 0; /*for first transmit*/

    /* initialize rx desc chain*/
    //    armond_printf ("Initializing descriptors at %08x (buffer at %08x)\n",eth0_rx_desc,eth0_rx_buffer[0]);
    gtDescInitRxChain(NR, (unsigned int)eth0_rx_desc,
                      //(((unsigned int)eth0_rx_buffer_alloc[0] & 0xfffffff0) + 0x10),
                      (unsigned int)eth0_rx_buffer[0],
                      1536, DESC_OWNED_BY_GT, CREATE_CYCLIC_CHAIN);
  
   /* initialize rx buffers array */
    for (temp = 0; temp < NR; temp++)  
    {
        eth0_rx_buffer[temp] = (char*)eth0_rx_desc[temp].buf_ptr;
    }   
//    armond_printf ("Initializing hash table (at %08x)...",hashTableBase);
    hashData.prio = 0;
    hashData.rdBit = 1;
    hashData.macLow = macLow;
    hashData.macHigh = macHigh;
    gtEtherInitHashTable(portNum, ETH_HASH_MODE_0, _512_ADDRESSES_TABLE, 
                         hashTableBase, 1, &hashData); 
//    armond_printf (" done\n");
    gtEtherEnableReception(portNum, (unsigned int) eth0_rx_desc, ETH_RX_PRIO0); 
//    armond_printf ("Task wait 7\n");
    if(flash_boot) {
	os_TaskDelay (100);
    } else {
	os_TaskDelay (1000);
    }
}







