#include "MvCore.h"
#include "mem_init.h"
                     

/******************************************************************************
 *
 * Name: memoryInitDefualt
 *
 * Description:
 *   Initialize the memory controller with the SDRAM defualt values. 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6208 Reference design board or 88E6218 Development Board.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
void memoryInitDefualt( void )
{
	/*here we have to program pll clock*/
	
	/* Control and Status Register*/ 
	GT_REG_WRITE(CSR_ADDR,((GTREGREAD(CSR_ADDR) & 0x0000ffff) | CSR));
	/* Device 0 configuration Registers */
	GT_REG_WRITE(CFGR0_ADDR,CFGR0);
	GT_REG_WRITE(TIMER00_ADDR,TIMR00);
	GT_REG_WRITE(TIMER01_ADDR,TIMR01);
	deviceInit();
}

/******************************************************************************
 *
 * Name: memoryInit
 *
 * Description:
 *   Initialize the memory controller with the SDRAM DETECTED values. 	 	
 *
 * Conditions For Use:
 *   Use this function with  88E6218 Development Board.
 *
 * Arguments:
 *   SDRAM_DATA * - pointer to struct that will hold the deteced data.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
void memoryInit(SDRAM_DATA * pSdramData)
{
	UINT32 sdramWidth;
	UINT32 moduleBankDensity;
	
	/*here we have to program pll clock*/
	
	/* Control and Status Register*/ 
	GT_REG_WRITE(CSR_ADDR,((GTREGREAD(CSR_ADDR) & 0x0000ffff) | CSR));
	/* Device 0 configuration Registers */
	switch(pSdramData->sdramWidth)
	{
		case _8BIT_SDRAM_WIDTH: sdramWidth = 1;
								break;
		case _16BIT_SDRAM_WIDTH: sdramWidth = 2;
								 break;
		default:				 sdramWidth = 2;
	}
	switch(pSdramData->moduleBankDensity)
	{
		case _16MB_MODULE_BANK_DENSITY:		moduleBankDensity = 1;
											break;
		case _32MB_MODULE_BANK_DENSITY:		
		case _64MB_MODULE_BANK_DENSITY:		moduleBankDensity = 2;
											break;
		default:							moduleBankDensity = 1;
	}
										
		  
	
		
	GT_REG_WRITE(CFGR0_ADDR,(BAR0 | BARMK0 | KRO0 | PEN0 | PACK_EN0 | (moduleBankDensity << 5)
							 | MEMORY_TYPE0 | (sdramWidth<<1) | CS0En | WS0));
	GT_REG_WRITE(TIMER00_ADDR,TIMR00);
	GT_REG_WRITE(TIMER01_ADDR,TIMR01);
	deviceInit();
}


/******************************************************************************
 *
 * Name: sdramDetect
 *
 * Description:
 *   Detects the SDRAM DIMM module parameters 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6218 Development Board.
 *
 * Arguments:
 *   SDRAM_DATA * - pointer to struct that will hold the deteced data.
 *
 * Return Value:
 *   UINT32 -   true upon detection.
 		   		false upon miss-detection
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
/* temporary define until function is implemnted */
#define I2C_READ(x) x;

 extern UINT16 sdramDetect(SDRAM_DATA *pSdramData)
 {
 	UINT32 r0;
 	
 	r0 = I2C_READ(BOARD_I2C__BOARD_ID_ADDR);
 	if (r0 == _88E8218_DB) /* This board has module and there for Memory I2C read is done */
 	{
 		pSdramData->sdramWidth = I2C_READ(SDRAM_MODULE_I2C__SDRAM_WIDTH_ADDR);
 		pSdramData->moduleBankDensity = I2C_READ(SDRAM_MODULE_I2C__MODULE_BANK_DENSITY_ADDR);
 		if ((pSdramData->moduleBankDensity != _16MB_MODULE_BANK_DENSITY) ||
 			(pSdramData->moduleBankDensity != _32MB_MODULE_BANK_DENSITY) ||
 			(pSdramData->moduleBankDensity != _64MB_MODULE_BANK_DENSITY))
 			return false;
 		return true;
 	}
 	return false;
 }
 		
 	
 	


/******************************************************************************
 *
 * Name: deviceInit
 *
 * Description:
 *   Initialize the memory controller devices with the  defualt values. 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6208 Reference design board or 88E6218 Development Board.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
 static void deviceInit( void )
 {
	/* Device 1 configuration Registers */
 	GT_REG_WRITE(CFGR1_ADDR,CFGR1);
	GT_REG_WRITE(TIMER10_ADDR,TIMR10);
	/* Device 2 configuration Registers */
	GT_REG_WRITE(CFGR2_ADDR,CFGR2);
	GT_REG_WRITE(TIMER20_ADDR,TIMR20);
	/* Device 3 configuration Registers */
	GT_REG_WRITE(CFGR3_ADDR,CFGR3);
	GT_REG_WRITE(TIMER30_ADDR,TIMR30);
 }
	
