/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/xmodem.c,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    This file contains routines that allow the Eagle boot rom to
 *    transfer code from the UART via the XMODEM protocol to a
 *    download address. Once all packets for the image have arrived
 *    correctly, control is passed to the start of the download address.
 *
 * Public Procedures:
 *    uart_boot               Entry point for Xmodem download.
 *
 * Private Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================
#include "88E6218.h"
#include "board.h"
#include "xmodem.h"
#include "uart.h"


//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================
/******************************************************************************
 *
 * Name: uartLoader
 *
 * Description:
 *   This function reads data from the UART in the XMODEM protocol and
 *   copies the packets into SDRAM starting address 0x20000.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   LOAD_STATUS.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern LOAD_STATUS uartLoader(void )
{
    UINT8 		*u8xferloc = (UINT8 *)LOAD_XFER_ADDR;
    UINT8	 	u8c;
    UINT8 		u8blkn, u8cblkn;
    short 		i;
    short		s16csum;
    UINT8 		*u8tmp_xferloc;
    int 		s32currtest;

//    setup_uart();
 /* eran - not sure why is needed */
    for(;;) {
		if((UINT32)u8xferloc != LOAD_XFER_ADDR) {
		    u8xferloc -= 0x80; /* Decrement the address if in xfer
					  (this is error recovery). */ 
	    }
	restart:
//		put_char(NAK);
	STATE0:
//		if((u8c = get_char()) == FAIL) {
//		    goto restart;
//		}
		if(u8c == SOH) {
		    goto STATE1;
		}
		if(((UINT32)u8xferloc != LOAD_XFER_ADDR) && (u8c == EOT)) { /* load finished*/
//	            put_char(ACK);
		    return L_SUCCESS;
		}
	STATE1:
		s32currtest = 0;
//		if((u8blkn = get_char()) == FAIL) {
//		    goto restart;
//		}
//		if((u8cblkn = get_char()) == FAIL) {
//		    goto restart;
//		}
		if(u8blkn != ((UINT8)~u8cblkn)) { // block number mismatch
		    s32currtest = BLK_MISMATCH;
		}
		s16csum = 0;
		u8tmp_xferloc = u8xferloc;         
		for(i=0;i < 128;i++) {
		    UINT8 u8ch;
//		    if((u8ch = get_char()) == FAIL) {
		    	
//		    }
		    s16csum += u8ch;
		    *u8tmp_xferloc++ = u8ch;
		}
	    s16csum &= 0xff;
//	    if((u8c = get_char()) == FAIL) {
//		    goto restart;  /* pointer has moved at this point. decrement
//				  and NAK */ 
//	    }
	    if(u8c != s16csum) {
	        s32currtest = CSUM_ERR;
	    }
	    if(s32currtest != 0) {
		    goto restart; /* NAK and backup the pointer */
	    } 
	    else {
	       u8xferloc = u8tmp_xferloc;
//	       put_char(ACK);
	    }
		goto STATE0;
	}
}
