/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/xmodem.h,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    This file contains the function prototypes and definitions for the
 *    XMODEM protocol code.
 *
 * Public Procedures:
 *    uart_boot               Perform XMODEM download of boot code.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#ifndef _XMODEM_H_
#define _XMODEM_H_

typedef enum {GENERAL_FAILURE, L_SUCCESS, INVALID_PROTOCOL} LOAD_STATUS ;


//=============================================================================
//                                DEFINITIONS
//=============================================================================

#define SOH            1 // Start Of Header
#define EOT            4 // End Of Transmission
#define ACK            6 // Acknowledge (positive)
#define DLE           16 // Data Link Escape
#define NAK           21 // Negative Acknowledge
#define SYN           22 // Synchronous idle
#define CAN           24 // Cancel


#define BLK_MISMATCH  0x21
#define CSUM_ERR      0x22


//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================
/******************************************************************************
 *
 * Name: uartLoader
 *
 * Description:
 *   This function reads data from the UART in the XMODEM protocol and
 *   copies the packets into SDRAM starting address 0x20000.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status: 1.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern LOAD_STATUS uartLoader( void );

#define FAIL	       0
#define OK	       1

#endif /* _XMODEM_H_ */


