#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "flash_types.h"

/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

typedef struct FLASH_TYPE_t
{
    unsigned index;
    unsigned manu;
    unsigned addr1;
    unsigned addr2;
    unsigned val1;
}FLASH_TYPE_t;

FLASH_TYPE_t flash[] =
{
    {0, AMD, 0xAAA, 0x555, 0x0030},
    {1, HYUNDAI, 0XAAA, 0x555, 0x0030},
    {2, MICRONIX, 0xAAA, 0x555, 0x0030},
    {3, SST, 0xAAAA, 0x5554, 0x0050}
};

#define MAX_VENDOR 4

unsigned flashindex = 0;
unsigned flashsize = 0;
unsigned flashbase = 0xffff0000;
unsigned flashNsect;
unsigned manu;
unsigned part;

extern int errors;

extern int total_size;
int split_sector = -1;
int split_sector_data[]={
    0,
    0x8000,
    0xa000,
    0xc000,
};
int split[2][4]={
    { 16, 8, 8, 32}, 
    { 32, 8, 8, 16}
};
void 
makesplit(unsigned ss)
{
    int top = 0;
    int start = 0;
    int i;
    split_sector = ss;
    if (ss)
    {
        top = 1;
    }
    for (i=0;i < 4;i++)
    {
        split_sector_data[i] = start;
        start += split[top][i] * 1024;
    }
}

void 
ConfigMDU(int boardtype)
{
    _REG(MDU_TIMR00) = 0x0442A20C;   /* FLASH timing */
    _REG(MDU_CSR) = 0x4190040;  /* for 88 Mhz clock cpu speed */
    _REG(MDU_TIMR01) = 0x00008080;
    _REG(MDU_CFGR0)  = 0xFFFF4285;   /* 0xffc00000 FLASH configuration pack_en = 1*/
    switch (boardtype)
    {
    case EVAL_BOARD:
        _REG(MDU_TIMR20) = 0x00000022;
        _REG(MDU_TIMR21) = 0x03232222;
        _REG(MDU_CFGR2)  = 0x40FE50B5; /* 0xd0000000 */
        break;
    case AP22N2_BOARD: // Uses CSI ram... seems like... AP21 Micron...
    case AP21_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x03232222;
        _REG(MDU_CFGR1)  = 0x40FF5095; /* 0xd0000000 */
        break;
    case AP22C_BOARD:
        _REG(MDU_CSR) = 0x157e0000;  /* for 88 Mhz clock cpu speed OVERRIDE!*/
        /* Fall in to AP22 setup */
    case AP22_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1)  = 0x40FC70B5; /* 0xd0000000 */
        break;
    case AP22N_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1)  = 0x40FE70B7; /* 0xd0000000 */
        break;
    case AP21M_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;  /* for 88 Mhz clock cpu speed */
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x0A3A2632;
        _REG(MDU_CFGR1)  = 0x40FE70B7; /* 0xd0000000 */
    case CONDOR_TEST_BOARD:
        _REG(MDU_CSR) = 0x04191001;  /* for 44 Mhz clock cpu speed */
        _REG(MDU_TIMR10) = 0x00000033; /* Burst of 8 bytes */
        _REG(MDU_TIMR11) = 0x0a3a2632;
        _REG(MDU_CFGR1)  = 0x40Fc70b5; /* 0xd0000000 */
        break;
    case CONDOR_BOARD:
        _REG(MDU_CSR) = 0x04191001;  /* for 44 Mhz clock cpu speed */
        _REG(MDU_TIMR10) = 0x00000033; /* Burst of 8 bytes */
        _REG(MDU_TIMR11) = 0x0a3a2632;
        _REG(MDU_CFGR1)  = 0x40FF7095; /* 0xd0000000 */
        break;
    }
    //disable devices 3 4 5
    _REG(MDU_CFGR3) = 0xC0FF0008;
    _REG(MDU_CFGR4) = 0x80FF0008;
    _REG(MDU_CFGR5) = 0x40FF0008;

}

void flashPreamble(void)
{
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
    _REG16(flashbase | flash[flashindex].addr1) = 0x0080;
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
}

void flashInterrogate(void)
{
    int i;
    for (i = 0; i<MAX_VENDOR; i++)
    {
        _REG16(flashbase) = 0xf0; /* reset */

        _REG16(flashbase | flash[i].addr1) = 0x00AA;  /* word address 555 */
        _REG16(flashbase | flash[i].addr2) = 0x0055;  /* word address 2AA */
        _REG16(flashbase | flash[i].addr1) = 0x0090;  /* word address 555 */

        manu = _REG16(flashbase);     /* Manu */
        part = _REG16(flashbase + 2); /* part ID. */

        _REG16(flashbase) = 0xf0; /* reset */

        if (manu == flash[i].manu)
        {
            flashindex = i;
            return;
        }
    }
}

void flashEraseChip(int sblk, int nblks)
{
    unsigned addr;
    int i;
    int j;

    for (i=sblk;i < (sblk+nblks);i++)
    {
        if (i == split_sector)
        {
            unsigned start = (i << (16));
            for (j=0;j < sizeof(split_sector_data)/sizeof(int);j++)
            {
                addr = flashbase + start + split_sector_data[j];
                flashPreamble();
                _REG16(addr) = flash[flashindex].val1;
                while (_REG16(addr) != 0xFFFF)
                    ;
            }
        }
        else
        {
            addr = flashbase + (i << (16));
            flashPreamble();
            _REG16(addr) = flash[flashindex].val1;
            while (_REG16(addr) != 0xFFFF)
                ;
        }
    }
}

int flashWriteWord( int offset, unsigned short dat)
{

    unsigned addr = flashbase | offset;
    unsigned short tmp;

    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;  /* word address 555 */
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;  /* word address 2AA */
    _REG16(flashbase | flash[flashindex].addr1) = 0x00A0;  /* word address 555 */

    /* if flash is not erase we don't write (avoid infinit loop) */
    /* we can change a bit from 1 to 0 but not from 0 to 1 */
    tmp = _REG16 (addr);
    if ((~tmp & dat) != 0x00000000)
        return(-1);   /* flash not erased */

    _REG16(addr) = dat;

    /* spin here until operation is done */
    while (_REG16(addr) != dat)
        ;

    return(0);
}

void
program_image(int what)
{
    int i;
    unsigned short val;
    unsigned short *addr;
    unsigned len;
    int sblk;
    int nblks;
    int saddr;

    // get info from the flash
    flashInterrogate();


    // is it a manufacturer we know?
    switch (manu)
    {
    case AMD:
        // Have we heard of the part?
        switch (part)
        {
        case 0x22c4: // 160BT
            flashsize = 2*1024*1024;
            flashNsect = 32;
            makesplit(31);
            break;
        case 0x2249: // 160BB
            flashsize = 2*1024*1024;
            flashNsect = 32;
            makesplit(0);
            break;
        case 0x22da: // LV800BT
            flashsize = 1*1024*1024;
            flashNsect = 16;
            makesplit(15);
            break;
        case 0x225b: // LV800BB
            flashsize = 1*1024*1024;
            flashNsect = 16;
            makesplit(0);
            break;
        default:
            printf("UNKNOWN device %x\n", part);
            exit(1);
        }
        break;
    case HYUNDAI: // Hyundai
        switch (part)
        {
        case 0xD5: // 29F080
            flashsize = 1*1024*1024;
            flashNsect = 16;
            break;
        default:
            printf("UNKNOWN device %x\n", part);
            exit(1);
        }
        break;
    case MICRONIX:
        // Have we heard of the part?
        switch (part)
        {
        case 0x22c4: // 160BT
            flashsize = 2*1024*1024;
            flashNsect = 32;
            makesplit(31);
            break;
        default:
            printf("UNKNOWN device %x\n", part);
            exit(1);
        }
        break;
    case SST: // SST
        switch (part)
        {
        case 0x2781: // SST39VF800A
            flashsize = 1*1024*1024;
            flashNsect = 16;
            makesplit(-1);
            break;
        default:
            return;
        }
        break;
    default:
        printf("UNKNOWN manu %x\n", manu);
        exit(1);
        break;
    }

    flashbase = (0 - flashsize);

    switch (what)
    {
    case ALL:
        sblk = 0;
        nblks = flashNsect;
        break;
    case BOOT:
        sblk = flashNsect - 1;;
        nblks = 1;
        break;
    case NO_BOOT:
        sblk = 0;
        nblks = flashNsect - 1;
        break;
    case NO_BOOT2:
    case NO_BOOT_FACTORY:
        sblk = 0;
        nblks = flashNsect - 2;
        break;
    case BOOT_FACTORY:
        sblk = flashNsect - 2;;
        nblks = 2;
    }
    printf("manu %d part 0x%x flashsize %d split sector @%d\n",
           manu, part, flashsize, split_sector);

    if (total_size > ((64*1024)*nblks))
    {
        printf("Too much data for that flash area (%d vs start %d nsect %d)\n", 
               total_size, sblk, nblks);
        exit(1);
    }

    printf("erasing\n");
    flashEraseChip(sblk, nblks);

    addr = (unsigned short *)0xd0000000;

    printf("Programming\n");
    len = nblks * (64*1024);
    saddr = (sblk * (64*1024));
    for (i=0;i < total_size;i+= 2)
    {
        val = *addr++;

        flashWriteWord(saddr+i, val);

    }

    addr = (unsigned short *)0xd0000000;
    for (i=0;i < total_size;i+= 2, addr++)
    {
        unsigned short val = *(unsigned short *)(saddr+i+flashbase);
        if (*addr != val)
        {
            printf("Mismatch @%x %x != %x\n", 
                   (saddr+i+flashbase), 
                   *addr, val);
            errors++;
            if (errors > ERROR_LIMIT)
            {
                printf("Too many errors, bailing\n");
                exit(1);
            }
        }
    }
}
