/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/incl/etherswitch.h,v 1.3 2003/09/22 21:48:45 jployer Exp $ 
* 
* Purpose: 
*    This file contains the function prototypes and definitions for the 
*    Ethernet Switch Module of AP. 
* 
* Public Procedures: 
*    etherSwitch_Init           Initialzies the Ethernet Switch Task and 
*                               related components 
*    etherSwitchTask_Start      Starts running the Ethernet Switch Task. 
*    etherSwitchTask_SendEvent  Mechanism used to trigger an event indicating a 
*                               message has been received on the message 
*                               queue. 
* 
* Notes: 
*    None. 
* 
*****************************************************************************/

#ifndef _ETHERSWITCH_H_
#define _ETHERSWITCH_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "osif.h"

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
#define etherSwitch_MSG_RCVD               (1<<0)
#define etherSwitch_TIMER_EXPIRED          (1<<1)

/*-------------------------------------*/
/* Ethernet Switch Task Messages       */
/*-------------------------------------*/
typedef struct etherSwitchQ_Msg_t
{
    UINT32 command;
    UINT32 data;
}
etherSwitchQ_Msg_t;

/*----------------*/
/* Timer Messages */
/*----------------*/
typedef struct etherSwitchQ_TimerMsg_t
{
    UINT32 *PendingData_p;
    UINT32 Id;
}
etherSwitchQ_TimerMsg_t;
/* */
/* Structure used for messages placed on the Ethernet Task's */
/* timer message queue. The fields in the structure are: */
/* 1) PendingData_p - Pointer to relevant saved data */
/* 2) Id            - ID of the timer */
/* */

/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/******************************************************************************
 * 
 * Name: etherSwitch_Init  
 * 
 * Description: 
 *   This routine is called to initialize the ethernet switch registers and 
 *   ports.
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern void etherSwitch_Init( void );

/******************************************************************************
 * 
 * Name: etherSwitchTask_Start 
 * 
 * Description: 
 *   This routine is called to start the Ethernet Switch Management Task running. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   etherSwitch_Init(). 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None.
 * 
 *****************************************************************************/
extern void etherSwitchTask_Start( void );

/******************************************************************************
 * 
 * Name: etherSwitchTask_SendEvent 
 * 
 * Description: 
 *   This routine provides the mechanism to trigger an event indicating to 
 *   the Ethernet Switch Task that a message has been written to its 
 *   queue. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   etherSwitchTask_Init(). 
 * 
 * Arguments: 
 *   Arg1 (i  ): Event - value indicating which queue has received a message 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * 
 *****************************************************************************/
extern WL_STATUS etherSwitchTask_SendEvent( UINT32 Event );


/******************************************************************************
 *
 * Name: etherSwitchQ_WriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where ethernet
 *   switch messages are placed for the Ethernet Switch Service Task. If
 *   writing to the queue cannot immediately occur, then the routine returns
 *   with a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling etherSwitchQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Msg_p - a pointer to the message to be placed on
 *                       the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS etherSwitchQ_WriteNoBlock( etherSwitchQ_Msg_t *Msg_p );


/******************************************************************************
 * 
 * Name: etherSwitchReset  
 * 
 * Description: 
 *   This routine is resets the ethernet switch using the GPIO port.
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * END PDL 
 * 
 *****************************************************************************/
extern void etherSwitchReset( void );

#endif /* _ETHERSWITCH_H_ */
