/*******************************************************************
*
* $Header :$
*    DESCRIPTION: Implement apio_* API
*    The APIO API is for receiving frames from and sending frames to
*    the network ports: wireless, wan, lan
*
*    AUTHOR: cvu@marvell.com
*
*    HISTORY:
*    06nov02,cvu,created
*
*    DATE:11/6/2002
*
*******************************************************************/
#include "osif.h"
#include "sysinfo.h"
#include "ap_api.h"
#include "oem.h"
#include "IEEE_types.h"
#include "bufmgt.h"
#include "bridge.h"

/* Imported functions */
extern UINT32 * apio_GetWlanTxBuf(void);
extern UINT32 * apio_GetEtherTxBuf(void);

#ifdef WDS_FEATURE
extern char *strstr( const char *, const char * );
#endif /* WDS_FEATURE */

#ifdef WDS_FEATURE
#ifndef MAX_WDS_PORT
#define MAX_WDS_PORT   6
#endif
int first_open_wds_port=1; 
#endif /* WDS_FEATURE */
/**********************************
* Functions defined in this file  *
**********************************/
apio_ErrCode_t default_close(apio_handle_t handle);

/*******************
* Local definition *
*******************/ 
/* Events to block on */
#define DS_EVENTS     (APIO_POLL_ETH_DATA_RDY|APIO_POLL_RADIO_DATA_RDY| \
                       APIO_POLL_WAN_DATA_RDY | \
                       APIO_POLL_PROT_STACK_DATA_RDY)

/* apio function pointer */
typedef apio_ErrCode_t (*apio_func_t)();

/* Definition for class network port */
typedef struct class_port_t
{
    UINT16 portType; /* use this to decode privateData */
    UINT16 portStatus;
    apio_func_t read;
    apio_func_t write;
    apio_func_t close;
    apio_func_t alloc;
    apio_func_t free;
    void (*txDoneCb)(); /* Transmit done callback function */
    void *privateData;
}
class_port_t;

#ifdef WDS_FEATURE
typedef struct _t_WDS_Port_CTL_Descr
{
   UINT8 repeaterID[6];
   apio_bufdescr_t BufDescr;
   int haveData;
}t_WDS_Port_CTL_Descr;
t_WDS_Port_CTL_Descr WDS_Port_CTL_Descr[MAX_WDS_PORT+1];
#endif

#define PORT_STATUS_CLOSED 0
#define PORT_STATUS_OPEN   0x0001 
/* Include specific APIO port definition for OEM */

/***************************************************
* Functions imported from outside this file        *
***************************************************/
#ifdef WDS_FEATURE
extern UINT32 apio_wirelessWrite( apio_bufdescr_t *pBufDescr,IEEEtypes_MacAddr_t *RepeatId);
extern UINT32 apio_wirelessRead(apio_bufdescr_t *pBufDescr,UINT8 *apmac);
extern void apEvtRun(UINT16 evtCode, UINT8 *body, UINT8 status);
#endif 
extern apio_ErrCode_t apio_wlanRead(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_wlanWrite(apio_bufdescr_t *pBufDescr);
extern void apio_FreeWlanRxBuf(apio_bufdescr_t *pBufDescr);
#ifdef WDS_FEATURE
extern void apio_FreeWlanRxBufDirect(apio_bufdescr_t *pBufDescr);
#endif 
extern void apio_FreeEtherRxBuf(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_wanRead(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_wanWrite(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_lanRead(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_lanWrite(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_etherWrite( apio_bufdescr_t *pBufDescr);
#ifdef SOHO_GWAY
extern apio_ErrCode_t apio_etherRead(apio_bufdescr_t *pBufDescr,
                                         int pktFilterSel);
#else
extern apio_ErrCode_t apio_etherRead(apio_bufdescr_t *pBufDescr);
#endif
extern apio_ErrCode_t apio_write_wan(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write_lan(apio_bufdescr_t *pBufDescr);
#ifdef WDS_FEATURE
extern apio_ErrCode_t apio_wdsread(apio_handle_t handle,apio_bufdescr_t *pBufDescr);
extern UINT32 apio_wdswrite(apio_handle_t handle,apio_bufdescr_t *pBufDescr);
extern int str2mac( char *str, char *mac );
#endif 


/**********************************
* Functions defined in this file  *
**********************************/
#ifdef WDS_FEATURE
apio_ErrCode_t apio_wlanAlloc(apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_wlanFree(apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_ethAlloc(apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_ethFree(apio_bufdescr_t *pBufDescr);
#else
apio_ErrCode_t apio_wlanAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_wlanFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_ethAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_ethFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
#endif 
apio_ErrCode_t apio_read_wan(apio_bufdescr_t *pBufDescr);
apio_ErrCode_t apio_read_lan(apio_bufdescr_t *pBufDescr);

/* Port type enumeration */
enum portType_e
{
    portType_wlan,
    portType_wan,
    portType_lan,
    portType_eth,
#ifdef WDS_FEATURE
    portType_wds,
    portType_llc,
#endif 
};

/* Index of packet filter function (see ether.c) */
/* Do not use 0. It's reserved for the default do-nothing filter */
#define RXPKTFILTER_WAN 1
#define RXPKTFILTER_LAN 2

#ifdef WDS_FEATURE
apio_ErrCode_t Empty_Deal(apio_bufdescr_t *pBufDescr)
{
	return APIO_NOERROR;
}

void Empty_Deal_Done(apio_bufdescr_t *pBufDescr)
{
	return;
}
#endif 

/************************************************
* The network port objects, statically defined  *
************************************************/

/* WLAN port object */
class_port_t wlanPort =
    {
        portType_wlan,
        PORT_STATUS_CLOSED,
        apio_wlanRead,
        apio_wlanWrite,
        default_close,
        apio_wlanAlloc,
        apio_wlanFree,
        apio_FreeWlanRxBuf,
        NULL
    };
#ifdef SOHO_GWAY 
/* WAN port object */
class_port_t wanPort =
    {
        portType_wan,
        PORT_STATUS_CLOSED,
        apio_read_wan,
        apio_etherRead,
        apio_etherWrite,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };

/* LAN port object */
class_port_t lanPort =
    {
        portType_lan,
        PORT_STATUS_CLOSED,
        apio_read_lan,
        apio_etherWrite,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };
#endif 
/* LAN port object */
class_port_t ethPort =
    {
        portType_eth,
        PORT_STATUS_CLOSED,
        apio_etherRead,
        apio_etherWrite,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };

#ifdef WDS_FEATURE
class_port_t wdsPort[MAX_WDS_PORT+1];
//bellow variable is used to init wdsPort[...]
class_port_t wdsPortModule =
    {
        portType_wds,
        PORT_STATUS_CLOSED,
        Empty_Deal,//apio_read() will call apio_wdsread() directly
        Empty_Deal,//apio_write() will call apio_wdswrite() directly
        default_close,
        apio_wlanAlloc,
        apio_wlanFree,
        apio_FreeWlanRxBuf,
        NULL //this filed will point to each port specified private struct in apio_open()
    };

/* llc simulate port object */
class_port_t llcPort =
    {
        portType_llc,
        PORT_STATUS_CLOSED,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal_Done,
        NULL
    };
apio_handle_t llcHandle = (apio_handle_t ) &llcPort;
#endif 
#ifdef SOHO_GWAY 
/**
 * Read from WAN port
 * 
 * @param pBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_read_wan(apio_bufdescr_t *pBufDescr)
{
    return (apio_etherRead(pBufDescr, RXPKTFILTER_WAN));
}

/**
 * Read from LAN port
 * 
 * @param pBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_read_lan(apio_bufdescr_t *pBufDescr)
{
    return (apio_etherRead(pBufDescr, RXPKTFILTER_LAN));
}
#endif 
/**
 * Allocate a buffer for the WLAN port
 * 
 * @param handle     The WLAN port handle
 * @param ppBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
#ifdef WDS_FEATURE
apio_ErrCode_t apio_wlanAlloc(apio_bufdescr_t *pBufDescr)
#else
apio_ErrCode_t apio_wlanAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
#endif 
{
    if ((pBufDescr->framePtr = (UINT8 *)apio_GetWlanTxBuf()) != NULL)
#ifdef AP_USE_BUFFER_OFFSET
    {
        *(pBufDescr->framePtr-1) = AP_BUFFER_OFFSET;
        return APIO_NOERROR;
    }
#else
        return APIO_NOERROR;
#endif
    else
        return APIO_ERROR;

}

/**
 * Free a buffer allocated by apio_wlanAlloc
 * 
 * @param handle    Handle of the WLAN port
 * @param pBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
#ifdef WDS_FEATURE
apio_ErrCode_t apio_wlanFree( apio_bufdescr_t *pBufDescr)
#else
apio_ErrCode_t apio_wlanFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
#endif
{
    apio_FreeWlanRxBuf(pBufDescr);
    return APIO_NOERROR;
}

/**
 * Allocate a buffer for the WAN and LAN port
 * 
 * @param handle     The WAN or LAN port handle
 * @param ppBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
#ifdef WDS_FEATURE
apio_ErrCode_t apio_ethAlloc(apio_bufdescr_t *pBufDescr)
#else
apio_ErrCode_t apio_ethAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
#endif 
{
    if ((pBufDescr->framePtr = (UINT8 *)apio_GetEtherTxBuf()) != NULL)
#ifdef AP_USE_BUFFER_OFFSET
    {
        *(pBufDescr->framePtr-1) = AP_BUFFER_OFFSET;
        return APIO_NOERROR;
    }
#else
        return APIO_NOERROR;
#endif
    else
        return APIO_ERROR;
}

/**
 * Free a buffer allocated by apio_ethAlloc
 * 
 * @param handle    Handle of the WAN or LAN port
 * @param pBufDescr Pointer to the buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
#ifdef WDS_FEATURE
apio_ErrCode_t apio_ethFree(apio_bufdescr_t *pBufDescr)
#else
apio_ErrCode_t apio_ethFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
#endif 
{
    apio_FreeEtherRxBuf(pBufDescr);
    return APIO_NOERROR;
}


/**
 * Open a network port.
 * Return a handle to the port.
 * 
 * @param pName    Pointer to the port name. The port name must be
 *                 a recognized name. Use the PORT_NAME macros in apctl.h
 * @param pHandle
 * @param txDoneCb
 * @return APIO_NOERROR : success
 *         APIO_ERROR   : failure
 */
apio_ErrCode_t apio_open(char *pName, apio_handle_t *pHandle,
                         void (*txDoneCb) (apio_bufdescr_t *pBufDescr))
{
    apio_ErrCode_t rc = APIO_NOERROR;

    if (strcmp(pName, PORT_NAME_WLAN) == 0)
    {
        if (txDoneCb != NULL)
        {
            wlanPort.txDoneCb = txDoneCb;
        }
        wlanPort.portStatus = PORT_STATUS_OPEN;
        *pHandle = (apio_handle_t ) & wlanPort;
#ifdef WDS_FEATURE
        goto success;
#endif 
    }
    else if (strcmp(pName, PORT_NAME_ETH) == 0)
    {
        if (txDoneCb != NULL)
        {
            ethPort.txDoneCb = txDoneCb;
        }
        ethPort.portStatus = PORT_STATUS_OPEN;
        *pHandle = (apio_handle_t ) & ethPort;
#ifdef WDS_FEATURE
        goto success;
#endif 
    }
#ifdef WDS_FEATURE
    else if(strstr(pName,"wds"))
    {
    	UINT8 mac[6];
        int i;
                if(first_open_wds_port)
                {
			first_open_wds_port = 0;
			for(i=0;i<=MAX_WDS_PORT;i++)
			{
                                wdsPort[i] = wdsPortModule;
			}
                }

                if(strlen(pName) != 15)
			goto error;
		    	
		pName = pName + 3;
		
		if(!(str2mac(pName,mac)))
		{
			for(i=0;i<=MAX_WDS_PORT;i++)
			{
				if((wdsPort[i].portStatus == PORT_STATUS_OPEN)&&(memcmp(mac,WDS_Port_CTL_Descr[i].repeaterID,6)==0))
					goto error;
			}
			for(i=0;i<=MAX_WDS_PORT;i++)
			{
				if(wdsPort[i].portStatus == PORT_STATUS_CLOSED)
				{
               wdsPort[i].portStatus = PORT_STATUS_OPEN;
               if (txDoneCb != NULL)
               {
                   wdsPort[i].txDoneCb = txDoneCb;
    }
               wdsPort[i].privateData = &WDS_Port_CTL_Descr[i];
               memcpy(WDS_Port_CTL_Descr[i].repeaterID,mac,6);
               WDS_Port_CTL_Descr[i].haveData = 0x55;
               *pHandle = (apio_handle_t)&wdsPort[i];
               goto success;
				}
			}
		}
		goto error;
	}
#endif // WDS_FEATURE
#ifdef SOHO_GWAY
    else if (strcmp(pName, PORT_NAME_WAN) == 0)
    {
        if (txDoneCb != NULL)
        {
            wanPort.txDoneCb = txDoneCb;
        }
        wanPort.portStatus = PORT_STATUS_OPEN;
        *pHandle = (apio_handle_t ) & wanPort;
#ifdef WDS_FEATURE
        goto success;
#endif 
    }
    else if (strcmp(pName, PORT_NAME_LAN) == 0)
    {
        if (txDoneCb != NULL)
        {
            lanPort.txDoneCb = txDoneCb;
        }
        lanPort.portStatus = PORT_STATUS_OPEN;
        *pHandle = (apio_handle_t ) & lanPort;
#ifdef WDS_FEATURE
        goto success;
#endif 
    }
#endif
#ifdef WDS_FEATURE
error:    	
    *pHandle = 0;
	rc = APIO_ERROR;
success:
#else
    else
    {
        *pHandle = 0;
        rc = APIO_ERROR;
    }
#endif 

    return rc;
}


/**
 * Send packet to a network port.
 * 
 * @param handle    The handle to the port
 * @param pBufDescr Pointer to the packet descriptor
 * @return APIO_NOERROR: success
 *         APIO_ERROR  : error
 */
apio_ErrCode_t apio_write(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
    register class_port_t *pPortObj = (class_port_t *) handle;

#ifdef WDS_FEATURE
    if(pPortObj->portType == portType_wds)
    return apio_wdswrite(handle,pBufDescr);
    else
#endif 
    return (pPortObj->write(pBufDescr));
}

/**
 * Receive a packet from the port specified by the handle
 * 
 * @param handle    Handle to the port
 * @param pBufDescr
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_read(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
    register class_port_t *pPortObj = (class_port_t *) handle;

#ifdef WDS_FEATURE
    if(pPortObj->portType == portType_wds)
    return apio_wdsread(handle,pBufDescr);
    else
#endif 
    return (pPortObj->read(pBufDescr));
}

/**
 * Close a port. Release resources.
 * 
 * @param handle Handle to the port
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_close(apio_handle_t handle)
{
    register class_port_t *pPortObj = (class_port_t *) handle;

#ifdef WDS_FEATURE
    if(pPortObj->portType == portType_wds)
    {   
        WdsPortClose(handle);
        pPortObj->portStatus = PORT_STATUS_CLOSED;
        return APIO_NOERROR;
    }
    else
#endif 
    return (pPortObj->close(handle));
}

/**
 * Allocate buffer for a port
 * 
 * @param handle     Handle to the port
 * @param pktSz
 * @param ppBufDescr
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_alloc(apio_handle_t handle,
                          apio_bufdescr_t *pBufDescr)
{
    register class_port_t *pPortObj = (class_port_t *) handle;

    return (pPortObj->alloc(handle, pBufDescr));
}

/**
 * Free a buffer allocated by apio_alloc
 * 
 * @param handle    Handle to the port
 * @param pBufDescr Pointer to the buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t apio_free(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
    register class_port_t *pPort = (class_port_t *) handle;

    return (pPort->free(pBufDescr));
}

/**
 * Block until a defined event occurs
 * 
 * @return The events that caused wake up
 */
UINT32 apio_select(void)
{
    OS_EVENTS EventsTriggered;
#ifdef USE_NEW_OSIF
    EventsTriggered = os_EventWait(dsEH,
#else
    EventsTriggered = os_EventWait(sysinfo_DS_EVENT,
#endif
                                   DS_EVENTS,
                                   OS_EVENT_WAITMODE_CLR_OR);
    return (EventsTriggered);
}

/**
 * Default close function. Does nothing.
 * 
 * @param handle Handle to the port
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
apio_ErrCode_t default_close(apio_handle_t handle)
{
    register class_port_t *pPort = (class_port_t *) handle;
    pPort->portStatus = PORT_STATUS_CLOSED;
    return APIO_NOERROR;
}

#ifdef WDS_FEATURE
apio_ErrCode_t apio_wdsread(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
t_WDS_Port_CTL_Descr *p;

	if(((class_port_t *)handle)->privateData != NULL)
	{
		p = (t_WDS_Port_CTL_Descr *)(((class_port_t *)handle)->privateData);
		if(p->haveData == 0xaa)
		{
	      p->haveData = 0x55;
            //*pBufDescr = p->BufDescr;
            pBufDescr->frameLen =p->BufDescr.frameLen;
            pBufDescr->framePtr  =p->BufDescr.framePtr;
            return 1;
		}
	}
    return 0;  
}

UINT32 apio_wdswrite(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
t_WDS_Port_CTL_Descr *p;
class_port_t *pPortObj;

	pPortObj= (class_port_t *) handle;
        p = (t_WDS_Port_CTL_Descr *)(pPortObj->privateData);
        return (apio_wirelessWrite(pBufDescr,&(p->repeaterID)));
}

UINT32 apio_GetRepeaterIndx(IEEEtypes_MacAddr_t *RepeatId)
{
    extern UINT8 mib_wdsEnable;
    UINT32 i;
    t_WDS_Port_CTL_Descr *p;
    for(i=0;i<(MAX_WDS_PORT+1);i++)
    {/*search all wds ports*/
    	if(wdsPort[i].portStatus == PORT_STATUS_OPEN)
    	{/*if port opened*/
    		p = (t_WDS_Port_CTL_Descr*)(wdsPort[i].privateData);
    		if(memcmp(p->repeaterID,RepeatId,6) == 0)
    		{
                      return(i|0x80);
    		}
		}
	}
    /* If static, do not add new entries */
    if ( mib_wdsEnable == 1 )
        return(0);

    return(NewRouterHandler((UINT8*)RepeatId));
}


/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
#if 0
extern apio_ErrCode_t apio_wlanRead(apio_bufdescr_t *pBufDescr)
{
UINT32 result;
int i;
UINT8 RepeatId[6];
t_WDS_Port_CTL_Descr *p;
apio_bufdescr_t tmpBufDescr;
UINT16 typelen;


    result = apio_wirelessRead(pBufDescr,RepeatId);
    if(result != APIO_WLAREAD_NO_PACKET) // 2003/08/19, fixed by jiaqi
    {
      typelen = pBufDescr->framePtr[12] << 8;
      typelen += pBufDescr->framePtr[13];
    }
    if((result != APIO_WLAREAD_NO_PACKET)&&(typelen < 1600))
    {//here allocate a new buffer for incoming packet not ethernet packet
		if ((tmpBufDescr.framePtr = (UINT8 *)apio_GetWlanTxBuf()) != NULL)
       	{
            		tmpBufDescr.frameLen = pBufDescr->frameLen;
            		memcpy(tmpBufDescr.framePtr,pBufDescr->framePtr,pBufDescr->frameLen);
            		apio_FreeWlanRxBuf(pBufDescr);
            		pBufDescr->frameLen = tmpBufDescr.frameLen;
            		pBufDescr->framePtr = tmpBufDescr.framePtr;
        }
		else
		{
            		apio_FreeWlanRxBuf(pBufDescr);
            		return APIO_WLAREAD_NO_PACKET;
		}
    }
    
    switch(result)
    {
    	case APIO_WLAREAD_PACKET_WDS:
    		for(i=0;i<=MAX_WDS_PORT;i++)
    		{//search all wds ports
    			if(wdsPort[i].portStatus == PORT_STATUS_OPEN)
    			{//if port opened
    				p = (t_WDS_Port_CTL_Descr*)(wdsPort[i].privateData);
    				if(memcmp(p->repeaterID,RepeatId,6) == 0)
    				{//if incoming packet for this port
#if 0
                        if(p->haveData != 0xaa)
    					{//no packet has been pended in this wds port
							p->BufDescr.frameLen = pBufDescr->frameLen;
	    					p->BufDescr.framePtr= pBufDescr->framePtr;
	    					p->haveData = 0xaa;
	    					break;
	    				}
	    				else
	    				{
	    					apio_wlanFree(pBufDescr);
	    					return 0;
	    				}
#else
                            DSForwardPacket(WDS_Port_CB[i].wdsxHandle,pBufDescr);
                            break;
#endif
	    				
    				}
			}
		}
		if(i == MAX_WDS_PORT+1)
		{//can't find a wds port for this wds packet,tigger a event and free packet
			apEvtRun(APEVT_UNKNOWN_WDS_ROUTE, RepeatId, 0);
			apio_wlanFree(pBufDescr);
			//to see if another packer received
		}
		return 0;
		
    	case APIO_WLAREAD_PACKET_BSS:
    		return 1;
    		
    	default:
    		return 0;
    }
}
#endif
int str2mac( char *str, char *mac )
{
	int i,j;
	char byte=0;

	if(strlen(str) != 12)
		return -1;

	for(i=0,j=0;i<12;)
	{
		if((str[i]>='A')&&(str[i]<='F'))
			str[i] = str[i] - 'A' + 'a';

		if((str[i]>='0')&&(str[i]<='f'))
		{
			byte = (byte<<4) + (str[i]>='a' ? str[i]-'a'+10 : str[i]-'0');
		}
		else
			return -1;

		if(((++i)&1)==0)
		{
			mac[j++] = byte;
			byte = 0;
		}
	}
	return 0;
}
#endif // WDS_FEATURE

