/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/tx80211.c,v 1.11 2004/01/16 18:24:06 rahulk Exp $
*
* Purpose:
*    This file contains the implementations of the function prototypes given
*    in the associated header file for the Station Managment Entity (SME)
*    Task.
*
* Public Procedures:
*    tx80211_Init       Initialzies the SME Task and related components
*    tx80211_Start      Starts running the SME Task
*    tx80211_SendEvent  Mechanism used to trigger an event indicating a
*                       message has been received on one of the message
*                       queues
*
* Private Procedures:
*    SmeTask            The actual SME Task
*
* Notes:
*    None.
*
*****************************************************************************/

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "memmap.h"
#include "sysinfo.h"
#include "tkip.h"
#include "tx80211.h"
#include "txsched.h"
#include "ap_mib.h"
#include "etherlow.h"
#include "ether.h"
#include "api2.h"
#include "ap_api.h"
#include "ds.h"
#include "bufmgt.h"
#include "Fragment.h"
#include "wltypes.h"
#include "eth_debug.h"
#include "wl_macros.h"
#include "macMgmtMlme.h"
#include "stats.h"
#ifdef USE_NEW_OSIF
static TASK_HANDLE tx80211TH;

#ifdef ECOS
static UINT8  Stack[sysinfo_802_11_MSG_TX_TASK_STACK_SIZE]__attribute__ ((section (".itcmbuf")));
#else
static UINT8  Stack[sysinfo_802_11_MSG_TX_TASK_STACK_SIZE];
#endif


static QUEUE_HANDLE msg_txQH, data11_txQH, data3_txQH; 
#endif

/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */
#define SHORT_PREAM 0

#define TX80211_MAIN_EVENT_TRIGGERS tx80211_MGMT_MSG_RCVD    | \
                               tx80211_DATA_RCVD | \
                               tx80211_CFP_START_RCVD | \
                               tx80211_CFP_END_RCVD | \
                               tx80211_FRAME_TX_DONE | \
                               tx8023_DATA_RCVD | \
							   tx80211_TX_WD_TIMEOUT | \
							   tx80211_ETHER_TX_DONE


extern QHANDLE ethRxQ, ethTxQ, wlRxQ, wlTxQ;


extern UINT32 EtherEventSentFlag;


extern void DoEncrypt(UINT32 des, UINT32 src, UINT32 len);
extern void InitFragBuf(void) ;
extern void MemDma1(UINT32 x, UINT32 y, UINT32 z);

WL_STATUS txsched_FreeQdData(void);
extern int FreeQdBuffers(void);
/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */
#ifdef WEP_ITCM
    #if WEP_ITCM > 1
extern void wep_encrypt(unsigned char *buf, int len) __attribute__ ((section (".itcmbuf")));
    #else
extern void wep_encrypt(unsigned char *buf, int len) __attribute__ ((section (".data")));
    #endif
void (*wep_encryptFp)(unsigned char *, int) = wep_encrypt;
#else
extern void wep_encrypt(unsigned char *buf, int len);
#endif

static void Tx80211Task( OS_DATA Data );

/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */

/******************************************************************************
 *
 * Name: tx80211_Init
 *
 * Description:
 *   This routine is called to initialize the the SME Task and related
 *   components.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *   Create the Queues
 *   If the queue was successfully initialized Then
 *      Create the Tx80211 Task by calling os_TaskCreate()
 *      If creating the Tx80211 Task succeeded Then
 *         Return OS_SUCCESS
 *      End If
 *   End If
 *
 *   Return OS_FAIL
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS tx80211_Init( void )
{
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueCreate(&msg_txQH)) !=
        OS_SUCCESS)
    {
        return(OS_FAIL);
    }

    if ((status = os_QueueCreate(&data11_txQH))
        != OS_SUCCESS)
    {
        return(OS_FAIL);
    }

    if ((status = os_QueueCreate(&data3_txQH))
        != OS_SUCCESS)
    {
        return(OS_FAIL);
    }

    os_EventInit(&tx11EH);
    if ((status = os_TaskCreate(&tx80211TH,
                                Tx80211Task,
                                (OS_DATA)0,
                                sysinfo_802_11_MSG_TX_TASK_NAME,
                                (void *)Stack,
                                sysinfo_802_11_MSG_TX_TASK_STACK_SIZE,
                                sysinfo_802_11_MSG_TX_TASK_PRI)) == OS_SUCCESS)
#else
    if ((status = os_QueueCreate(sysinfo_802_11_MSG_TX_Q)) !=
        os_SUCCESS)
    {
        return(OS_FAIL);
    }

    if ((status = os_QueueCreate(sysinfo_802_11_DATA_TX_Q))
        != os_SUCCESS)
    {
        return(OS_FAIL);
    }

    if ((status = os_QueueCreate(sysinfo_802_3_DATA_TX_Q))
        != os_SUCCESS)
    {
        return(OS_FAIL);
    }

    if ((status = os_QueueCreate(sysinfo_802_11_RX_PS_POLL_Q))
        != os_SUCCESS)
    {
        return(OS_FAIL);
    }

    os_EventInit(sysinfo_802_11_TX_Q_EVENT);
    if ((status = os_TaskCreate(sysinfo_802_11_MSG_TX_TASK,
                                Tx80211Task,
                                (os_Addrword_t)0,
                                sysinfo_802_11_MSG_TX_TASK_NAME,
                                sysinfo_802_11_MSG_TX_TASK_STACK_SIZE,
                                sysinfo_802_11_MSG_TX_TASK_PRI)) == OS_SUCCESS)
#endif
    {
#ifdef ENABLE_TX_FRAG
    InitFragBuf();  /* init fragmentation structures */
#endif
        return(OS_SUCCESS);
    }

    return(OS_FAIL);
}


/******************************************************************************
 *
 * Name: tx80211_Start
 *
 * Description:
 *   This routine is called to start the SME Task running.
 *
 * Conditions For Use:
 *   The task and its associated queues have been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    Start the SME Task by calling os_TaskResume()
 * END PDL
 *
 *****************************************************************************/
extern void tx80211_Start( void )
{
#ifdef USE_NEW_OSIF
    os_TaskResume(tx80211TH);
#else
    os_TaskResume(sysinfo_802_11_MSG_TX_TASK);
#endif
}


/******************************************************************************
 *
 * Name: tx80211_SendEvent
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *   the SME Task that a message has been written to one of its queues.
 *
 * Conditions For Use:
 *   The task and its associated queues has been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    If Event is one of the defined events for this task Then 
 *       Call os_EventTrigger to trigger the received event
 *       Return OS_SUCCESS
 *    Else
 *       Return OS_FAIL
 *    End If
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS tx80211_SendEvent( UINT8 Event )
{


    if (Event == tx80211_MGMT_MSG_RCVD ||
        Event == tx80211_DATA_RCVD ||
        Event == tx8023_DATA_RCVD ||
        Event == tx80211_CFP_START_RCVD ||
        Event == tx80211_CFP_END_RCVD ||
        Event == tx80211_FRAME_TX_DONE ||
        Event == tx80211_TX_WD_TIMEOUT ||
        Event == tx80211_ETHER_TX_DONE)
    {
#ifdef USE_NEW_OSIF
        os_EventTrigger(tx11EH, Event);
#else
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, Event);
#endif
        return(OS_SUCCESS);
    }
    else
    {
        return(OS_FAIL);
    }
}


/*============================================================================= */
/*                         CODED PRIVATE PROCEDURES */
/*============================================================================= */

/******************************************************************************
 *
 * Name: Tx80211Task
 *
 * Description:
 *   This routine is the task that runs continuously, waking to process
 *   messages received on its queues. It commences when the tx80211_Start()
 *   routine is invoked.
 *
 * Conditions For Use:
 *   The task and related components have been initialized by calling
 *   tx80211_Init() and then the task has been started by calling
 *   tx80211_Start().
 *
 * Arguments:
 *   Arg1 (i  ): Data - Data passed to the task; not actually used - just
 *                      included to conform to the structure the OS
 *                      expects when creating a task
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *   While forever
 *      Wait for an event to occur by calling os_EventWait()
 *      If an error occurred waiting for an event Then
 *         Do some error handling
 *      Else
 *         If the event tx80211_MGMT_MSG_RCVD occurred Then
 *         End If
 *
 *      End If
 *   End While
 * END PDL
 *
 *****************************************************************************/
static void Tx80211Task( OS_DATA Data )
{
    OS_EVENTS eventsTriggered;
    IEEEtypes_Frame_t *MgmtMsg_p;
/*    txInfo_t *txInfo_p; */
#ifdef ETH_PACKET_FIX
    volatile 
#endif    
    ETHERNET_PRIV *ether_priv;
    ether_priv = (ETHERNET_PRIV *)eth_dev.PrivData;
    while (1)
    {

#ifdef USE_NEW_OSIF
        eventsTriggered = os_EventWait(tx11EH,
#else
        eventsTriggered = os_EventWait(sysinfo_802_11_TX_Q_EVENT,
#endif
                                       TX80211_MAIN_EVENT_TRIGGERS,
                                       OS_EVENT_WAITMODE_CLR_OR);

        /*------------------------------*/
        /* Check for an error condition */
        /*------------------------------*/
        if (eventsTriggered == 0)
        {
            /* Do some error handling */
        }
        else
        {
            /*---------------------------------------------------*/
            /* Check for receipt of an 802.11 management message */
            /*---------------------------------------------------*/
            if (eventsTriggered & tx80211_MGMT_MSG_RCVD)
            {
                /* Read the data from the queue */
#ifdef USE_NEW_OSIF
                while ((MgmtMsg_p = os_QueueReadNoBlock(msg_txQH))
#else
                while ((MgmtMsg_p = os_QueueReadNoBlock(sysinfo_802_11_MSG_TX_Q))
#endif
                       != NULL)
                {
                    USR_BUF_DESC *usp;
                    WLAN_TX_FRAME *Frame_p;

                    usp = msi_wl_GetUsrBufPtr(MgmtMsg_p);
                    
                    if (!usp)
                    {
                        break;
                    }

                    Frame_p = (WLAN_TX_FRAME *)usp->info;

                    Frame_p->IEEEFrame = (IEEEtypes_Frame_t *)MgmtMsg_p;

                    Frame_p->wlanTxHdr.StnId = 0;
                    Frame_p->wlanTxHdr.RateIdx = 1;
                    Frame_p->wlanTxHdr.Preample = PREAMBLE_LONG;

                    txsched_ScheduleFrame(Frame_p);
                }
            }
#ifdef FREE_IN_TXTASK
            if (eventsTriggered & tx80211_FRAME_TX_DONE)
            {

                ProcessTxDone();



            }
            if (eventsTriggered & tx80211_ETHER_TX_DONE)
            {
                msi_wl_Eth_free_tx_queue (&eth_dev, 0);
#ifdef ETH_PACKET_FIX
                ENTER_CRITICAL;
#endif                
                EtherEventSentFlag &= ~(0x8);
                GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                              ~(unsigned int)(0xc));
#ifdef ETH_PACKET_FIX
                EXIT_CRITICAL;
#endif                
            }
#endif
            /*----------------------------------------------------------*/
            /* Check for receipt of commands from the CB Processor Task */
            /*----------------------------------------------------------*/
            if (eventsTriggered & tx80211_DATA_RCVD)
            {
                /* data is not processed here */

                WLAN_TX_FRAME *Frame_p;
#ifdef USE_NEW_OSIF
                while( Frame_p = os_QueueReadNoBlock(data11_txQH))
#else
                while( Frame_p = os_QueueReadNoBlock(sysinfo_802_11_DATA_TX_Q))
#endif
                {
                    txsched_ScheduleFrame(Frame_p);
                }

            }

            if (eventsTriggered & tx8023_DATA_RCVD)
            {
                apio_bufdescr_t *pBufDescr;
#ifdef USE_NEW_OSIF
                while( pBufDescr = os_QueueReadNoBlock(data3_txQH))
#else
                while( pBufDescr = os_QueueReadNoBlock(sysinfo_802_3_DATA_TX_Q))
#endif
                {
                    w8100_eth_xmit(pBufDescr->framePtr, pBufDescr->frameLen,
                                   &eth_dev);
                }

            }


/* //Rahul. Not used
            if (eventsTriggered & tx80211_CFP_END_RCVD)
            {
                //ProcessCfpEnd(); 

                WL_WRITE_WORD(MIR_INTR_STATUS, ~CFP_END);

            }

            if (eventsTriggered & tx80211_CFP_START_RCVD)
            {
                //ProcessCfpStart();
            }

*/
            if ( eventsTriggered& tx80211_TX_WD_TIMEOUT )
            {       
                SendResetCmd(1);
            }

        }  /* if events == 0 */
    }  /* end while */
}



/******************************************************************************
 *
 * Name: tx80211Q_MgmtWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMst_p - a pointer to the message to be placed on
 *                           the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS tx80211Q_MgmtWriteNoBlock( macmgmtQ_MgmtMsg_t *MgmtMsg_p )
{
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueWriteNoBlock(msg_txQH,
                                       MgmtMsg_p)) == OS_SUCCESS)
    {
        os_EventTrigger(tx11EH,tx80211_MGMT_MSG_RCVD);
        return(OS_SUCCESS);
    }
#else
    if ((status = os_QueueWriteNoBlock(sysinfo_802_11_MSG_TX_Q,
                                       MgmtMsg_p)) == OS_SUCCESS)
    {
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT,tx80211_MGMT_MSG_RCVD);
        return(OS_SUCCESS);
    }
#endif
    else
    {
        return(OS_FAIL);
    }
}

extern WL_STATUS tx80211Q_DataWriteNoBlock( WLAN_TX_FRAME *Data_p )
{
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueWriteNoBlock(data11_txQH,
                                       Data_p)) == OS_SUCCESS)
    {
        os_EventTrigger(tx11EH,tx80211_DATA_RCVD);
        return(OS_SUCCESS);
    }
#else
    if ((status = os_QueueWriteNoBlock(sysinfo_802_11_DATA_TX_Q,
                                       Data_p)) == OS_SUCCESS)
    {
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT,tx80211_DATA_RCVD);
        return(OS_SUCCESS);
    }
#endif
    else
    {
        return(OS_FAIL);
    }
}


extern WL_STATUS tx8023Q_DataWriteNoBlock( apio_bufdescr_t *Data_p )
{
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueWriteNoBlock(data3_txQH,
                                       Data_p)) == OS_SUCCESS)
    {
        os_EventTrigger(tx11EH,tx8023_DATA_RCVD);
        return(OS_SUCCESS);
    }
#else
    if ((status = os_QueueWriteNoBlock(sysinfo_802_3_DATA_TX_Q,
                                       Data_p)) == OS_SUCCESS)
    {
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT,tx8023_DATA_RCVD);
        return(OS_SUCCESS);
    }
#endif
    else
    {
        return(OS_FAIL);
    }
}



void tx80211_SetTxRate(txInfo_t *txInfo_p,UINT8 rateRegIndex)
{
    txInfo_p->Rate = rateRegIndex;
}
