/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/txsched.c,v 1.33 2004/04/30 01:22:06 jshen Exp $ 
* 
* Purpose: 
*    This file contains the implementations of the transmit queue functions. 
* 
* Public Procedures: 

* 
* Private Procedures: 
*      
* 
* Notes: 
*    None. 
* 
****************************************************************************/

#include "buildModes.h"
#include "wltypes.h"
#include "IEEE_types.h"
#include "tkip.h"
#include "tx80211.h"
#include "txsched.h"
#include "bcngen.h"
#include "StaDb.h"
#include "ds.h"
#include "eth_debug.h"

#include "ap_api.h"    /* TX Rates */

#include "wltypes.h"
#include "bufmgt.h"
#include "psQue.h"
#include "memmap.h"
#include "wl_mib.h"
#include "wl_hal.h"
#include "sysinfo.h"
#include "wl_macros.h"
#include "stats.h"
#include "timer.h"
#include "rateadapt.h"
#include "macMgmtMlme.h"
#include "bufmgt.h"
#include "ap_mib.h"

#ifdef PORT_COEXIST
#define HIGHEST_11B_RATE_REG_INDEX   4
#endif /* PORT_COEXIST */

extern QHANDLE ethRxQ, ethTxQ, wlRxQ, wlTxQ;
/* Imported functions */
extern void Reset(void);



#define IEEE_MSG_TYPE(Hdr_p) (Hdr_p->FrmCtl.Type)
#define IEEE_MSG_SUBTYPE(Hdr_p) (Hdr_p->FrmCtl.Subtype)

#define DONOT_ADD_TO_ALLSTN_LIST 0
#define ADD_TO_ALLSTN_LIST  1

int CurTxDataRate;

UINT32 CBP_QID = TC7_QID;
UINT32 BRATE_QID = TC7_QID;

UINT8 txRateAdaptMethod=RATE_ADAPT_TX;  /* Use TX_RATE_ADAPTATION by default */
#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR
    #define TX_HISTORY_COUNT       14  /*100 -> 30 -> 20 -> 10 -> 15 */
UINT8 tx_success_rate_limit=3;  /*95 -> 27 -> 17 -> 8 -> 12 */
UINT8 tx_failure_rate_limit=2;  /*10 -> 4 - > 3 -> 2 -> 4 */
UINT8 tx_retry_as_failure=3;    /* 2 -> 1 */

typedef enum
{
    TX_NO_STATUS,
    TX_SUCCESS,
    TX_FAILURE
} TxStatus_e;

/*static UINT8 TxRateProb[MAX_STNS][IEEEtypes_MAX_DATA_RATES]; */
static UINT8 *TxRateProb;
void SetupTxRateProb();
static void AdjustTxRateProb(int Aid, BOOLEAN TxAck, UINT8 TxRateProb);

/*UINT32 TxIntervalSuccesses[MAX_STNS]; */
UINT32 *TxIntervalSuccesses;
/* */
/* The number of transmit successes over a static sample size */
/* */

/*UINT32 TxIntervalFailures[MAX_STNS]; */
UINT32 *TxIntervalFailures;
/* */
/* The number of transmit failures over a static sample size */
/* */

static UINT32 TxIntervalTransactions __SQRAM__ = 0;
/* */
/* The number of transmit events have that occurred over a given */
/* interval; used to help decide when to try for a higher data rate */
/* if data rate adaptation is being used */
/* */

    #ifdef buildModes_ENABLE_STEADY_RATE
static UINT32 TxIntervals __SQRAM__ = 0;
    #endif 

/* */
/* The number of transmit intervals that have occurred; used to help */
/* decide when to try for a higher data rate if data rate adaptation */
/* is being used */
/* */

static UINT32 TxTransactions __SQRAM__ = 0;
/* */
/* The number of transmit events - either successes or failures */
/* */

static UINT32 TxTransactionIdx __SQRAM__ = 0;
/* */
/* Index into the TxTransaction array */
/* */

static TxStatus_e TxTransaction[TX_HISTORY_COUNT] __SQRAM__;
/* */
/* An array that records transmit successes and failures; the size */
/* of the array represents a sample size over which transmit */
/* statistics are kept; based on the number of transmit successes */
/* or failures over this sample, the transmit data rate may be */
/* adjusted up or down */
void AdaptTxRate(txInfo_t * txInfo_p, UINT16 Aid);
#endif 
UINT32 UpdateQFailCnt;
/*UINT32 UpdateQDataCnt[MAX_STNS]; */
UINT32 *UpdateQDataCnt;
/*UINT32 PendingQDataCnt[MAX_STNS]; */
UINT32 *PendingQDataCnt;
/*UINT32 DoneQDataCnt[MAX_STNS]; */
UINT32 *DoneQDataCnt;
extern void SetDurationId(txInfo_t *txInfo_p);

UINT32 TxRateReg=12;
UINT32 TxRetryCnt = 4;
UINT32 TxFixedRate = 0;
UINT32 TxMcastRateReg = 0;

int adaptTxRate = 0;

extern UINT32 BStnAroundCnt;

/******************************************************************************
 *
 * Name: txsched_Init
 *
 * Description:
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
WL_STATUS txsched_Init(void)
{
#ifdef POWER_SAVE
    psQue_Init(MAX_STNS);
#endif
#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR
    TxRateProb = malloc( MAX_STNS * IEEEtypes_MAX_DATA_RATES);
    TxIntervalSuccesses = malloc(MAX_STNS*4);
    TxIntervalFailures = malloc(MAX_STNS*4);
    UpdateQDataCnt = malloc(MAX_STNS*4);
    PendingQDataCnt = malloc(MAX_STNS*4);
    DoneQDataCnt = malloc(MAX_STNS*4);
#endif

#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR

    /* */
    /* Code to initialize the TxRateProb array */
    /* */
    SetupTxRateProb(); 
#endif

    return(OS_SUCCESS);
}
/******************************************************************************
 *
 * Name: txsched_ScheduleFrame
 *
 * Description: This function either queues the packet to a HW Queue or adds 
 *     the packet to the buffer list for later queueing.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
UINT8 fixedRate = 12;
extern WL_STATUS txsched_ScheduleFrame(WLAN_TX_FRAME *txFrame_p)
{
    USR_BUF_DESC *usp;
    IEEEtypes_Frame_t *Frame_p;
    UINT8 MsgType, stnId=0;
    /*txInfo_t *txInfo_p; */
    /* Look at the dest addr first
     *  if the dest addr is mcast or broadcast
     * queue the data to MCqueue */
    if (txFrame_p == NULL)
        return(OS_FAIL);

    Frame_p = txFrame_p->IEEEFrame;
    MsgType = txFrame_p->IEEEFrame->Hdr.FrmCtl.Type;

    usp = msi_wl_GetUsrBufPtr((char *)Frame_p);

    if (!WlanEnabled)
    {
        msi_wl_FreeUsrBuf(usp);
        return(OS_FAIL);
    }
    /* If there are any Stations in Power Save Mode, then queue the data in
     * MC/BC queue. If the Queue is already full buffer the data in MC/BC list
     * If there are no Stations in Power Save Mode, Add the data in MC/BC list
     * and the global list and queue them in the CB Q along with other data.
     * When the condition changes from No power save station to some power save
     * stations, the entries in the global list should be removed */

    stnId = txFrame_p->wlanTxHdr.StnId;
    if (stnId == MCBC_STN_ID )
    {
#ifdef ENABLE_WIFI_PROTECTION
	  if (!bOnlyStnCnt && !BStnAroundCnt)
#else
        if (!bOnlyStnCnt)
#endif /* PORT_COEXIST */
            txFrame_p->wlanTxHdr.RateIdx = HighestBasicRateIndex;
        else
            txFrame_p->wlanTxHdr.RateIdx =  HighestBasicRateIndexB;
        {
            if(isAnyStaInPwrSaveMode())
             txFrame_p->wlanTxHdr.QosControl = MCBC_QID;
            else
                txFrame_p->wlanTxHdr.QosControl = CBP_QID;
            txFrame_p->wlanTxHdr.TxParam = 0; 

            txFrame_p->wlanTxHdr.RateIdx =  0; /*  force to broadcast at 1Mbps*/
            if(msi_wl_RFTxData(txFrame_p) == FALSE)
                msi_wl_FreeUsrBuf(usp);
            return(OS_SUCCESS);
        }
    }
#ifdef POWER_SAVE
    /* if the dest stn in power save mode
       queue the buffer for later tx 
       Update the traffic Map for this Aid */
    if (isStaInPowerSave(stnId) && (IEEE_MSG_TYPE((&Frame_p->Hdr)) == IEEE_TYPE_DATA))
    {
        eprintf("save to ps queue, stnid = %d, aid = %d\n", stnId, psStaAid(stnId));
        if (psQue_Put(stnId, txFrame_p, DONOT_ADD_TO_ALLSTN_LIST, MsgType)==OS_SUCCESS)
        /* Get the Aid of this destination */
        /* Update the TIM Element in the Beacon for this AID */
        bcngen_UpdateBitInTim(psStaAid(stnId), SETBIT);
        else
            bcngen_UpdateBitInTim(psStaAid(stnId), RESETBIT);
        return(OS_SUCCESS);
    }
#endif
    {
		txFrame_p->wlanTxHdr.TxParam = 0; 
        if((txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
            txFrame_p->wlanTxHdr.QosControl = CBP_QID;
        else
            txFrame_p->wlanTxHdr.QosControl = BRATE_QID;
#ifdef PORT_COEXIST /* Most of these are handle in hal */

#ifdef ENABLE_ERP_PROTECTION
        if (macErpProtModeEnabled && (bOnlyStnCnt||BStnAroundCnt) && (txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
        {
            txFrame_p->wlanTxHdr.TxParam |= TX_PARAM_SEND_RTS;
        }
#endif
#endif /* PORT_COEXIST */
#ifdef FIXED_RATE
        txFrame_p->wlanTxHdr.RateIdx = fixedRate;
#endif
        if(msi_wl_RFTxData(txFrame_p) == FALSE)
            msi_wl_FreeUsrBuf(usp);
    }
    return(OS_SUCCESS);
}

extern int DoWepAndScheduleFrame(WLAN_TX_FRAME *txFrame_p)
{
    USR_BUF_DESC *usp, *usp1;
    IEEEtypes_Frame_t * Data11Frame2_p, *Data11Frame_p;
    WLAN_TX_FRAME *txFrame2_p;

    Data11Frame_p = txFrame_p->IEEEFrame;
    usp = msi_wl_GetUsrBufPtr((char *)Data11Frame_p);

    if ((Data11Frame2_p = (IEEEtypes_Frame_t *)msi_wl_AllocPktBuf(wlTxQ)) != NULL)
    {
        /* put apt values in the frame control */
        Data11Frame2_p->Hdr.FrmBodyLen = Data11Frame_p->Hdr.FrmBodyLen+8;
        DoEncrypt((UINT32)&Data11Frame2_p->Hdr, (UINT32)&Data11Frame_p->Hdr,
                  Data11Frame2_p->Hdr.FrmBodyLen - 8); /* 8 is wep overhead */
    
        memcpy(&Data11Frame2_p->Hdr, &Data11Frame_p->Hdr, sizeof(IEEEtypes_GenHdr_t));
        Data11Frame2_p->Hdr.FrmCtl.Wep = 1;
        Data11Frame2_p->Hdr.FrmBodyLen += 8;  /* 8 is wep overhead */
        usp1 = msi_wl_GetUsrBufPtr((UINT8 *)Data11Frame2_p);
        txFrame2_p = (WLAN_TX_FRAME *)usp1->info;
        txFrame2_p->IEEEFrame = Data11Frame2_p;
        memcpy(&txFrame2_p->wlanTxHdr, &txFrame_p->wlanTxHdr, sizeof(WLAN_TX_INFO));
        txsched_ScheduleFrame(txFrame2_p);
    } else
    {
        /* no buffer to do wep, drop the frame( done after else */
        Stats.DroppedCnt++;
    }
        msi_wl_FreeUsrBuf(usp);           /* free the orig one */
    return(1);
}

/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS ProcessPsPoll(UINT16 StnId, UINT16 Aid)
{                      
    WLAN_TX_FRAME *txFrame_p;
    txFrame_p = psQue_Get(StnId);
    if (txFrame_p)
    {
        eprintf("get from ps queue by pspoll, data left %d\n",isMoreDataQuedFor(StnId));
        if (isMoreDataQuedFor(StnId))
        {
            txFrame_p->IEEEFrame->Hdr.FrmCtl.MoreData = 1;
        } else
        {
            txFrame_p->IEEEFrame->Hdr.FrmCtl.MoreData = 0;
        }
        txFrame_p->wlanTxHdr.TxParam = 0; 
        if((txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
            txFrame_p->wlanTxHdr.QosControl = CBP_QID;
        else
            txFrame_p->wlanTxHdr.QosControl = BRATE_QID;
#ifdef PORT_COEXIST /* Most of these are handle in hal */

#ifdef ENABLE_ERP_PROTECTION
        if (macErpProtModeEnabled && (bOnlyStnCnt||BStnAroundCnt) && (txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
        {
            txFrame_p->wlanTxHdr.TxParam |= TX_PARAM_SEND_RTS;
        }
#endif
#endif /* PORT_COEXIST */
#ifdef FIXED_RATE
        txFrame_p->wlanTxHdr.RateIdx = fixedRate;
#endif
        if(msi_wl_RFTxData(txFrame_p) == FALSE)
            msi_wl_FreePktBuf(txFrame_p);
    }
    else
    {
        sendNullData(psStaMacAddr(StnId));
         bcngen_UpdateBitInTim(psStaAid(StnId), RESETBIT);
    }
    return(OS_SUCCESS);
}
/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
WL_STATUS ProcessPwrModeChange(extStaDb_StaInfo_t *StaInfo_p)
{
    WLAN_TX_FRAME *txFrame_p;
    eprintf("======= station %d %s =======\n", StaInfo_p->StnId,
            (StaInfo_p->PwrMode==PWR_MODE_ACTIVE)?"active":"sleep" );

    if (StaInfo_p->PwrMode == PWR_MODE_ACTIVE)
    {
        while ( txFrame_p = ((WLAN_TX_FRAME *)psQue_Get(StaInfo_p->StnId)))
        {
            if (isMoreDataQuedFor(StaInfo_p->StnId))
            {
                txFrame_p->IEEEFrame->Hdr.FrmCtl.MoreData = 1;
            } else
            {
                txFrame_p->IEEEFrame->Hdr.FrmCtl.MoreData = 0;
            }
            txFrame_p->wlanTxHdr.TxParam = 0; 
            if((txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
                txFrame_p->wlanTxHdr.QosControl = CBP_QID;
            else
                txFrame_p->wlanTxHdr.QosControl = BRATE_QID;
#ifdef PORT_COEXIST /* Most of these are handle in hal */

#ifdef ENABLE_ERP_PROTECTION
        if (macErpProtModeEnabled && (bOnlyStnCnt||BStnAroundCnt) && (txFrame_p->wlanTxHdr.RateIdx > HIGHEST_11B_RATE_REG_INDEX))
        {
            txFrame_p->wlanTxHdr.TxParam |= TX_PARAM_SEND_RTS;
        }
#endif
#endif /* PORT_COEXIST */
#ifdef FIXED_RATE
            txFrame_p->wlanTxHdr.RateIdx = fixedRate;
#endif
            if(msi_wl_RFTxData(txFrame_p) == FALSE)
                msi_wl_FreePktBuf(txFrame_p);
        }
    }
    /* Set TIM bit in the Beacon */
    if (isMoreDataQuedFor(StaInfo_p->StnId))
    {
        bcngen_UpdateBitInTim(psStaAid(StaInfo_p->StnId), SETBIT);
    } else
    {
        bcngen_UpdateBitInTim(psStaAid(StaInfo_p->StnId), RESETBIT);
    }
    psStaPwrModeSet(StaInfo_p->StnId, StaInfo_p->PwrMode);
    return(OS_SUCCESS);
}

void FreePowerSaveQueue(UINT32 StnId)
{
    USR_BUF_DESC *usp;
    WLAN_TX_FRAME *txFrame_p;

    while (txFrame_p = psQue_Get(StnId))
    {
        usp = msi_wl_GetUsrBufPtr((char *)(txFrame_p->IEEEFrame));
        msi_wl_FreeUsrBuf(usp);
    }
}
/******************************************************************************
 *
 * Name: ProcessTxDone
 *
 * Description: This function is called in response to the TX_DONE interrupt.
 *    This frees the buffer, adjusts the counters and queues another message
 *    as necessary.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
WL_STATUS ProcessTxDone(void)
{
    IEEEtypes_Frame_t *DoneMsg_p;
    /*txInfo_t *txInfo_p; */
    UINT32 stnId;
    UINT16 macQId;
    UINT32 noPkts;
    MRVL_TX_DONE_INFO doneInfo;
    /* if  more stations have to be polled, find if any
     * msg is queued for that station. If data is queued
     * form DATA+CF_POLL and queue to CFQ. otherwise
     * queue CF_POLL onto CFQ.
     * if no stations need to be polled, queue the first
     * data in the list which is does not have outstanding
     * data onto the CBQ */
    /* Process the Done message */


    noPkts = 0;
    while ( (msi_wl_RFTxDone(&doneInfo)==TRUE))
    {
        /*txInfo_p = txFrame_p->wlanTxHdr.reserved; */
#ifndef FREEINTXQUEUE
        DoneMsg_p = doneInfo.wlFrame->IEEEFrame;
        if(!doneInfo.wlFrame)
            continue;
        if ( (doneInfo.status & 0x01) !=
             IEEEtypes_STATUS_SUCCESS )
        {
            if (doneInfo.retryCnt) 
            {
                USR_BUF_DESC *usp;
                WLAN_TX_FRAME *txFrame_p;

                usp = msi_wl_GetUsrBufPtr(DoneMsg_p);

                if (!usp)
                {
                    continue;
                }

                txFrame_p = (WLAN_TX_FRAME *)usp->info;

                txFrame_p->IEEEFrame = DoneMsg_p;
#ifdef SW_FIX_TX_DONE_PROBLEM
                txFrame_p->cookie = 0x12345678;
#else                
                Stats.DataTxFailCnt++;
                if ( msi_wl_RFTxData(txFrame_p) == FALSE )
                    msi_wl_FreeUsrBuf(usp);
#endif                
                continue;

            }
        }
#endif
        stnId = doneInfo.wlFrame->wlanTxHdr.StnId;
        if(stnId >= MCBC_STN_ID)
        {
            stnId = 0;
        }
        macQId = (UINT16)(doneInfo.wlFrame->wlanTxHdr.QosControl); 

        #ifdef ENABLE_RATE_ADAPTATION_BASEBAND
        /*Get The Tx Stats */
        /*  UpdateStationTxStats(&AssocTable[StnQingInfo[stnId].Aid], txInfo_p, StnQingInfo[stnId].Aid); */
        /* Check that stnId is not 0, stnId 0 is used for management messages which are Tx at fixed rates. */
        if (stnId)
        {
            (*UpdateStationTxStatsFp)(&AssocTable[StnQingInfo[stnId].Aid], &doneInfo, StnQingInfo[stnId].Aid);
        }
        #ifdef UPDATE_TX_QUEUE

        UpdateTxQueue();
        #endif  /*UPDATE_TX_QUEUE */
        #ifdef ENABLE_RATE_ADAPTATION_ACKS
        /*To get the SQm etc values from the Ack packets */
        if ((txInfo_p->Status & 0x01) ==
                IEEEtypes_STATUS_SUCCESS)
            lastTxStnId = StnQingInfo[stnId].Aid;
        #endif /*ENABLE_RATE_ADAPTATION_ACKS */
        #endif /*ENABLE_RATE_ADAPTATION_BASEBAND */

        /* From the txInfo incr statistics */
        switch (macQId )
        {
        case TC0_QID:
        case TC1_QID:
        case TC2_QID:
        case TC3_QID:
        case TC4_QID:
        case TC5_QID:
        case TC6_QID:
        case TC7_QID:
#if !defined( FREEINTXQUEUE ) && !defined( SW_FIX_TX_DONE_PROBLEM)
            msi_wl_FreePktBuf((char *)DoneMsg_p);
#endif
            if ( !isMoreDataQuedFor(stnId) )
            {
                bcngen_UpdateBitInTim(psStaAid(stnId), RESETBIT);
            }
            break;
        case MCBC_QID:
#if !defined( FREEINTXQUEUE ) && !defined( SW_FIX_TX_DONE_PROBLEM)
            msi_wl_FreePktBuf((char *)DoneMsg_p);
#endif
            break;
        case CM_QID: 
        case BW_QID: 
        case PS_QID: 
        case CFP_QID:
            break;
        default:
            break;
        }
        ++noPkts;

    }

    {
        Stats.DoneQCnt+= noPkts;
    }

#if defined( FREEINTXQUEUE ) || defined( SW_FIX_TX_DONE_PROBLEM)
    Stats.DoneQCnt1+= freefromtxqueue();
#endif    
    WL_WRITE_WORD(MIR_INTR_STATUS, ~TX_DONE); 
    return(OS_SUCCESS);
}




