#ifndef _APCFG_H_
#define _APCFG_H_ 
/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    apcfg.h
 * \brief   Define structure of the MIB data saved on flash
 *
 * $Revision: 1.26 $
 * $Date: 2004/06/15 20:12:15 $
 */

/* Version number starts from 1 */
/* Increment the version number if mib_AP_t changes */
#if !defined(WIPOD)
#define MIB_AP_VERSION 2
#else 
#define MIB_AP_VERSION 3
#endif

/* AP configuration data structure */
typedef struct mib_AP_t
{
    UINT8 mibVersion;      /* MIB version number 1..N */
    UINT8 PrivInvoked;     /* BOOLEAN */
    UINT8 PrivOption;
    UINT8 DesiredSsId[32]; /* 32 byte string */
    UINT8 OpRateSet[126];  /* 126 byte string */
    UINT8 DtimPeriod;      /* 1 to 255 */
    /*16 bit alignment before unit16 */
    UINT16 BcnPeriod;       /* 1 to 65535 */
    UINT16 RtsThresh;       /* 0 to 2347 */
    UINT16 FragThresh;      /* 256 to 2346 */
    UINT8 RtsRetryLim;     /* 1 to 255 */
    UINT8 AuthType;
    UINT8 AuthEnable;      /* boolean */
    UINT8 CurrRFChan;      /* 0 to 14 */
    /*32 bit alignment before 32 */
    UINT8 WepType[4]; /*0=not_set, 1=40_bit, 2=104_bit */
    UINT8 WepDefaultKeyValue1[13];
    UINT8 WepDefaultKeyValue2[13];
    UINT8 WepDefaultKeyValue3[13];
    UINT8 WepDefaultKeyValue4[13];
    UINT8 DefaultWepKeyindex;
    UINT8 CurrTxPwrLevel;    /* 1 to 8 */
    /*alignment */
    UINT16 TxPwrLevel1;       /* 0 to 10000 */
    UINT16 TxPwrLevel2;       /* 0 to 10000 */
    UINT16 TxPwrLevel3;       /* 0 to 10000 */
    UINT16 TxPwrLevel4;       /* 0 to 10000 */
    UINT16 TxPwrLevel5;       /* 0 to 10000 */
    UINT16 TxPwrLevel6;       /* 0 to 10000 */
    UINT16 TxPwrLevel7;       /* 0 to 10000 */
    UINT16 TxPwrLevel8;       /* 0 to 10000 */
    UINT8 CurrTxAnt;         /* 1 to 255 */
    UINT8 CurrRxAnt;         /* 1 to 255 */
    UINT8 rxAntenna;
    UINT8 txAntenna;
    UINT8 PreAmble;
    UINT8 broadcastssid;     /* SSID blanking, true or false */
    UINT8 EnableDHCP;        /* True or False */
    UINT8 txDataRate;
    UINT8 mib_wep;           /* special mode */
    UINT8 wipodMode;         /* WiPod mode */
    /*alignment */
    UINT32 IPAddr;            /* IP address */
    UINT32 SubnetMask;        /* Subnet Mask */
    UINT32 GwyAddr;           /* Gateway address */
    UINT8 EthMACAddr[6];     /* Eth MAC addr */
    UINT8 RFMACAddr[6];      /* RF MAC addr */
    /*alignment */
    UINT8 username[32];      /* User name */
    UINT8 password[32];      /* password */
    UINT8 devName[16];
    UINT8 apmode;
    /* For WB31 */
    UINT8 opMode;            // 0 for infrastructure, 1 for ad-hoc
    UINT8 macCloneEnable;    // boolean
    /* For WB31 end */
    UINT8 RSNEnabled;        //WPA mode
    UINT32 GrpRekeyTime;      //GroupRekeyTime
#ifdef OS_VxW
    /* Added for vxWorks version of ap32 - Lan and Wan ports. */
    UINT32 LanIPAddr;            /* Lan IP address */
    UINT32 LanSubnetMask;        /* Lan Subnet Mask */
    UINT32 LanGwyAddr;           /* Lan Gateway address */
    
    UINT32 WanIPAddr;            /* WAN IP address */
    UINT32 WanSubnetMask;        /* WAN Subnet Mask */
    UINT32 WanGwyAddr;           /* WAN Gateway address */
#endif    
//    UINT8 reserved1[5];
#if defined(WIPOD)
    /* The maximum size of this structure is fixed at 512 bytes. 
    Note that changing this structure size will affect many. */
    UINT8               mib_WPA_passphrase[64];
    UINT8               mib_WPA_encr_algo;
    UINT16              mib_WPA_cfgkey;              
    UINT8 pad[77];
#endif
}
PACK_END mib_AP_t;  /*total 320 bytes */
#ifdef WB_EXCEPTION
/* mfg data struct*/
#define SZ_BOARD_NAME       8
#define SZ_BOOT_VERSION     12
#define SZ_PRODUCT_ID       58
#define SZ_INTERNAL_PA_CFG  14
#define SZ_EXTERNAL_PA_CFG  1
#define SZ_CCA_CFG          8
#define SZ_LED              4
#ifdef AP_PGRINE
typedef struct PaOpt_t_wb
{
   UINT8  IntPaCalTabOpt:1;  /* 0 = Delta Method, 1 = Absolute Method */
   UINT8  ant2NotCalSep:1;
   UINT8  ExtPaNegPol:1;     /* Ext PA polarization. 0 = Positive, 1 = Negative */
   UINT8  ExtPaUsed:1;
   UINT8  Reserved:4;
} PACK_END PaOpt_t_wb;

typedef struct Ant_t_wb
{
   UINT8   Tx:2;        /* AntDivModeTx[7:6](BBP:0x38),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Rx:2;        /* AntDivModeRx[7:6](BBP:0x37),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Reserved:4;
} PACK_END Ant_t_wb;

typedef struct Delta_t_wb
{
   UINT8 CalVal_C0;
   UINT8 CalVal_C1;
} PACK_END Delta_t_wb;

typedef struct Absolute_t_wb
{
   UINT8 Reg_1C;
   UINT8 Reg_1B;
} PACK_END Absolute_t_wb;

typedef union Pa_t_wb
{
   Delta_t_wb    Delta;
   Absolute_t_wb Absolute;
} PACK_END Pa_t_wb;

typedef struct PaConfig_t_wb
{
   UINT32 Reg_1E:8;
   UINT32 Reg_1F:8;
   UINT32 Reg_20:8;
   UINT32 Reg_1A:6;
   UINT32 Reg_1D_0:1;
   UINT32 _2CalValTbl:1;
} PACK_END PaConfig_t_wb;

typedef struct AssmSN_t_wb
{
   UINT32 AssmCode:8;
   UINT32 SerialNum:24;
} PACK_END AssmSN_t_wb;

typedef struct LCTCal_t_wb
{
   UINT8 LctCalCh11:3;
   UINT8 LctCalCh4:3;
   UINT8 Valid0:1;
   UINT8 Valid1:1;	
}PACK_END LCTCal_t_wb; 
#endif
typedef struct mfg_data_t
{
    UINT8 BrdDscrpt[8];      // 8 byte ASCII to descript the type and version of the board
    UINT8 Rev;
#ifdef AP_PGRINE
/* To define Peregrine and Falcon structure is different. To add auto-detect RF Chip function need to 
   modify MFG_CAL_DATA. We define AUTO_DETECT_MFG to discriminate auto-detect RF MFG function */
#ifdef AUTO_DETECT_MFG
    union {
        struct mfg_peregrine_s_wb {
            PaOpt_t_wb      PaOpt_p;
            uint8           ExtPA_p[SZ_EXTERNAL_PA_CFG];
            Ant_t_wb        Ant_p;
            Pa_t_wb         Pa_p[SZ_INTERNAL_PA_CFG];;
            PaConfig_t_wb   PaConfig_p;
            AssmSN_t_wb     AssmSN_p;
            uint16          Domain_p;            /* Wireless domain */
            uint8	        ECO_p;
            LCTCal_t_wb     LCT_cal_p;
            uint8           SocVer_p;
            uint8           RFVer_p;
            uint8	        SPISize_p;
            uint8    	    AntPart_p;
            uint16          TestVer_p;
            uint16          DllVer_p;
            uint16          Xosc_p;
        } PACK_END mfg_peregrine;
        struct mfg_falcon_s_wb {
            uint8 PAOptions_f;
            uint8 ExtPA_f[SZ_EXTERNAL_PA_CFG];
            uint8 Ant_f;
            uint16 IntPA_f[SZ_INTERNAL_PA_CFG];
            uint8 CCA_f[SZ_CCA_CFG];
            uint16 Domain_f;            // Wireless domain
            uint16 CstmrOpts_f;
            uint8 LED_f[SZ_LED];
            UINT16 Xosc_f;
            UINT8 Reserved_1_f[2];
            uint16 Magic_f;
        } PACK_END mfg_falcon;
    } PACK_END mfg_u;
    uint16 ChkSum;
#else /* ! AUTO_DETECT_MFG */
    PaOpt_t_wb      PaOpt;
    UINT8           ExtPA[SZ_EXTERNAL_PA_CFG];
    Ant_t_wb        Ant;
    Pa_t_wb         Pa[SZ_INTERNAL_PA_CFG];;
    PaConfig_t_wb   PaConfig;
    AssmSN_t_wb     AssmSN;
    UINT16          Domain;            /* Wireless domain */
    UINT8	        ECO;
    LCTCal_t_wb     LCT_cal;
    UINT8           SocVer;
    UINT8           RFVer;
    UINT8	        SPISize;
    UINT8    	    AntPart;
    UINT16          TestVer;
    UINT16          DllVer;
    UINT16          Xosc;
    UINT16          ChkSum;
#endif /* AUTO_DETECT_MFG */
#else /* ! AP_PGRINE */
    UINT8 PAOptions;
    UINT8 ExtPA;
    UINT8 Ant;
    UINT16 IntPA[14];
    UINT8 CCA[8];
    UINT16 Domain;            // Wireless domain
    UINT16 CstmrOpts;
    UINT8 Reserved_1[8];
    UINT16 Magic;
    UINT16 ChkSum;
#endif /* AP_PGRINE */
    UINT8 MfgMacAddr[6];     // Mfg mac address
    UINT8 Reserved_2[4];
    UINT8 PID[58];           // Production ID
    UINT8 BootVersion[12];
}
PACK_END mfg_data_t;

#ifdef AP_PGRINE
#ifdef AUTO_DETECT_MFG
#define PAOptions   mfg_u.mfg_falcon.PAOptions_f
#define ExtPA       mfg_u.mfg_falcon.ExtPA_f
#define Ant         mfg_u.mfg_falcon.Ant_f
#define IntPA       mfg_u.mfg_falcon.IntPA_f
#define CCA         mfg_u.mfg_falcon.CCA_f
#define Domain      mfg_u.mfg_falcon.Domain_f
#define CstmrOpts   mfg_u.mfg_falcon.CstmrOpts_f
#define LED         mfg_u.mfg_falcon.LED_f
#define Xosc        mfg_u.mfg_falcon.Xosc_f
#define Reserved_1  mfg_u.mfg_falcon.Reserved_1_f
#define Magic       mfg_u.mfg_falcon.Magic_f

#define PaOpt       mfg_u.mfg_peregrine.PaOpt_p       
#define ExtPAp      mfg_u.mfg_peregrine.ExtPA_p       
#define Antp        mfg_u.mfg_peregrine.Ant_p         
#define Pa          mfg_u.mfg_peregrine.Pa_p          
#define PaConfig    mfg_u.mfg_peregrine.PaConfig_p    
#define AssmSN      mfg_u.mfg_peregrine.AssmSN_p      
#define Domainp     mfg_u.mfg_peregrine.Domain_p      
#define ECO         mfg_u.mfg_peregrine.ECO_p         
#define LCT_cal     mfg_u.mfg_peregrine.LCT_cal_p     
#define SocVer      mfg_u.mfg_peregrine.SocVer_p      
#define RFVer       mfg_u.mfg_peregrine.RFVer_p       
#define SPISize     mfg_u.mfg_peregrine.SPISize_p     
#define AntPart     mfg_u.mfg_peregrine.AntPart_p     
#define TestVer     mfg_u.mfg_peregrine.TestVer_p     
#define DllVer      mfg_u.mfg_peregrine.DllVer_p      
#define Xoscp       mfg_u.mfg_peregrine.Xosc_p
#endif
#endif
#endif /* WB_EXCEPTION */

typedef struct mib_user_AP_t
{
	mib_AP_t	        mib_ap;
#if defined(WIPOD)
    mib_AP_t            mib_wb;
#endif
	UINT32				mib_wlanfilter_cfgkey;
	UINT8				mib_wlanfilter_no;
	UINT8				mib_wlanfilter_mode;
	UINT8				mib_wlanfilter_mac[ 256 * 6];
    UINT8               mib_WPA_passphrase[64];
    UINT8               mib_WPA_encr_algo;
    UINT16              mib_WPA_cfgkey;              
    UINT8               mib_forceProtectiondisable;
    UINT8               mib_enableFixedRateTx;
    UINT8               mib_txDataRate;  /* fixed rate to be used for b only clients */
    UINT8               mib_txDataRateG; /* fixed rate to be used for g clients */
    UINT8               mib_WPA2Enabled;
    UINT8               reserved0[2];
#ifdef WDS_FEATURE
    UINT32              mib_wds_cfgkey;
    UINT8               mib_wds_Enable; /*0: disable 1: static wds only 2: dynamical wds */
    UINT8               mib_wds_no;
    UINT8               mib_wds_mac[32 * 6]; /* MAX_WDS_PORT*/
    UINT8               mib_disable_assoc; /* TRUE = disable all clients assoc */
    UINT8               mib_wds_option; /* WDS Interop option: 0 = broadcast; 1 = unicast; TBD */
#endif /* WDS_FEATURE*/    
#ifdef BURST_MODE
    UINT32              mib_burstrate;
    UINT8               mib_burstmode;
#endif
#ifdef SELF_CTS
    UINT8               mib_SelfCts;
#endif
#ifdef TURBO_SETUP
    UINT8               mib_TSMode; /* 0x00 or 0xff default factory settings, 0x02 Turbo Setup, 0x04 Web GUI*/
#endif
}
PACK_END mib_user_AP_t;  /*it may be within 0xfffe0400 to 0xffff0000*/
/* Make sizeof(mib_AP_t) 16-bit aligned to simplify writing to flash */
#define SIZEOF_MIB_AP (((sizeof(mib_AP_t)+1)/2)*2)
#define SIZEOF_MIB_USER_AP		(((sizeof(mib_user_AP_t)+1)/2)*2)

typedef struct _LOGIN
{
    unsigned char username[32];
    unsigned char password[32];
}
LOGIN;

extern LOGIN login;
#endif /* _APCFG_H_ */
