/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    wins.c
 * \brief   A Simple Wins Client that listen on port 137 for NetBios Name Query
 *          If name Query match its name, response if IP Address information.
 *
 * Author(s):    Binh Tran 
 * Date:         2003-03-25
 *
 * $Revision: 1.3 $
 * $Date: 2003/09/23 00:24:48 $
 */

#include  <network.h>
#include <netinet/in.h>

#include "wins.h"

#include "wltypes.h" 
#include "etherlow.h"

#define sysinfo_TASK_NAME  "winsClient"
#define sysinfo_TASK_STACK_SIZE    8192
#define sysinfo_TASK_PRI         20
static cyg_thread       Task;
static cyg_handle_t     TaskHdl;
static UINT8            Stack[sysinfo_TASK_STACK_SIZE]__attribute__ ((section (".itcmbuf")));

static void winsTask( OS_DATA Data );


/****************** Global Declaration ****************/
/*
 * IP Version 4 socket address.
 */


int sock = -1;
struct sockaddr_in sockAddr;


/****************** Global Declaration End ***********/


/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern WL_STATUS winsInit( void ) 
{ 
    cyg_thread_create(sysinfo_TASK_PRI,
                      winsTask,
                      0,
                      sysinfo_TASK_NAME,
                      (void *)(Stack),
                      sysinfo_TASK_STACK_SIZE,
                      &TaskHdl,
                      &Task);
#if 0
    if (status = os_TaskCreate(sysinfo_WINS_TASK, 
                               winsTask, 
                               (OS_DATA)0, 
                               sysinfo_WINS_TASK_NAME, 
                               sysinfo_WINS_TASK_STACK_SIZE, 
                               sysinfo_WINS_TASK_PRI) == OS_SUCCESS)
    {
        return(OS_SUCCESS); 
    }
    return(OS_FAIL); 
#endif
	return(OS_SUCCESS); 
} 

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern void winxStart( void ) 
{ 

    cyg_thread_resume(TaskHdl); 
    /* os_TaskResume(sysinfo_WINS_TASK); */ 

} 


/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static int winsDecodeData(unsigned char *inData, int lenData, unsigned char *outData) 
{ 
    int i;
    int lenResult = 0;
    unsigned char outChar;


    if ((inData == NULL) || (outData == NULL) || (lenData % 2))
    {
        return -1;
    }

    for (i = 1; i < lenData; i+=2)
    {
        if (*(inData + i) != 0)
        {
            outChar = ((*(inData + i) - 0x41) << 4) | (*(inData +i +1) - 0x41);
            if (outChar == 0x20)
            {
                *(outData + lenResult) = 0x00;
                return lenResult;
            }
            *(outData + lenResult) = outChar;
            lenResult ++;
        }
    }

    return lenResult;
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static int winsVerifyName(unsigned char * qName) 
{ 
    int rst;

    if (qName == NULL)
    {
        return -1;
    }

    if ((rst = strcmp(qName, winsGetDevName())))
    {
        return -1;
    }

    return 0;
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static int winsSendTo(void *pack, int size, struct sockaddr *toAddr, int lenAddr)
{ 
    int sndResult;

    sndResult = sendto(sock, pack, size, 0, toAddr, lenAddr);
    if (sndResult != size)
    {
        return -1;
    }
    return 0;
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static int winsMakeRspPack(nBiosNameSrv *recvPack, unsigned char *sndPack) 
{ 
    nBiosNameSrvOut * answPack;
    unsigned long ipAddr;
    int lenPack;


    if ((recvPack == NULL) || (sndPack == NULL))
    {
        return -1;
    }
    answPack = (nBiosNameSrvOut *)sndPack;

    answPack->transId = recvPack->transId;
    answPack->flag = 0x8085;
    answPack->question = 0x0000;
    answPack->answer_rrs = 0x0100;
    answPack->auth_rrs = 0x0000;
    answPack->add_rrs = 0x0000;
    lenPack = NAME_SRV_HDR_LEN;

    memcpy((unsigned char *)&answPack->answer.query, (unsigned char *)&recvPack->query, sizeof(nameSrvQuery));
    lenPack += sizeof(nameSrvQuery);

    answPack->answer.ttl = 0x10;
    lenPack += 4;
    answPack->answer.lenData = htons(6);
    lenPack+= 2;
    answPack->answer.answFlag = 0x0040;
    lenPack += 2;

    ipAddr = winsGetEthIp();
    memcpy(answPack->answer.data, &ipAddr, 4);
    lenPack += 4;

    lenPack +=2;

    return lenPack;
}


/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static int winsProcessPack(nBiosNameSrv *pack, struct sockaddr_in *fromAddr, int lenAddr) 
{ 
    unsigned char sndPack[RD_BUF_MAX_LEN];
    unsigned char namebuf[WINS_NAME_BUF_MAX_LEN];
    int lenName;
    int lenPack;

    if (pack == NULL)
    {
        return -1;
    }

    if (ntohs(pack->question) != 1)
    {
        return -1;
    }

    if ((lenName = winsDecodeData(pack->query.name, WINS_NAME_BUF_MAX_LEN, namebuf)) < 1)
    {
        return -1;
    }

    if (winsVerifyName(namebuf) == 0)
    {
        memset(sndPack, 0, RD_BUF_MAX_LEN);
        lenPack = winsMakeRspPack(pack, sndPack);
        winsSendTo(sndPack, lenPack, (struct sockaddr *)fromAddr, lenAddr);
    }

    return 0;
}


/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
static void winsTask( OS_DATA Data ) 
{ 
    char bufPack[RD_BUF_MAX_LEN];
    int lenPack;
    struct sockaddr_in fromAddr;
    int lenFrom;

    char testBuf[RD_BUF_MAX_LEN];
    int retval;
    wait_ipReady();
    /*open a udp socket */
    if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        return; 
    }

    /* fill the address structure */
    memset(&sockAddr, 0, sizeof(sockAddr));
    sockAddr.sin_family = AF_INET;
    sockAddr.sin_port = htons(WINS_PORT);
    sockAddr.sin_addr.s_addr = htonl(INADDR_ANY);

    /* bind the socket */
    retval = bind(sock, (struct sockaddr*)&sockAddr, sizeof(sockAddr));
    if (retval == -1)
    {
        return;
    }

    while (1)
    {

        lenFrom = sizeof(struct sockaddr_in);
        lenPack = recvfrom(sock, bufPack, RD_BUF_MAX_LEN, 0, (struct sockaddr*)&fromAddr, &lenFrom);
        if (lenPack == -1)
        {
            continue;
        }

        memcpy(testBuf, bufPack, lenPack);
        winsProcessPack(bufPack, &fromAddr, lenFrom);

    }

    return;
}

