/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
 *
 * $Header: E:\\APCVSDEPOT/wlan/wlandrv/incl/IEEE_types.h,v 1.25 2004/05/26 18:16:03 binht Exp $
 *
 * Purpose:
 *    This file contains definitions relating to messages specified in the
 *    IEEE 802.11 spec.
 *
 * Public Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#ifndef _IEEE_TYPES_H_
#define _IEEE_TYPES_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"

/*============================================================================= */
/*                            PUBLIC DEFINITIONS */
/*============================================================================= */

/*--------------------------------------------------------------*/
/* Reason Codes - these codes are used in management message    */
/* frame bodies to indicate why an action is taking place (such */
/* as a disassociation or deauthentication).                    */
/*--------------------------------------------------------------*/
#define IEEEtypes_REASON_RSVD                    0
#define IEEEtypes_REASON_UNSPEC                  1
#define IEEEtypes_REASON_PRIOR_AUTH_INVALID      2
#define IEEEtypes_REASON_DEAUTH_LEAVING          3
#define IEEEtypes_REASON_DISASSOC_INACTIVE       4
#define IEEEtypes_REASON_DISASSOC_AP_BUSY        5
#define IEEEtypes_REASON_CLASS2_NONAUTH          6
#define IEEEtypes_REASON_CLASS3_NONASSOC         7
#define IEEEtypes_REASON_DISASSOC_STA_HASLEFT    8
#define IEEEtypes_REASON_CANT_ASSOC_NONAUTH      9
/***************WPA Reasons*******************/
#define IEEEtypes_REASON_INVALID_IE              13
#define IEEEtypes_REASON_MIC_FAILURE             14
#define IEEEtypes_REASON_4WAY_HANDSHK_TIMEOUT    15
#define IEEEtypes_REASON_GRP_KEY_UPD_TIMEOUT     16
#define IEEEtypes_REASON_IE_4WAY_DIFF            17
#define IEEEtypes_REASON_INVALID_MCAST_CIPHER    18
#define IEEEtypes_REASON_INVALID_UNICAST_CIPHER  19
#define IEEEtypes_REASON_INVALID_AKMP            20
#define IEEEtypes_REASON_UNSUPT_RSNE_VER         21
#define IEEEtypes_REASON_INVALID_RSNE_CAP        22
#define IEEEtypes_REASON_8021X_AUTH_FAIL         23
/*********************************************/

/*------------------------------------------------------------*/
/* Status Codes - these codes are used in management message  */
/* frame bodies to indicate the results of an operation (such */
/* as association, reassociation, and authentication).        */
/*------------------------------------------------------------*/
#define IEEEtypes_STATUS_SUCCESS                 0
#define IEEEtypes_STATUS_UNSPEC_FAILURE          1
#define IEEEtypes_STATUS_CAPS_UNSUPPORTED        10
#define IEEEtypes_STATUS_REASSOC_NO_ASSOC        11
#define IEEEtypes_STATUS_ASSOC_DENIED_UNSPEC     12
#define IEEEtypes_STATUS_UNSUPPORTED_AUTHALG     13
#define IEEEtypes_STATUS_RX_AUTH_NOSEQ           14
#define IEEEtypes_STATUS_CHALLENGE_FAIL          15
#define IEEEtypes_STATUS_AUTH_TIMEOUT            16
#define IEEEtypes_STATUS_ASSOC_DENIED_BUSY       17
#define IEEEtypes_STATUS_ASSOC_DENIED_RATES      18

/* */
/* 802.11b additions */
/* */
#define IEEEtypes_STATUS_ASSOC_DENIED_NOSHORT    19
#define IEEEtypes_STATUS_ASSOC_DENIED_NOPBCC     20
#define IEEEtypes_STATUS_ASSOC_DENIED_NOAGILITY  21
/* */
/* 802.11g additions */
/* */
#define IEEEtypes_STATUS_ASSOC_DENIED_NOSHORTSLOTTIME 25
#define IEEEtypes_STATUS_ASSOC_DENIED_NODSSSOFDM      26
/*--------------------------------------------*/
/* Various sizes used in IEEE 802.11 messages */
/*--------------------------------------------*/
#define IEEEtypes_ADDRESS_SIZE         6
#define IEEEtypes_BITMAP_SIZE          251
#define IEEEtypes_CHALLENGE_TEXT_SIZE  253
#define IEEEtypes_CHALLENGE_TEXT_LEN   128
#define IEEEtypes_MAX_DATA_RATES       8
#define IEEEtypes_MAX_DATA_BODY_LEN    2312
#define IEEEtypes_MAX_MGMT_BODY_LEN    2312
#define IEEEtypes_SSID_SIZE            32
#define IEEEtypes_TIME_STAMP_SIZE      8
#define IEEEtypes_MAX_CHANNELS         14
#define IEEEtypes_MAX_BSS_DESCRIPTS    16
#define IEEEtypes_MAX_DATA_RATES_G     14
/*---------------------------------------------------------------------*/
/* Define masks used to extract fields from the capability information */
/* structure in a beacon message.                                      */
/*---------------------------------------------------------------------*/
#define IEEEtypes_CAP_INFO_ESS             1
#define IEEEtypes_CAP_INFO_IBSS            2
#define IEEEtypes_CAP_INFO_CF_POLLABLE     4
#define IEEEtypes_CAP_INFO_CF_POLL_RQST    8
#define IEEEtypes_CAP_INFO_PRIVACY         16
#define IEEEtypes_CAP_INFO_SHORT_PREAMB    32
#define IEEEtypes_CAP_INFO_PBCC            64
#define IEEEtypes_CAP_INFO_CHANGE_AGILITY  128
#define IEEEtypes_CAP_INFO_SHORT_SLOT_TIME  0x0400
#define IEEEtypes_CAP_INFO_DSSS_OFDM        0x2000

/*---------------------------*/
/* Miscellaneous definitions */
/*---------------------------*/
#define IEEEtypes_PROTOCOL_VERSION     0

#define IEEEtypes_BASIC_RATE_FLAG 0x80
   /* */
   /* Used to determine which rates in a list are designated as basic rates */
   /* */

#define IEEEtypes_BASIC_RATE_MASK 0x7F
   /* */
   /* Used to mask off the basic rate flag, if one exists, for given */
   /* data rates */
   /* */

#define IEEEtypes_RATE_MIN 2
   /* */
   /* The minimum allowable data rate in units of kb/s */
   /* */
   
#define IEEEtypes_RATE_MAX 127
   /* */
   /* The maximum allowable data rate in units of kb/s */
   /* */
   
#define IEEEtypes_TIME_UNIT 1024

#define IS_GROUP(macaddr)  ((*(UINT8*)macaddr & 0x01) == 0x01) 
  
   /* */
   /* The number of microseconds in 1 time unit, as specified in the */
   /* 802.11 spec */
   /* */
   

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */

/*---------------------------------------------------------------------------*/
/*                 Enumerated types used in 802.11 messages                  */
/*---------------------------------------------------------------------------*/
typedef enum
{
   IEEE_TYPE_MANAGEMENT = 0,
   IEEE_TYPE_CONTROL,
   IEEE_TYPE_DATA
} IEEEtypes_MsgType_e;
   /* */
   /* The 3 possible types of 802.11 messages */
   /* */

typedef enum
{
   IEEE_MSG_ASSOCIATE_RQST = 0,
   IEEE_MSG_ASSOCIATE_RSP,
   IEEE_MSG_REASSOCIATE_RQST,
   IEEE_MSG_REASSOCIATE_RSP,
   IEEE_MSG_PROBE_RQST,
   IEEE_MSG_PROBE_RSP,
   IEEE_MSG_BEACON = 8,
   IEEE_MSG_ATIM,
   IEEE_MSG_DISASSOCIATE,
   IEEE_MSG_AUTHENTICATE,
   IEEE_MSG_DEAUTHENTICATE
} IEEEtypes_MgmtSubType_e;
   /* */
   /* The possible types of management messages */
   /* */

typedef enum
{
   PS_POLL = 10,
   RTS,
   CTS,
   ACK,
   CF_END,
   CF_END_CF_ACK
} IEEEtypes_CtlSubType_e;
   /* */
   /* The possible types of control messages */
   /* */

typedef enum
{
   DATA = 0,
   DATA_CF_ACK,
   DATA_CF_POLL,
   DATA_CF_ACK_CF_POLL,
   NULL_DATA,
   CF_ACK,
   CF_POLL,
   CF_ACK_CF_POLL
} IEEEtypes_DataSubType_e;
   /* */
   /* The possible types of data messages */
   /* */

typedef enum
{
   SME_CMD_ASSOCIATE,
   SME_CMD_AUTHENTICATE,
   SME_CMD_DEAUTHENTICATE,
   SME_CMD_DISASSOCIATE,
   SME_CMD_JOIN,
   SME_CMD_REASSOCIATE,
   SME_CMD_RESET,
   SME_CMD_SCAN,
   SME_CMD_START
} IEEEtypes_SmeCmd_e;
   /* */
   /* The possible types of commands sent from the SME */
   /* */

typedef enum
{
   SME_NOTIFY_PWR_MGMT_CFRM,
   SME_NOTIFY_SCAN_CFRM,
   SME_NOTIFY_JOIN_CFRM,
   SME_NOTIFY_AUTH_CFRM,
   SME_NOTIFY_AUTH_IND,
   SME_NOTIFY_DEAUTH_CFRM,
   SME_NOTIFY_DEAUTH_IND,
   SME_NOTIFY_ASSOC_CFRM,
   SME_NOTIFY_ASSOC_IND,
   SME_NOTIFY_REASSOC_CFRM,
   SME_NOTIFY_REASSOC_IND,
   SME_NOTIFY_DISASSOC_CFRM,
   SME_NOTIFY_DISASSOC_IND,
   SME_NOTIFY_RESET_CFRM,
   SME_NOTIFY_START_CFRM
} IEEEtypes_SmeNotify_e;
   /* */
   /* The possible types of notifications sent from the SME */
   /* */

typedef enum
{
   CBP_CMD_ASSOCIATE = SME_CMD_ASSOCIATE,
   CBP_CMD_AUTHENTICATE = SME_CMD_AUTHENTICATE,
   CBP_CMD_DEAUTHENTICATE = SME_CMD_DEAUTHENTICATE,
   CBP_CMD_DISASSOCIATE = SME_CMD_DISASSOCIATE,
   CBP_CMD_JOIN = SME_CMD_JOIN,
   CBP_CMD_REASSOCIATE = SME_CMD_REASSOCIATE,
   CBP_CMD_RESET = SME_CMD_RESET,
   CBP_CMD_SCAN = SME_CMD_SCAN,
   CBP_CMD_START = SME_CMD_START,
   CBP_CMD_MIB_READ,
   CBP_CMD_MIB_WRITE,
   CBP_CMD_REGISTER_READ,
   CBP_CMD_REGISTER_WRITE,
   CBP_CMD_SIGNAL_STRENGTH,
   CBP_CMD_STATION_INFO_READ,
   CBP_CMD_STATION_INFO_WRITE,
   CBP_CMD_STATION_LIST,
   CBP_CMD_STATISTICS
} IEEEtypes_CbProcessorCmd_e;
   /* */
   /* The possible types of commands sent from the CB Processor */
   /* */

typedef enum
{
   BSS_INFRASTRUCTURE = 1,
   BSS_INDEPENDENT,
   BSS_ANY
} IEEEtypes_Bss_e;
   /* */
   /* The possible types of Basic Service Sets */
   /* */

typedef enum
{
   SSID = 0,
   SUPPORTED_RATES,
   FH_PARAM_SET,
   DS_PARAM_SET,
   CF_PARAM_SET,
   TIM,
   IBSS_PARAM_SET,
   CHALLENGE_TEXT = 16,
   ERP_INFO = 42,
   EXT_SUPPORTED_RATES = 50,
   RSN_IE = 221,
   RSN_IEWPA2 = 48
} IEEEtypes_ElementId_e;
   /* */
   /* Variable length mandatory fields or optional frame body components */
   /* within management messages are called information elements; these */
   /* elements all have associated with them an Element ID. IDs 7 to 15 */
   /* are reserved; IDs 17 to 31 are reserved for challenge text; IDs */
   /* 32 to 255 are reserved. */
   /* */

typedef enum
{
   PWR_MODE_ACTIVE = 1,
   PWR_MODE_PWR_SAVE
} IEEEtypes_PwrMgmtMode_e;
   /* */
   /* The possible power management modes */
   /* */

typedef enum
{
   SCAN_ACTIVE,
   SCAN_PASSIVE
} IEEEtypes_Scan_e;
   /* */
   /* The possible methods to scan for APs */
   /* */

typedef enum
{
   AUTH_OPEN_SYSTEM = 0,
   AUTH_SHARED_KEY,
   AUTH_NOT_SUPPORTED
} IEEEtypes_AuthType_e;
   /* */
   /* The possible types of authentication */
   /* */

typedef enum
{
   SCAN_RESULT_SUCCESS,
   SCAN_RESULT_INVALID_PARAMETERS
} IEEEtypes_ScanResult_e;
   /* */
   /* The possible responses to a request to scan */
   /* */

typedef enum
{
   JOIN_RESULT_SUCCESS,
   JOIN_RESULT_INVALID_PARAMETERS,
   JOIN_RESULT_TIMEOUT
} IEEEtypes_JoinResult_e;
   /* */
   /* The possible responses to a request to join a BSS */
   /* */

typedef enum
{
   AUTH_RESULT_SUCCESS,
   AUTH_RESULT_INVALID_PARAMETERS,
   AUTH_RESULT_TIMEOUT,
   AUTH_RESULT_TOO_MANY_SIMULTANEOUS_RQSTS,
   AUTH_RESULT_REFUSED
} IEEEtypes_AuthResult_e;
   /* */
   /* The possible results to a request to authenticate */
   /* */

typedef enum
{
   DEAUTH_RESULT_SUCCESS,
   DEAUTH_RESULT_INVALID_PARAMETERS,
   DEAUTH_RESULT_TOO_MANY_SIMULTANEOUS_RQSTS
} IEEEtypes_DeauthResult_e;
   /* */
   /* The possible results to a request to deauthenticate */
   /* */

typedef enum
{
   ASSOC_RESULT_SUCCESS,
   ASSOC_RESULT_INVALID_PARAMETERS,
   ASSOC_RESULT_TIMEOUT,
   ASSOC_RESULT_REFUSED
} IEEEtypes_AssocResult_e;
   /* */
   /* The possible results to a request to associate */
   /* */

typedef enum
{
   REASSOC_RESULT_SUCCESS,
   REASSOC_RESULT_INVALID_PARAMETERS,
   REASSOC_RESULT_TIMEOUT,
   REASSOC_RESULT_REFUSED
} IEEEtypes_ReassocResult_e;
   /* */
   /* The possible results to a request to reassociate */
   /* */

typedef enum
{
   DISASSOC_RESULT_SUCCESS,
   DISASSOC_RESULT_INVALID_PARAMETERS,
   DISASSOC_RESULT_TIMEOUT,
   DISASSOC_RESULT_REFUSED
} IEEEtypes_DisassocResult_e;
   /* */
   /* The possible results to a request to disassociate */
   /* */

typedef enum
{
   PWR_MGMT_RESULT_SUCCESS,
   PWR_MGMT_RESULT_INVALID_PARAMETERS,
   PWR_MGMT_RESULT_NOT_SUPPORTED
} IEEEtypes_PwrMgmtResult_e;
   /* */
   /* The possible results to a request to change the power management mode */
   /* */

typedef enum
{
   RESET_RESULT_SUCCESS
} IEEEtypes_ResetResult_e;
   /* */
   /* The possible results to a request to reset */
   /* */

typedef enum
{
   START_RESULT_SUCCESS,
   START_RESULT_INVALID_PARAMETERS,
   START_RESULT_BSS_ALREADY_STARTED_OR_JOINED
} IEEEtypes_StartResult_e;
   /* */
   /* The possible results to a request to start */
   /* */

typedef enum
{
   STATE_IDLE,
   STATE_SCANNING,
   STATE_JOINING,
   STATE_JOINED,
   STATE_AUTHENTICATED_WITH_AP,
   STATE_ASSOCIATING,
   STATE_ASSOCIATED,
   STATE_REASSOCIATING,
   STATE_RESTORING_FROM_SCAN,
   STATE_IBSS_STARTED,
} IEEEtypes_MacMgmtStates_e;

   /* */
   /* The possible states the MAC Management Service Task can be in */
   /* */


/*---------------------------------------------------------------------------*/
/*           Types Used In IEEE 802.11 MAC Message Data Structures           */
/*---------------------------------------------------------------------------*/
typedef UINT8 IEEEtypes_Len_t;
   /* */
   /* Length type */
   /* */

typedef UINT8 IEEEtypes_Addr_t;
   /* */
   /* Address type */
   /* */

typedef IEEEtypes_Addr_t IEEEtypes_MacAddr_t[IEEEtypes_ADDRESS_SIZE];
   /* */
   /* MAC address type */
   /* */

typedef UINT8 IEEEtypes_DataRate_t;
   /* */
   /* Type used to specify the supported data rates */
   /* */

typedef UINT8 IEEEtypes_SsId_t[IEEEtypes_SSID_SIZE];
   /* */
   /* SS ID type */
   /* */

/*---------------------------------------------------------------------------*/
/*                 IEEE 802.11 MAC Message Data Structures                   */
/*                                                                           */
/* Each IEEE 802.11 MAC message includes a MAC header, a frame body (which   */
/* can be empty), and a frame check sequence field. This section gives the   */
/* structures that used for the MAC message headers and frame bodies that    */
/* can exist in the three types of MAC messages - 1) Control messages,       */
/* 2) Data messages, and 3) Management messages.                             */
/*---------------------------------------------------------------------------*/
typedef PACK_START struct IEEEtypes_FrameCtl_t
{
   UINT16  ProtocolVersion:2;
   UINT16  Type:2;
   UINT16  Subtype:4;
   UINT16  ToDs:1;
   UINT16  FromDs:1;
   UINT16  MoreFrag:1;
   UINT16  Retry:1;
   UINT16  PwrMgmt:1;
   UINT16  MoreData:1;
   UINT16  Wep:1;
   UINT16  Order:1;
} PACK_END IEEEtypes_FrameCtl_t;
   /* */
   /* The frame control field in the header of a MAC message */
   /* */

typedef PACK_START struct IEEEtypes_GenHdr_t
{
   UINT16                FrmBodyLen;
   IEEEtypes_FrameCtl_t  FrmCtl;
   UINT16                DurationId;
   IEEEtypes_MacAddr_t   Addr1;
   IEEEtypes_MacAddr_t   Addr2;
   IEEEtypes_MacAddr_t   Addr3;
   UINT16                SeqCtl;
   IEEEtypes_MacAddr_t   Addr4;
} PACK_END IEEEtypes_GenHdr_t;
   /* */
   /* The general header for MAC messages */
   /* */

typedef PACK_START struct IEEEtypes_MgmtHdr_t
{
   UINT16                FrmBodyLen;
   IEEEtypes_FrameCtl_t  FrmCtl;
   UINT16                Duration;
   IEEEtypes_MacAddr_t   DestAddr;
   IEEEtypes_MacAddr_t   SrcAddr;
   IEEEtypes_MacAddr_t   BssId;
   UINT16                SeqCtl;
   IEEEtypes_MacAddr_t   Rsrvd;
} PACK_END IEEEtypes_MgmtHdr_t;
   /* */
   /* The header for MAC management messages */
   /* */

typedef PACK_START struct IEEEtypes_PsPollHdr_t
{
   UINT16                FrmBodyLen;
   IEEEtypes_FrameCtl_t  FrmCtl;
   UINT16                Aid;
   IEEEtypes_MacAddr_t   BssId;
   IEEEtypes_MacAddr_t   TxAddr;
} PACK_END IEEEtypes_PsPollHdr_t;
   /* */
   /* The header for power-save poll messages (the only control message */
   /* processed by the MAC software) */
   /* */

typedef PACK_START struct IEEEtypes_CtlFrame_t
{
   IEEEtypes_PsPollHdr_t  Hdr;
   UINT32                 FCS;
} PACK_END IEEEtypes_CtlFrame_t;
   /* */
   /* The structure for control frames (of which only the power-save */
   /* poll is processed by the MAC software) */
   /* */

typedef PACK_START struct IEEEtypes_DataFrame_t
{
   IEEEtypes_GenHdr_t   Hdr;
   UINT8                FrmBody[IEEEtypes_MAX_DATA_BODY_LEN];
   UINT32               FCS;
} PACK_END IEEEtypes_DataFrame_t;
   /* */
   /* The structure for data frames */
   /* */


/*-------------------------------------------------*/
/* Management Frame Body Components - Fixed Fields */
/*-------------------------------------------------*/
typedef UINT16 IEEEtypes_AId_t;
   /* */
   /* Association ID assigned by an AP during the association process */
   /* */

typedef UINT16 IEEEtypes_AuthAlg_t;
   /* */
   /* Number indicating the authentication algorithm used (it can take */
   /* on the values given by IEEEtypes_AuthType_e): */
   /*    0 = Open system */
   /*    1 = Shared key */
   /*    All other values reserved */
   /* */

typedef UINT16 IEEEtypes_AuthTransSeq_t;
   /* */
   /* Authentication transaction sequence number that indicates the current */
   /* state of progress through a multistep transaction */
   /* */

typedef UINT16 IEEEtypes_BcnInterval_t;
   /* */
   /* Beacon interval that represents the number of time units between */
   /* target beacon transmission times */
   /* */

typedef UINT8 IEEEtypes_DtimPeriod_t;
   /* */
   /* Interval that represents the number of time units between DTIMs. */
   /* */

typedef PACK_START struct IEEEtypes_CapInfo_t
{
   unsigned   Ess:1;
   unsigned   Ibss:1;
   unsigned   CfPollable:1;
   unsigned   CfPollRqst:1;
   unsigned   Privacy:1;
   unsigned   ShortPreamble:1;
   unsigned   Pbcc:1;
   unsigned   ChanAgility:1 ;
#ifndef ERP
   unsigned   Rsrvd:8;
#else
   unsigned   Rsrvd1:2;
   unsigned   ShortSlotTime:1;
   unsigned   Rsrvd2:2;
   unsigned   DsssOfdm:1;
   unsigned   Rsrvd3:2;
#endif
} PACK_END IEEEtypes_CapInfo_t;
   /* */
   /* Capability information used to indicate requested or advertised */
   /* capabilities */
   /* */

typedef UINT16 IEEEtypes_ListenInterval_t;
   /* */
   /* Listen interval to indicate to an AP how often a STA wakes to listen */
   /* to beacon management frames */
   /* */

typedef UINT16 IEEEtypes_ReasonCode_t;
   /* */
   /* Reason code to indicate the reason that an unsolicited notification */
   /* management frame of type Disassociation or Deauthentication was */
   /* generated */
   /* */

typedef UINT16 IEEEtypes_StatusCode_t;
   /* */
   /* Status code used in a response management frame to indicate the */
   /* success or failure of a requested operation */
   /* */

typedef UINT8 IEEEtypes_TimeStamp_t[IEEEtypes_TIME_STAMP_SIZE];
   /* */
   /* Time stamp */
   /* */
typedef PACK_START struct IEEEtypes_QosCtl_t
{
  UINT16 QosControl;
} PACK_END IEEEtypes_QosCtl_t;
/*-------------------------------------------------------*/
/* Management Frame Body Components - Information Fields */
/*-------------------------------------------------------*/
typedef PACK_START struct IEEEtypes_InfoElementHdr_t
{
  IEEEtypes_ElementId_e  ElementId;
  IEEEtypes_Len_t        Len;
} PACK_END IEEEtypes_InfoElementHdr_t;

typedef PACK_START struct IEEEtypes_SsIdElement_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   IEEEtypes_SsId_t       SsId;
} PACK_END IEEEtypes_SsIdElement_t;
   /* */
   /* SSID element that idicates the identity of an ESS or IBSS */
   /* */

typedef PACK_START struct IEEEtypes_SuppRatesElement_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   IEEEtypes_DataRate_t   Rates[IEEEtypes_MAX_DATA_RATES];
} PACK_END IEEEtypes_SuppRatesElement_t;
   /* */
   /* Supported rates element that specifies the rates in the operational */
   /* rate set in the MLME join request and the MLME start request */
   /* */

typedef PACK_START struct IEEEtypes_FhParamSet_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT16                 DwellTime;
   UINT8                  HopSet;
   UINT8                  HopPattern;
   UINT8                  HopIndex;
} PACK_END IEEEtypes_FhParamSet_t;
   /* */
   /* FH parameter set that conatins the set of parameters necessary to */
   /* allow sychronization for stations using a frequency hopping PHY */
   /* */

typedef PACK_START struct IEEEtypes_DsParamSet_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT8                  CurrentChan;
} PACK_END IEEEtypes_DsParamSet_t;
   /* */
   /* DS parameter set that contains information to allow channel number */
   /* identification for stations using a direct sequence spread spectrum PHY */
   /* */

typedef PACK_START struct IEEEtypes_CfParamSet_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT8                  CfpCnt;
   UINT8                  CfpPeriod;
   UINT16                 CfpMaxDuration;
   UINT16                 CfpDurationRemaining;
} PACK_END IEEEtypes_CfParamSet_t;
   /* */
   /* CF parameter set that contains the set of parameters necessary to */
   /* support the PCF */
   /* */

typedef PACK_START struct IEEEtypes_Tim_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT8                  DtimCnt;
   UINT8                  DtimPeriod;
   UINT8                  BitmapCtl;
   UINT8                  PartialVirtualBitmap[IEEEtypes_BITMAP_SIZE];
} PACK_END IEEEtypes_Tim_t;
   /* */
   /* TIM, which contains: */
   /* 1) DTIM count - how many beacons (including the current beacon */
   /*    frame) appear before the next DTIM; a count of 0 indicates the */
   /*    current TIM is the DTIM */
   /* */
   /* 2) DTIM period - indicates the number of beacon intervals between */
   /*    successive DTIMs */
   /* */
   /* 3) Bitmap control - contains the traffic indicator bit associated */
   /*    with association ID 0 - this is set to 1 for TIM elements with a */
   /*    a value of 0 in the DTIM count field when one or more broadcast */
   /*    or multicast frames are buffered at the AP. The remaining bits */
   /*    of the field form the bitmap offset */
   /* */
   /* 4) Partial virtual bitmap - indicates which stations have messages */
   /*    buffered at the AP, for which the AP is prepared to deliver at */
   /*    the time the beacon frame is transmitted */

typedef PACK_START struct IEEEtypes_IbssParamSet_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT16                 AtimWindow;
} PACK_END IEEEtypes_IbssParamSet_t;
   /* */
   /* IBSS parameters necessary to support an IBSS */
   /* */

typedef PACK_START struct IEEEtypes_ChallengeText_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   UINT8                  Text[IEEEtypes_CHALLENGE_TEXT_SIZE];
} PACK_END IEEEtypes_ChallengeText_t;
   /* */
   /* The challenge text used in authentication exchanges */
   /* */

/*-------------------------*/
/* Management Frame Bodies */
/*-------------------------*/
typedef PACK_START union IEEEtypes_PhyParamSet_t
{
   IEEEtypes_FhParamSet_t  FhParamSet;
   IEEEtypes_DsParamSet_t  DsParamSet;
} PACK_END IEEEtypes_PhyParamSet_t;
   /* */
   /* The parameter set relevant to the PHY */
   /* */

typedef PACK_START union IEEEtypes_SsParamSet_t
{
   IEEEtypes_CfParamSet_t    CfParamSet;
   IEEEtypes_IbssParamSet_t  IbssParamSet;
} PACK_END IEEEtypes_SsParamSet_t;
   /* */
   /* Service set parameters - for a BSS supporting, PCF, the */
   /* CF parameter set is used; for an independent BSS, the IBSS */
   /* parameter set is used. */
   /* */
typedef PACK_START struct IEEEtypes_ExtSuppRatesElement_t
{
   IEEEtypes_ElementId_e  ElementId;
   IEEEtypes_Len_t        Len;
   IEEEtypes_DataRate_t   Rates[IEEEtypes_MAX_DATA_RATES];
} PACK_END IEEEtypes_ExtSuppRatesElement_t;

typedef PACK_START struct IEEEtypes_ERPInfo_t
{
	UINT8   NonERPPresent  :1;
	UINT8   UseProtection  :1;
	UINT8   BarkerPreamble :1;
	UINT8   reserved       :5;
} PACK_END IEEEtypes_ERPInfo_t;

typedef PACK_START struct IEEEtypes_ERPInfoElement_t
{
	IEEEtypes_ElementId_e ElementId;
	IEEEtypes_Len_t        Len;
	IEEEtypes_ERPInfo_t    ERPInfo;
} PACK_END IEEEtypes_ERPInfoElement_t;

typedef PACK_START struct IEEEtypes_RSN_IE_t
{
    UINT8   ElemId;
    UINT8   Len;
    UINT8   OuiType[4];    /*00:50:f2:01 */
    UINT8   Ver[2];
    UINT8   GrpKeyCipher[4];
    UINT8   PwsKeyCnt[2];
    UINT8   PwsKeyCipherList[4];
    UINT8   AuthKeyCnt[2];
    UINT8   AuthKeyList[4];
    //UINT8   RsnCap[2];
} PACK_END IEEEtypes_RSN_IE_t;

typedef PACK_START struct IEEEtypes_RSN_IE_WPA2_t
{
    UINT8 ElemId;
    UINT8 Len;
    //uint8   OuiType[4];    //00:50:f2:01
    UINT8   Ver[2];
    UINT8 GrpKeyCipher[4];
    UINT8   PwsKeyCnt[2];
    UINT8 PwsKeyCipherList[4];
    UINT8   AuthKeyCnt[2];
    UINT8 AuthKeyList[4];
    UINT8   RsnCap[2];
    UINT8   PMKIDCnt[2];
    UINT8 PMKIDList[16];
}PACK_END IEEEtypes_RSN_IE_WPA2_t;

typedef PACK_START struct IEEEtypes_Bcn_t
{
   IEEEtypes_TimeStamp_t         TimeStamp;
   IEEEtypes_BcnInterval_t       BcnInterval;
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_SsIdElement_t       SsId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_PhyParamSet_t       PhyParamSet;
   IEEEtypes_SsParamSet_t        SsParamSet;
   /* for ap the following structure is used only for size calculation, 
   the location and size of the elements can be changed*/
   IEEEtypes_RSN_IE_t            RsnIE;
   IEEEtypes_RSN_IE_WPA2_t       RsnIEWPA2;
   IEEEtypes_Tim_t               Tim;
   IEEEtypes_ERPInfoElement_t    ERPInfo;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
} PACK_END IEEEtypes_Bcn_t;
   /* */
   /* Beacon message body */
   /* */

typedef PACK_START struct IEEEtypes_DisAssoc_t
{
   IEEEtypes_ReasonCode_t  ReasonCode;
} PACK_END IEEEtypes_DisAssoc_t;
   /* */
   /* Disassociation message body */
   /* */

typedef PACK_START struct IEEEtypes_AssocRqst_t
{
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_ListenInterval_t    ListenInterval;
   IEEEtypes_SsIdElement_t       SsId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#if defined (ECL_WPA)	 
   IEEEtypes_RSN_IE_t            RsnIE;
#endif	 
} PACK_END IEEEtypes_AssocRqst_t;
   /* */
   /* Association request message body */
   /* */

typedef PACK_START struct IEEEtypes_AssocRsp_t
{
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_StatusCode_t        StatusCode;
   IEEEtypes_AId_t               AId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
} PACK_END IEEEtypes_AssocRsp_t;
   /* */
   /* Association response message body */
   /* */

typedef PACK_START struct IEEEtypes_ReassocRqst_t
{
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_ListenInterval_t    ListenInterval;
   IEEEtypes_MacAddr_t           CurrentApAddr;
   IEEEtypes_SsIdElement_t       SsId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#if defined (ECL_WPA)	 
   IEEEtypes_RSN_IE_t            RsnIE;
#endif	 
} PACK_END IEEEtypes_ReassocRqst_t;
   /* */
   /* Reassociation request message body */
   /* */

typedef PACK_START struct IEEEtypes_ReassocRsp_t
{
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_StatusCode_t        StatusCode;
   IEEEtypes_AId_t               AId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
} PACK_END IEEEtypes_ReassocRsp_t;
   /* */
   /* Reassociation response message body */
   /* */

typedef PACK_START struct IEEEtypes_ProbeRqst_t
{
   IEEEtypes_SsIdElement_t       SsId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
} PACK_END IEEEtypes_ProbeRqst_t;
   /* */
   /* Probe request message body */
   /* */

typedef PACK_START struct IEEEtypes_ProbeRsp_t
{
   IEEEtypes_TimeStamp_t         TimeStamp;
   IEEEtypes_BcnInterval_t       BcnInterval;
   IEEEtypes_CapInfo_t           CapInfo;
   IEEEtypes_SsIdElement_t       SsId;
   IEEEtypes_SuppRatesElement_t  SuppRates;
   IEEEtypes_PhyParamSet_t       PhyParamSet;
   IEEEtypes_SsParamSet_t        SsParamSet;
   IEEEtypes_Tim_t               Tim;
   IEEEtypes_ERPInfoElement_t    ERPInfo;
   IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
} PACK_END IEEEtypes_ProbeRsp_t;
   /* */
   /* Probe response message body */
   /* */

typedef PACK_START struct IEEEtypes_Auth_t
{
   IEEEtypes_AuthAlg_t        AuthAlg;
   IEEEtypes_AuthTransSeq_t   AuthTransSeq;
   IEEEtypes_StatusCode_t     StatusCode;
   IEEEtypes_ChallengeText_t  ChallengeText;
} PACK_END IEEEtypes_Auth_t;
   /* */
   /* Authentication message body */
   /* */

typedef PACK_START struct IEEEtypes_Deauth_t
{
   IEEEtypes_ReasonCode_t  ReasonCode;
} PACK_END IEEEtypes_Deauth_t;
   /* */
   /* Deauthentication message body */
   /* */

/*---------------------------------------------------------------------------*/
/*              IEEE 802.11 MLME SAP Interface Data Structures               */
/*                                                                           */
/* According to IEEE 802.11, services are provided by the MLME to the SME.   */
/* In the current architecture, the services are provided to the SME by the  */
/* MAC Management Service Task. This section describes data structures       */
/* needed for these services.                                                */
/*---------------------------------------------------------------------------*/

/*---------------------*/
/* BSS Description Set */
/*---------------------*/
typedef PACK_START struct IEEEtypes_BssDesc_t
{
   IEEEtypes_MacAddr_t       BssId;
   IEEEtypes_SsId_t          SsId;
   IEEEtypes_Bss_e           BssType;
   IEEEtypes_BcnInterval_t   BcnPeriod;
   IEEEtypes_DtimPeriod_t    DtimPeriod;
   IEEEtypes_TimeStamp_t     Tstamp;
   IEEEtypes_TimeStamp_t     StartTs;
   IEEEtypes_PhyParamSet_t   PhyParamSet;
   IEEEtypes_SsParamSet_t    SsParamSet;
   IEEEtypes_CapInfo_t       Cap;
#ifndef ERP
   IEEEtypes_DataRate_t      DataRates[IEEEtypes_MAX_DATA_RATES];
#else
   IEEEtypes_DataRate_t      DataRates[IEEEtypes_MAX_DATA_RATES_G];
#endif
#if defined (ECL_WPA)	 
   IEEEtypes_RSN_IE_t            RsnIE;
#endif	
} PACK_END IEEEtypes_BssDesc_t;
   /* */
   /* A description of a BSS, providing the following: */
   /* BssId:        The ID of the BSS */
   /* SsId:         The SSID of the BSS */
   /* BssType:      The type of the BSS (INFRASTRUCTURE or INDEPENDENT) */
   /* BcnPeriod:    The beacon period (in time units) */
   /* DtimPeriod:   The DTIM period (in beacon periods) */
   /* Tstamp:       Timestamp of a received frame from the BSS; this is an 8 */
   /*                  byte string from a probe response or beacon */
   /* StartTs:      The value of a station's timing synchronization function */
   /*                  at the start of reception of the first octet of the */
   /*                  timestamp field of a received frame (probe response or */
   /*                  beacon) from a BSS; this is an 8 byte string */
   /* PhyParamSet:  The parameter set relevant to the PHY (empty if not */
   /*                  needed by the PHY) */
   /* SsParamSet:   The service set parameters. These can consist of either */
   /*                  the parameter set for CF periods or for an IBSS. */
   /* Cap:          The advertised capabilities of the BSS */
   /* DataRates:    The set of data rates that must be supported by all */
   /*                  stations (the BSS basic rate set) */
   /* */

typedef PACK_START struct IEEEtypes_BssDescSet_t
{
   UINT8                NumSets;
   IEEEtypes_BssDesc_t  BssDesc[IEEEtypes_MAX_BSS_DESCRIPTS];
} PACK_END IEEEtypes_BssDescSet_t;
   /* */
   /* The set of BSS descriptions */
   /* */

/*
Data structure to hold the Cipher Suites of AP's while scanning
*/
typedef struct
{
    UINT8 MulticastCipher[4];
    UINT8 UnicastCipher[4];
}WPA_AP_Ciphers_t;
                  
/*-------------------*/
/* MLME SAP Messages */
/*-------------------*/
typedef PACK_START struct IEEEtypes_PwrMgmtCmd_t
{
   IEEEtypes_PwrMgmtMode_e  PwrMgmtMode;
   BOOLEAN                  WakeUp;
   BOOLEAN                  RcvDTIMs;
} PACK_END IEEEtypes_PwrMgmtCmd_t;
   /* */
   /* Power management request message from the SME */
   /* */

typedef PACK_START struct IEEEtypes_PwrMgmtCfrm_t
{
   IEEEtypes_PwrMgmtResult_e  Result;
} PACK_END IEEEtypes_PwrMgmtCfrm_t;
   /* */
   /* Power management confirm message sent from the MLME as a result */
   /* of a power management request; it is sent after the change has */
   /* taken place */
   /* */

typedef PACK_START struct IEEEtypes_ScanCmd_t
{
   IEEEtypes_Bss_e      BssType;
   IEEEtypes_MacAddr_t  BssId;
   IEEEtypes_SsId_t     SsId;
   IEEEtypes_Scan_e     ScanType;
   UINT16               ProbeDelay;
   UINT8                ChanList[IEEEtypes_MAX_CHANNELS];
   /*UINT8                Reserved; */
   UINT16               MinChanTime;
   UINT16               MaxChanTime;
} PACK_END IEEEtypes_ScanCmd_t;
   /* */
   /* Scan request message from the SME to determine if there are BSSs */
   /* that can be joined */
   /* */
   /* Note: The "Reserved" field is inserted for alignment for */
   /* commands coming from the host */
   /* */

typedef PACK_START struct IEEEtypes_ScanCfrm_t
{
   IEEEtypes_BssDescSet_t  BssDescSet;
   IEEEtypes_ScanResult_e  Result;
} PACK_END IEEEtypes_ScanCfrm_t;
   /* */
   /* Scan confirm message sent from the MLME as a result of a scan request; */
   /* it reports the results of the scan */
   /* */

typedef PACK_START struct IEEEtypes_JoinCmd_t
{
   IEEEtypes_BssDesc_t    BssDesc;
   UINT16                 FailTimeout;
   UINT16                 ProbeDelay;
#ifndef ERP
   IEEEtypes_DataRate_t   OpRateSet[IEEEtypes_MAX_DATA_RATES];
#else   
   IEEEtypes_DataRate_t   OpRateSet[IEEEtypes_MAX_DATA_RATES_G];
#endif
} PACK_END IEEEtypes_JoinCmd_t;
   /* */
   /* Join request message from the SME to establish synchronization with */
   /* a BSS */
   /* */

typedef PACK_START struct IEEEtypes_JoinCfrm_t
{
   IEEEtypes_JoinResult_e  Result;
} PACK_END IEEEtypes_JoinCfrm_t;
   /* */
   /* Join confirm message sent from the MLME as a result of a join request; */
   /* it reports the result of the join */
   /* */

typedef PACK_START struct IEEEtypes_AuthCmd_t
{
   IEEEtypes_MacAddr_t   PeerStaAddr;
   UINT16                FailTimeout;
   IEEEtypes_AuthType_e  AuthType;
} PACK_END IEEEtypes_AuthCmd_t;
   /* */
   /* Authenticate request message sent from the SME to establish */
   /* authentication with a specified peer MAC entity */
   /* */

typedef PACK_START struct IEEEtypes_AuthCfrm_t
{
   IEEEtypes_MacAddr_t     PeerStaAddr;
   IEEEtypes_AuthType_e    AuthType;
   IEEEtypes_AuthResult_e  Result;
} PACK_END IEEEtypes_AuthCfrm_t;
   /* */
   /* Authenticate confirm message sent from the MLME as a result of an */
   /* authenticate request; it reports the result of the authentication */
   /* */

typedef PACK_START struct IEEEtypes_AuthInd_t
{
   IEEEtypes_MacAddr_t   PeerStaAddr;
   IEEEtypes_AuthType_e  AuthType;
} PACK_END IEEEtypes_AuthInd_t;
   /* */
   /* Authenticate indication message sent from the MLME to report */
   /* authentication with a peer MAC entity that resulted from an */
   /* authentication procedure that was initiated by that MAC entity */
   /* */

typedef PACK_START struct IEEEtypes_DeauthCmd_t
{
   IEEEtypes_MacAddr_t     PeerStaAddr;
   IEEEtypes_ReasonCode_t  Reason;
} PACK_END IEEEtypes_DeauthCmd_t;
   /* */
   /* Deauthenticate request message sent from the SME to invalidate */
   /* authentication with a specified peer MAC entity */
   /* */

typedef PACK_START struct IEEEtypes_DeauthCfrm_t
{
   IEEEtypes_MacAddr_t       PeerStaAddr;
   IEEEtypes_DeauthResult_e  Result;
} PACK_END IEEEtypes_DeauthCfrm_t;
   /* */
   /* Deauthenticate confirm message sent from the MLME as a result of a */
   /* deauthenticate request message; it reports the result of the */
   /* deauthentication */
   /* */

typedef PACK_START struct IEEEtypes_DeauthInd_t
{
   IEEEtypes_MacAddr_t     PeerStaAddr;
   IEEEtypes_ReasonCode_t  Reason;
} PACK_END IEEEtypes_DeauthInd_t;
   /* */
   /* Deauthentication indication message sent from the MLME to report */
   /* invalidation of an authentication with a peer MAC entity; the message */
   /* is generated as a result of an invalidation of the authentication */
   /* */

typedef PACK_START struct IEEEtypes_AssocCmd_t
{
   IEEEtypes_MacAddr_t         PeerStaAddr;
   UINT16                      FailTimeout;
   IEEEtypes_CapInfo_t         CapInfo;
   IEEEtypes_ListenInterval_t  ListenInterval;
} PACK_END IEEEtypes_AssocCmd_t;
   /* */
   /* Association request message sent from the SME to establish an */
   /* association with an AP */
   /* */

typedef PACK_START struct IEEEtypes_AssocCmd_ext_t
{
   IEEEtypes_MacAddr_t         PeerStaAddr;
   UINT16                      FailTimeout;
   IEEEtypes_CapInfo_t         CapInfo;
   IEEEtypes_ListenInterval_t  ListenInterval;
   IEEEtypes_SsId_t         	SsId;
   IEEEtypes_Bss_e           	BssType;
   IEEEtypes_BcnInterval_t   	BcnPeriod;
   IEEEtypes_DtimPeriod_t    	DtimPeriod;
   IEEEtypes_PhyParamSet_t   	PhyParamSet;
   IEEEtypes_SsParamSet_t    	SsParamSet;
#ifndef ERP
   IEEEtypes_DataRate_t      	DataRates[IEEEtypes_MAX_DATA_RATES];
#else
   IEEEtypes_DataRate_t      	DataRates[IEEEtypes_MAX_DATA_RATES_G];
#endif
#if defined (ECL_WPA)	 
   IEEEtypes_RSN_IE_t           RsnIE;
#endif
} PACK_END IEEEtypes_AssocCmd_ext_t;
   /* */
   /* Association request message sent from the SME to establish an */
   /* association with an AP */
   /* */

typedef PACK_START struct IEEEtypes_AssocCfrm_t
{
   IEEEtypes_AssocResult_e  Result;
} PACK_END IEEEtypes_AssocCfrm_t;
   /* */
   /* Association confirm message sent from the MLME as a result of an */
   /* association request message; it reports the result of the assoication */
   /* */

typedef PACK_START struct IEEEtypes_AssocInd_t
{
   IEEEtypes_MacAddr_t  PeerStaAddr;
} PACK_END IEEEtypes_AssocInd_t;
   /* */
   /* Association indication message sent from the MLME to report an */
   /* association with a specified peer MAC entity acting as an AP; the */
   /* indication is the result of an association procedure that was */
   /* initiated by the peer MAC entity */
   /* */

typedef PACK_START struct IEEEtypes_ReassocCmd_t
{
   IEEEtypes_MacAddr_t         NewApAddr;
   UINT16                      FailTimeout;
   IEEEtypes_CapInfo_t         CapInfo;
   IEEEtypes_ListenInterval_t  ListenInterval;
} PACK_END IEEEtypes_ReassocCmd_t;
   /* */
   /* Reassociation request message sent from the SME to change association */
   /* to a specified new peer MAC entity acting as an AP */
   /* */

typedef PACK_START struct IEEEtypes_ReassocCfrm_t
{
   IEEEtypes_ReassocResult_e  Result;
} PACK_END IEEEtypes_ReassocCfrm_t;
   /* */
   /* Reassociation confirm message sent from the MLME as the result of a */
   /* reassociate request message; it reports the result of the reassociation */
   /* */

typedef PACK_START struct IEEEtypes_ReassocInd_t
{
   IEEEtypes_MacAddr_t  PeerStaAddr;
} PACK_END IEEEtypes_ReassocInd_t;
   /* */
   /* Reassociate indication message sent from the MLME to report a */
   /* reassociation with a specified peer MAC entity; the */
   /* indication is the result of a reassociation procedure that was */
   /* initiated by the peer MAC entity */
   /* */

typedef PACK_START struct IEEEtypes_DisassocCmd_t
{
   IEEEtypes_MacAddr_t     PeerStaAddr;
   IEEEtypes_ReasonCode_t  Reason;
} PACK_END IEEEtypes_DisassocCmd_t;
   /* */
   /* Disassociate request message sent from the SME to establish */
   /* disassociation with an AP */
   /* */

typedef PACK_START struct IEEEtypes_DisassocCfrm_t
{
   IEEEtypes_DisassocResult_e  Result;
} PACK_END IEEEtypes_DisassocCfrm_t;
   /* */
   /* Disassociate confirm message sent from the MLME as a result of a */
   /* disassociate request message; it reports the result of the */
   /* disassociation */
   /* */

typedef PACK_START struct IEEEtypes_DisassocInd_t
{
   IEEEtypes_MacAddr_t     PeerStaAddr;
   IEEEtypes_ReasonCode_t  Reason;
} PACK_END IEEEtypes_DisassocInd_t;
   /* */
   /* Disassociate indication message sent from the MLME to report the */
   /* invalidation of an association relationship with a peer MAC entity; */
   /* the message is generated as a result of an invalidation of an */
   /* association relationship */
   /* */

typedef PACK_START struct IEEEtypes_ResetCmd_t
{
   IEEEtypes_MacAddr_t  StaAddr;
   BOOLEAN              SetDefaultMIB;
   BOOLEAN              quiet;
} PACK_END IEEEtypes_ResetCmd_t;
   /* */
   /* Reset request message sent from the SME to reset the MAC to initial */
   /* conditions; the reset must be used prior to a start command */
   /* */

typedef PACK_START struct IEEEtypes_ResetCfrm_t
{
   IEEEtypes_ResetResult_e  Result;
} PACK_END IEEEtypes_ResetCfrm_t;
   /* */
   /* Reset confirm message sent from the MLME as a result of a reset */
   /* request message; it reports the result of the reset */
   /* */

typedef PACK_START struct IEEEtypes_StartCmd_t
{
   IEEEtypes_SsId_t           SsId;
   IEEEtypes_Bss_e            BssType;
   IEEEtypes_BcnInterval_t    BcnPeriod;
   IEEEtypes_DtimPeriod_t     DtimPeriod;
   IEEEtypes_SsParamSet_t     SsParamSet;
   IEEEtypes_PhyParamSet_t    PhyParamSet;
   UINT16                     ProbeDelay;
   IEEEtypes_CapInfo_t        CapInfo;
#ifndef ERP
   IEEEtypes_DataRate_t       BssBasicRateSet[IEEEtypes_MAX_DATA_RATES];
   IEEEtypes_DataRate_t       OpRateSet[IEEEtypes_MAX_DATA_RATES];
#else
   IEEEtypes_DataRate_t       BssBasicRateSet[IEEEtypes_MAX_DATA_RATES_G];
   IEEEtypes_DataRate_t       OpRateSet[IEEEtypes_MAX_DATA_RATES_G];
#endif
} PACK_END IEEEtypes_StartCmd_t;
   /* */
   /* Start request message sent from the SME to start a new BSS; the BSS */
   /* may be either an infrastructure BSS (with the MAC entity acting as the */
   /* AP) or an independent BSS (with the MAC entity acting as the first */
   /* station in the IBSS) */
   /* */

typedef PACK_START struct IEEEtypes_StartCfrm_t
{
   IEEEtypes_StartResult_e  Result;
} PACK_END IEEEtypes_StartCfrm_t;
   /* */
   /* Start confirm message sent from the MLME as a result of a start request */
   /* message; it reports the results of the BSS creation procedure */
   /* */

typedef PACK_START struct IEEEtypes_Frame_t
{
    IEEEtypes_GenHdr_t Hdr;
    UINT8 Body[8];
} PACK_END IEEEtypes_Frame_t;

typedef PACK_START struct ether_hdr_t
{
    IEEEtypes_MacAddr_t  da;
    IEEEtypes_MacAddr_t  sa;
    UINT16               type;
} PACK_END ether_hdr_t;


typedef PACK_START struct IEEEtypes_8023_Frame_t
{
    ether_hdr_t   Hdr;
    UINT8         Body[1600-14];
} PACK_END IEEEtypes_8023_Frame_t;


typedef enum
{
    IEEEtypes_RSN_AUTH_KEY_SUITE_RSVD = 0,
    IEEEtypes_RSN_AUTH_KEY_SUITE_8021X,
    IEEEtypes_RSN_AUTH_KEY_SUITE_PSK
}IEEEtypes_RSN_Auth_Key_Suite;

/* Cipher Suite Selector */
typedef enum
{
    IEEEtypes_RSN_CIPHER_SUITE_NONE = 0,
    IEEEtypes_RSN_CIPHER_SUITE_WEP40,
    IEEEtypes_RSN_CIPHER_SUITE_TKIP,
    IEEEtypes_RSN_CIPHER_SUITE_WRAP,
    IEEEtypes_RSN_CIPHER_SUITE_CCMP,
    IEEEtypes_RSN_CIPHER_SUITE_WEP104
} IEEEtypes_RSN_Cipher_Suite;

#define MAX_SIZE_RSN_IE_BUF 32  /* number of bytes */


#endif /* _IEEE_TYPES_H_ */
