#ifdef WDS_FEATURE
//Powerful MAC address hash function V1.04
//      Editor : NEO_SHEU, Taiwan. (07)5882241

#ifndef	_HASH_H
#define	_HASH_H

typedef UINT8 U8;
typedef UINT16 U16;
typedef UINT32 U32;
#define	U16of(p) *(UINT16 *)((UINT8 *)p)

//OS critical section defination, the critical control must be nestable
#define	OSincCritical 	{}//OS_Enter_Critical()
#define	OSdecCritical	{}//OS_Exit_Critical()

//#define	HASH_TEST

//User defined HASH_ENTRYS
// The default HASH_ENTRYS is defined here, user may redefine it by your need.
// The HASH_ENTRYS must be !!! 2 powered constant !!!
//	1. Define it in front of #include "hash.h"
//	2. Update the default defination here
#ifndef	HASH_ENTRYS
	#define	HASH_ENTRYS		32//MAC_DB_SIZE	//Default HASH_ENTRYS
#endif

//Critical section control
//	0:don't care critical
//	else: see the define of OSincCritical & OSdecCritical in utility.h
#define	HASH_NEED_CRITICAL	1		

//**********************
#if	HASH_NEED_CRITICAL
	#define	HASHincCritical	OSincCritical
	#define	HASHdecCritical	OSdecCritical
#else
	#define	HASHincCritical
	#define	HASHdecCritical
#endif

typedef struct HASHstruc
{
	struct	HASHstruc	*ptNextHASH,*ptNextUse,*ptLastUse;
	U32		ctHASHage;
	U16		HASHaddress[3],padding;
} sHASH;

//for HASHautoSearch() output
#define HASH_BCAST	((sHASH *)0xffffffff)
#define HASH_MCAST	((sHASH *)0x80000000)
#define	HASH_NOT_FOUND ((sHASH *)0x7fffffff)
//since ( ctHASHage ) will always between 0 to 0x7fffffff, define
//follows to keep hash buffer used or not
#define	HASH_FREE	0xffffffff
#define	HASH_ME		0xfffffffe
//for HASHscan's ScanRule
#define	HASH_SAFE_SCAN	0
#define	HASH_OtoN_SCAN	1
#define	HASH_NtoO_SCAN	2

typedef struct HASHbufferStruc
{
	sHASH	*HASHentry[HASH_ENTRYS];
	sHASH	HASHtable;
} sHASHbuffer;

//main io control block
typedef struct HASHioStruc
{
//input
	void 	(*HASHcargoCB)(sHASH *pHASH);
		//called when a new member is inserted into HASH table,
		//For application to initialize the member's cargo
	void 	(*HASHdeleteCB)(sHASH *pHASH);
		//!!! called when a member will be killed !!!
		//set to 0 if you don't care the kill
	void 	*ptHASHmyAddress;
		//the MACaddress will always be insert first, never be kill
		//set to 0 if you don't have a MACaddress
	U32		CargoSize;
	U32		HASHageTTL;
		//See HASHage()
//output
	sHASH	*MyHASH;
	U32		ctHASHmember;
	U32		ctHASHkilled;
//library buffer
	U32		TotalMember;
	U32		HASHsize;	//size of sHASH+CargoSize
	sHASH	**HASHentry,*HASHtable;
	sHASH	*ptFreeHASH;
	sHASH	*ptUseHASH;		//The link from newest to oldest by ptNextUse
	sHASH	*ptKillHASH;	//The link from oldest to newest by ptLastUse
	U32		ctAGE;
	U16		_McastBit;
	U16		OSlittle;
} sHASHio;

//define and alloc memory by user
#define	HASHsize(members,CargoSize)	(sizeof(sHASHbuffer)+(members-1)*sizeof(sHASH)+members*CargoSize)
#define	HASHcargo(pHASH) ((HASHcargoName *)((U32)pHASH+sizeof(sHASH)))

extern	sHASH *HASHinit(sHASHio *pHASHio,void *pHASHbuffer,U32 HASHmember);
	//1. Define your local HASH cargo struct, EX:
	//		typedef HASHcargoStruc
	//		{
	//			U32	cargo1;
	//			U32	cargo2;
	//			...
	//		} sHASHcargo;
	//2. Define your local HASHcargoName for later use of HASHcargo macro
	//		--> #define	HASHcargoName	sHASHcargo
	//3. Get and allocate the required buffer by HASHsize,EX:
	//		U8 HASH1[HASHsize(512,sizeof(sHASHcargo))];
	//4. Setup your IO control block HASHio, EX:
	//		sHASHio HASHio={CargoCB,DeleteCB,MyAddress,sizeof(sHASHcargo),30};
	//5. Call HASHinit(&HASHio,HASH1,512);
	//6. Write all Call Back functions.
	//*. You may define HASH2, HASH3, HASH..... with different cargo !
	//*. Access your HASH cargo member by HASHcargo macro, EX:
	//		void CargoCB(sHASH *pHASH)
	//		{
	//			HASHcargo(pHASH)->cargo1=HASHcargo(pHASH)->cargo2=0;
	//		}
	//()-> is my MACaddress HASH entry
extern	sHASH *HASHsetMACaddress(sHASHio *pHASHio,void *MACaddress);
	//Support application to reset My MACaddress after HASHinit()
	//!!! This is the only way to update My MACaddress !!!
extern	sHASH *HASHservice(sHASHio *pHASHio,void *MACaddress,U32 InsertDisable);
	//If InsertDisable == 0, the searched & not found member would be insert
	//	into table automaticlly. The insert control would be left to application.
	//()-> == HASH_BCAST : is broadcast
	//()-> == HASH_MCAST : is multicast
	//()-> == pHASHio.MyHASH / ()->.ctHASHage == HASH_ME : is my address
	//()-> & HASH_MCAST != 0 : is broadcast or multicast
	//()-> & HASH_MCAST == 0 : is unicast
	//()-> == HASH_NOT_FOUND : not found
#define	HASHautoSearch(pHASHio,MAC)	HASHservice(pHASHio,MAC,0)
#define	HASHSearch(pHASHio,MAC)		HASHservice(pHASHio,MAC,-1)
#define	HASHinsert					HASHautoSearch
extern	void HASHdelete(sHASHio *pHASHio,void *MACaddress,sHASH *pKILL);
	//if pKILL (kill the entry) else (kill by MACaddress)
	//kill by entry would be faster
extern	void HASHscan(sHASHio *pHASHio,void (*HASHscanCB)(sHASH *pHASH),U32 ScanRule);
	//ScanRule can be :
	//	(HASH_SAFE_SCAN)
	//		Call back with My MACaddress
	//		Search HASH members from array head to tail.
	//		If HASHservice() group may be called within your HASHscan period,
	//			it's better to use this ScanRule.
	//	(HASH_OtoN_SCAN)
	//		No call back with My MACaddress
	//		Call back with HASH members from Oldest to Newest
	//	(HASH_NtoO_SCAN)
	//		No call back with My MACaddress
	//		Call back with HASH members from Newest to Oldest
	//	HASHservice() group :
	//		HASHservice(), HASHautoSearch(), HASHsearch()
	//		HASHinsert(), HASHdelete(), HASHage()
extern	void HASHage(sHASHio *pHASHio);
	//How to age ?
	//1. Set the HASHio.HASHageTTL, it means when a member not accessed
	//		after HASHio.HASHageTTL times of HASHage() been called, the
	//		member will be killed.
	//2. If no age requirement, don't call HASHage().
	//3. 30 seconds age example : Set HASHio.HASHageTTL=30, call HASHage()
	//		per second.
	//4. All timeout members would be killed in a HASHage() call.

//if you want to run HASHtest(), do	#define	HASH_TEST
#ifdef HASH_TEST
extern	void GetKillList(sHASHio *pHASHio,void *list);
extern	sHASH *SearchKillList(sHASHio *pHASHio,void *list,U8 a1);
extern	void AgeKillList(sHASHio *pHASHio,void *list);
extern	void HASHtest(void);
/*
HASHtest() been put in HASH.C, It's also a sample code of the HASH library.
HASHtest() help to confirm hash function and do performance test
performance test :
1. normal operation rate test, hash buffer member will always keep
	the same 512 members, we search for 10,000,000 times
	The rate under ZCOM XI1000 platform is about 181,000 times/second
2. HASH under reflashing test, hash member will always be different,
	this will cause hash to kick the oldest member, still search
	for 10,000,000 times
	The rate under ZCOM XI1000 platform is about 78,000 times/second

A sample C main to run HASHtest():

//======================================================
#define	HASH_TEST	1
#include "hash.h"
#include "hash.c"

void main(void)
{
	HASHtest();
}
//======================================================

Different from V1.00 to V1.01
    Can run at another 32bit platform.
    Better application interface.
    Can maintain different piece of HASH buffer.
    Contain initial version of HASHtest().
Different from V1.01 to V1.02
    The kill list been linked for the situation when out of hash buffer
Different from V1.02 to V1.03
    AGE function support(sHASHio.HASHageTTL,HASHage())
	User defined HASH cargo(sHASHio.(*HASHcargoCB),sHASHio.CargoSize,HASHcargo())
	Support user don't have a MACaddress(sHASHio.ptHASHmyAddress)
Different from V1.03 to V1.04
	HASHscan input with ScanRule.
	Kill all time out members in a HASHage()
	User defined critical control macro
	Add input parameter 'InsertDisable' for HASHservice()
	Add HASHsetMACaddress() to reset My MACaddress after HASHinit()
	User defined HASH_ENTRYS to save entry space
*/
#endif

#endif
#endif // WDS_FEATURE