/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/wlandrv/incl/macMgmtMlme.h,v 1.16 2004/05/16 03:54:35 cfan Exp $
*
* Purpose:
*    This file .
*
* Public Procedures:
*    macMgmtMlme_AssociateCmd      Process a cmd to associate with an AP
*    macMgmtMlme_AssociateRsp      Process an associate rsp from an AP
*    macMgmtMlme_Atim              Process an ATIM msg from another STA
*    macMgmtMlme_AuthenticateCmd   Process a cmd to authenticate with
*                                  another station or an AP
*    macMgmtMlme_AutheticateMsg    Process an authentication msg from a
*                                  station or an AP
*    macMgmtMlme_BeaconMsg         Process a beacon msg from an AP or a
*                                  station
*    macMgmtMlme_DeauthenticateCmd Process a cmd to deauthenticate with a
*                                  station or an AP
*    macMgmtMlme_DeauthenticateMsg Process a deauthentication msg from a
*                                  station or an AP
*    macMgmtMlme_DisassociateCmd   Process a cmd to disassociate with an AP
*    macMgmtMlme_DisassociateMsg   Process a disassociation msg from an AP
*    macMgmtMlme_JoinCmd           Process a cmd to join a BSS
*    macMgmtMlme_ProbeRqst         Process a probe request from another
*                                  station in an IBSS
*    macMgmtMlme_ProbeRsp          Process a probe response from a station
*                                  or an AP
*    macMgmtMlme_ReassociateCmd    Process a cmd to reassociate with a new AP
*    macMgmtMlme_ReassociateRsp    Process a reassociation rsp from an AP
*    macMgmtMlme_ResetCmd          Process a cmd to peform a reset
*    macMgmtMlme_ScanCmd           Process a cmd to perform a scan for BSSs
*    macMgmtMlme_StartCmd          Process a cmd to start an IBSS
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _MACMGMTMLME_H_
#define _MACMGMTMLME_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "macmgmtap.h"
#include "osif.h"

/*============================================================================= */
/*                            PUBLIC DEFINITIONS */
/*============================================================================= */

#define ADD_TO_POLLING_LIST 1
#define DONOT_ADD_TO_POLLING_LIST 0
#define AID_PREFIX 0xC000
#define MAX_AID   2007
#define RATE_ADAPT_STNCNT_THRESH   5 

#define SLOT_TIME_MODE_SHORT 0
#define SLOT_TIME_MODE_LONG  1

/*Define how many Margins you will have for Rate Adaptation. */
#define NUM_MARGINS 6

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
/* */
/* Structure used to store data given at the time of successful */
/* association */
/* */
typedef struct AssocReqData_t
{
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_ListenInterval_t ListenInterval;
    IEEEtypes_SuppRatesElement_t SuppRates;
    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
    UINT32 SuppRateSetRegMap[IEEEtypes_MAX_DATA_RATES];
    UINT32 HighestRateIndex;
    UINT32 RateToBeUsedForTx;
#ifdef ENABLE_RATE_ADAPTATION_BASEBAND
    UINT8                        AvgRSSI;
    UINT8                        InvAlpha; /*to get running average. */
   
    UINT8                        TxSuccess; /*For Rate Adaptation */
    UINT8                        TxFailures; /*For Rate Adaptation. */
    UINT8                        TxRetry; /*For Rate Adaptation. */
#ifdef ENABLE_OSCILLATION_CODE
    UINT8                        OSC_FLAG; /*To prevent oscillations */
    UINT8                        RSSI_Store;
    UINT32                        DisableOscFlagCounter;

#endif /*ENABLE_OSCILLATION_CODE */
    //For maintaining the statistical counter for logging failures.

    UINT8 PktTxSinceFail;
    UINT8 TxFailuresStats;
    UINT8 RateIncreaseIncrement;
    UINT8 Current_RSSI;

#endif /*ENABLE_RATE_ADAPTATION_BASEBAND */
   UINT32                        RecvClientRate;/*to keep track of the client rate */

}
AssocReqData_t;

/*============================================================================= */
/*                                PUBLIC DATA */
/*============================================================================= */

extern AssocReqData_t AssocTable[MAX_AID + 1];
extern UINT32 HighestBasicRateIndex;
extern UINT32 bOnlyStnCnt;
extern UINT32 AssocStationsCnt;
#define macErpProtModeEnabled (!mib_forceProtectiondisable)
extern UINT32 macChangeSlotTimeModeTo;
extern UINT32 macCurSlotTimeMode;
#ifdef ERP
#ifndef WB_EXCEPTION
extern IEEEtypes_DataRate_t OpRateSet[IEEEtypes_MAX_DATA_RATES_G];
extern IEEEtypes_DataRate_t BasicRateSet[IEEEtypes_MAX_DATA_RATES_G];
extern UINT32 BasicRateSetLen;
extern UINT32 OpRateSetLen;
extern UINT32 bOnlyStnCnt;
extern IEEEtypes_ExtSuppRatesElement_t ExtSuppRateSet;
#endif /* WB_EXCEPTION */
#endif

extern IEEEtypes_SuppRatesElement_t SuppRateSet;
extern UINT32 HighestBasicRateIndexB;

extern volatile UINT32 ResetDone;

extern IEEEtypes_MacAddr_t macStaAddr;
extern IEEEtypes_MacAddr_t macBssId;
extern IEEEtypes_SsIdElement_t macSsId;
extern BOOLEAN WlanEnabled;
extern IEEEtypes_MacAddr_t bcast;
/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/******************************************************************************
 *
 * Name: macMgmtMlme_AssociateReq
 *
 * Description:
 *   Routine to handle a received associate reqeust.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an associate request
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_AssociateReq( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_AssociateRsp
 *
 * Description:
 *   This routine handles a response from an AP to a prior associate request.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing an associate response
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_AssociateRsp( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_Atim
 *
 * Description:
 *   This routine handles an ATIM sent from another station in an IBSS.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtHdr_p - Pointer to an 802.11 management message header
 *                           that contains an ATIM message
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   Only the 802.11 message header is input since the message body for an
 *   ATIM message is empty.
 *
 *****************************************************************************/
extern void macMgmtMlme_Atim( IEEEtypes_MgmtHdr_t *MgmtHdr_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_AuthenticateCmd
 *
 * Description:
 *   Routine to handle a command to carry out an authentication with another
 *   station or an AP.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): AuthCmd_p - Pointer to an authenticate command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_AuthenticateCmd( IEEEtypes_AuthCmd_t *AuthCmd_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_AuthenticateMsg
 *
 * Description:
 *   This routine handles a message from either another station of from an
 *   AP relating to authentication; the message can be a request for
 *   authentication or a response to a prior authentication message.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing an authentication message
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_AuthenticateMsg( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_BeaconMsg
 *
 * Description:
 *   This routine handles a beacon message received from an AP or station
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing an beacon message
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_BeaconMsg( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_DeauthenticateCmd
 *
 * Description:
 *   Routine to handle a command to carry out a deauthentication with another
 *   station or an AP.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): DeauthCmd_p - Pointer to a deauthenticate command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_DeauthenticateCmd(
                                         IEEEtypes_DeauthCmd_t *DeauthCmd_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_DeauthenticateMsg
 *
 * Description:
 *   This routine handles a deauthentication notification from another
 *   station or an AP.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing a deauthentication message
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_DeauthenticateMsg(
                                         macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_DisassociateCmd
 *
 * Description:
 *   Routine to handle a command to carry out a disassociation with an AP.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): DisassocCmd_p - Pointer to a disassociate command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_DisassociateCmd(
                                       IEEEtypes_DisassocCmd_t *DisassocCmd_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_DisassociateMsg
 *
 * Description:
 *   This routine handles a disassociation notification from an AP.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing a disassociation message
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_DisassociateMsg( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_JoinCmd
 *
 * Description:
 *   Routine to handle a command to join a BSS found during a scan.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): JoinCmd_p - Pointer to a join command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_JoinCmd( IEEEtypes_JoinCmd_t *JoinCmd_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_ProbeRqst
 *
 * Description:
 *   This routine handles a request from another station in an IBSS to
 *   respond to a probe.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing a probe request
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ProbeRqst( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_ProbeRsp
 *
 * Description:
 *   This routine handles a response from another station in an IBSS or an
 *   AP in a BSS to a prior probe request.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing a probe response
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ProbeRsp( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_ReassociateReq
 *
 * Description:
 *   Routine to handle received reassociate request.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to a reassociate reqeust
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ReassociateReq( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_ReassociateRsp
 *
 * Description:
 *   This routine handles a response from an AP to a prior reassociate request.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - Pointer to an 802.11 management message
 *                           containing a reassociate response
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ReassociateRsp( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_ResetCmd
 *
 * Description:
 *   Routine to handle a command to perform a reset, which resets the MAC
 *   to initial conditions.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): ResetCmd_p - Pointer to a reset command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ResetCmd( IEEEtypes_ResetCmd_t *ResetCmd_p);

/******************************************************************************
 *
 * Name: macMgmtMlme_ScanCmd
 *
 * Description:
 *   Routine to handle a command to perform a scan of potential BSSs that
 *   a station may later elect to join.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): ScanCmd_p - Pointer to a scan command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_ScanCmd( IEEEtypes_ScanCmd_t *ScanCmd_p );

/******************************************************************************
 *
 * Name: macMgmtMlme_StartCmd
 *
 * Description:
 *   Routine to handle a command to start an IBSS.
 *
 * Conditions For Use:
 *   All software components have been initialized and started.
 *
 * Arguments:
 *   Arg1 (i  ): StartCmd_p - Pointer to a start command
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void macMgmtMlme_StartCmd( IEEEtypes_StartCmd_t *StartCmd_p );


extern WL_STATUS macMgmtMlme_Init(UINT32 maxStns, IEEEtypes_MacAddr_t *stnMacAddr);
extern void macMgmtMlme_PsPollMsg(macmgmtQ_MgmtMsg_t *MgmtMsg_p);
extern void macMgmtMlme_SendDeauthenticateMsg(
                                             IEEEtypes_MacAddr_t *Addr, UINT16 StnId, UINT16 Reason );
extern void macMgmtMlme_SendDisassociateMsg(
                                           IEEEtypes_MacAddr_t *Addr, UINT16 StnId, UINT16 Reason );
void macMgmtMlme_StepUpRate(UINT16 Aid, UINT8 LastPktRate);
void macMgmtMlme_StepDownRate(UINT16 Aid, UINT8 LastPktRate);

#define SETBIT    1
#define RESETBIT  0
/* imported functions*/
extern WL_STATUS tx80211Q_MgmtWriteNoBlock( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

extern BOOLEAN isInMacAccessList(IEEEtypes_MacAddr_t *destAddr_p);

/*imported data*/
extern UINT8 mib_ErpProtEnabled;
extern UINT8 mib_ApMode;
extern UINT8 mib_shortSlotTime;
extern UINT8 mib_beaconDisable;
extern UINT8 mib_broadcastssid;

#ifndef WB_EXCEPTION
extern macmgmtQ_MgmtMsg_t * BcnBuffer_p;
extern IEEEtypes_RSN_IE_t thisStaRsnIE;
#endif /* WB_EXCEPTION */
#endif /* _MACMGMTMLME_H_ */
