/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* $Header: E:\\APCVSDEPOT/wlan/wlandrv/incl/macmgmtap.h,v 1.6 2004/04/08 00:27:58 cvu Exp $ 
* 
* Purpose: 
*    This file contains the function prototypes and definitions for the 
*    MAC Management Service Module of AP. 
* 
* Public Procedures: 
*    macMgmtAp_Init       Initialzies the MAC Management Service Task and 
*                           related components 
*    macMgmtAp_Start      Starts running the MAC Management Service Task 
* 
* Notes: 
*    None. 
* 
*****************************************************************************/

#ifndef _MACMGMTAP_H_
#define _MACMGMTAP_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "IEEE_types.h"
#include "osif.h"
#ifndef ECL_WPA
#include "StaDb.h"
#endif

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
#define macMgmtMain_802_11_MGMT_MSG_RCVD   (1<<0)
#define macMgmtMain_BCN_RCVD               (1<<1)
#define macMgmtMain_PWR_MODE_CHANGE_RCVD   (1<<2)
#define macMgmtMain_SME_MSG_RCVD           (1<<3)
#define macMgmtMain_CB_PROC_MSG_RCVD       (1<<4)
#define macMgmtMain_TIMER_EXPIRED          (1<<5)
#define macMgmtMain_TIMER_CALLBACK         (1<<6)

typedef enum
{
    AUTHENTICATE_ACTION,
    AGING_ACTION
} macMgmtMain_Action_e;

/*typedef struct macMgmtMain_PendingData_t */
/*{ */
/*    macMgmtMain_Action_e Action; */
/*    IEEEtypes_MacAddr_t Addr; */
/*    IEEEtypes_AuthType_e AuthType; */
/*    extStaDb_State_e State; */
/*} */
/*macMgmtMain_PendingData_t; */
/* */
/* Structure that is used to record data associated with sent messages */
/* for which there is a response expected. Typically this is the data */
/* that is attached with a timer used to keep from waiting forever for */
/* a response. */
/* */
/*---------------------------------*/
/* IEEE 802.11 Management Messages */
/*---------------------------------*/
#if !defined (PORTABLE_ARCH)
typedef struct PACK_START macmgmtQ_MgmtMsg_t
#else
typedef PACK_START struct macmgmtQ_MgmtMsg_t
#endif
{
#ifdef STA
   w8100_MrvlRxHdr_t     MrvlHdr;
   IEEEtypes_MgmtHdr_t   IeeeHdr;
#else
    IEEEtypes_MgmtHdr_t Hdr;
#endif
#if defined (PORTABLE_ARCH)
    PACK_START
#endif
    union
    {
        IEEEtypes_Bcn_t Bcn;
        IEEEtypes_DisAssoc_t DisAssoc;
        IEEEtypes_AssocRqst_t AssocRqst;
        IEEEtypes_AssocRsp_t AssocRsp;
        IEEEtypes_ReassocRqst_t ReassocRqst;
        IEEEtypes_ReassocRsp_t ReassocRsp;
        IEEEtypes_ProbeRqst_t ProbeRqst;
        IEEEtypes_ProbeRsp_t ProbeRsp;
        IEEEtypes_Auth_t Auth;
        IEEEtypes_Deauth_t Deauth;
    }
    
#if defined (PORTABLE_ARCH)
    PACK_END
#endif    
    Body;
    UINT32 FCS;
} PACK_END macmgmtQ_MgmtMsg_t;

/* Structure used for messages placed on the MAC Management Service */
/* Task's 802.11 message queue. The fields in the structure are: */
/* 1a) MrvlHdr     - The Marvell header portion for 802.11 mgmt msgs */
/*  b) IeeeHdr & Hdr    - The IEEE header portion for 802.11 mgmt msgs */
/* 2a) Bcn         - Beacon message body */
/*  b) DisAssoc    - Disassociate message body */
/*  c) AssocRqst   - Associate request message body */
/*  d) AssocRsp    - Associate response message body */
/*  e) ReassocRqst - Reassociate request message body */
/*  f) ReassocRsp  - Reassociate response message body */
/*  g) ProbeRqst   - Probe request message body */
/*  h) ProbeRsp    - Probe response message body */
/*  i) Auth        - Authentication message body */
/*  j) Deauth      - Deauthentication message body */
/* 3)  FCS         - Frame check sequence */


/*--------------*/
/* SME Commands */
/*--------------*/
typedef struct macmgmtQ_SmeCmd_t
{
    IEEEtypes_SmeCmd_e CmdType;
#ifdef STA   
   host_MsgHdr_t       HostHdr;
#endif
    union
    {
#ifdef STA
      macMgmtQ_ScanCmd_t       ScanCmd;
#else
        IEEEtypes_ScanCmd_t ScanCmd;
#endif
        IEEEtypes_JoinCmd_t JoinCmd;
        IEEEtypes_AuthCmd_t AuthCmd;
        IEEEtypes_DeauthCmd_t DeauthCmd;
        IEEEtypes_AssocCmd_t AssocCmd;
        IEEEtypes_ReassocCmd_t ReassocCmd;
        IEEEtypes_DisassocCmd_t DisassocCmd;
        IEEEtypes_ResetCmd_t ResetCmd;
        IEEEtypes_StartCmd_t StartCmd;
    } Body;
} PACK_END macmgmtQ_SmeCmd_t;
   /* */
/* Structure used for messages placed on the MAC Management Service */
/* Task's SME message queue. The fields in the structure are: */
/* 1)  CmdType     - The type of SME command this message contains */
/* 2a) ScanCmd     - Scan command message body */
/*  b) JoinCmd     - Join command message body */
/*  c) AuthCmd     - Authenticate command message body */
/*  d) DeauthCmd   - Deauthenticate command message body */
/*  e) AssocCmd    - Associate command message body */
/*  f) ReassocCmd  - Reassociate commmand message body */
/*  g) DisassocCmd - Disassociate command message body */
/*  h) ResetCmd    - Reset command message body */
/*  i) StartCmd    - Start command message body */


/*-----------------------*/
/* CB Processor Messages */
/*-----------------------*/
typedef struct macmgmtQ_CbProcDataCmd_t
{
    IEEEtypes_CbProcessorCmd_e CmdType;
    union
    {
        IEEEtypes_PwrMgmtCmd_t PwrMgmtCmd;
        /*TBD cmd for MIB_READ, */
        /*TBD cmd for MIB_WRITE, */
        /*TBD cmd for REGISTER_READ, */
        /*TBD cmd for REGISTER_WRITE, */
        /*TBD cmd for SIGNAL_STRENGTH, */
        /*TBD cmd for STATION_INFO_READ, */
        /*TBD cmd for STATION_INFO_WRITE, */
        /*TBD cmd for STATION_LIST, */
        /*TBD cmd for STATISTICS */
    } Body;
} PACK_END macmgmtQ_CbProcDataCmd_t;
   /* */
/* Structure used for messages placed on the MAC Management Service */
/* Task's CB Processor message queue. The fields in the structure are: */
/* 1)  CmdType     - The type of CB Processor command this message contains */
/* 2a) PwrMgmtCmd  - Scan command message body */
/*  b) JoinCmd     - Join command message body */
/*  c) AuthCmd     - Authenticate command message body */
/*  d) DeauthCmd   - Deauthenticate command message body */
/*  e) AssocCmd    - Associate command message body */
/*  f) ReassocCmd  - Reassociate commmand message body */
/*  g) DisassocCmd - Disassociate command message body */
/*  h) ResetCmd    - Reset command message body */
/*  i) StartCmd    - Start command message body */

/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */



/******************************************************************************
 *
 * Name: macMgmtQ_MgmtWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where IEEE 802.11
 *   management messages are placed for the MAC Management Service Task. If
 *   writing to the queue cannot immediately occur, then the routine returns
 *   with a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMst_p - a pointer to the message to be placed on
 *                           the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS macMgmtQ_MgmtWriteNoBlock( macmgmtQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: macMgmtQ_SmeWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where messages
 *   from the SME task are placed for the MAC Management Service Task. If
 *   writing to the queue cannot immediately occur, then the routine returns
 *   with a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): SmeCmd_p - a pointer to the message to be placed on
 *                          the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS macMgmtQ_SmeWriteNoBlock( macmgmtQ_SmeCmd_t *SmeCmd_p );

/******************************************************************************
 *
 * Name: macMgmtQ_TimerWriteNoBlock
 *
 * Description:
 *   This routine is called to write a timer expired message on the timer
 *   queue used by the MAC Management Service Task. If writing to the queue
 *   cannot immediately occur, then the routine returns with a failure status
 *   (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): TimerMsg_p - Pointer to the message to be placed on
 *                            the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
/*extern WL_STATUS macMgmtQ_TimerWriteNoBlock( macmgmtQ_TimerMsg_t *TimerMsg_p ); */



/******************************************************************************
 *
 * Name: macMgmtQ_CbProcWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where messages
 *   from the CB Processor task are placed for the MAC Management Service
 *   Task. If writing to the queue cannot immediately occur, then the routine
 *   returns with a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): CbProcDataCmd_p - a pointer to the message to be placed on
 *                                 the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS macMgmtQ_CbProcWriteNoBlock(
        macmgmtQ_CbProcDataCmd_t *CbProcDataCmd_p );



/******************************************************************************
 * 
 * Name: macMgmAp_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the MAC Management Service Task 
 *   and related components. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern WL_STATUS macMgmtAp_Init(UINT32 maxStns, IEEEtypes_MacAddr_t *stnMacAddr);

/******************************************************************************
 * 
 * Name: macMgmtAp_Start 
 * 
 * Description: 
 *   This routine is called to start the MAC Management Service Task running. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues have been initialized by calling 
 *   macMgmtAp_Init(). 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern void macMgmtAp_Start( void );


extern BOOLEAN isStrictShareMode(void);
extern BOOLEAN isAuthAlgTypeMatch(UINT8 x);
extern BOOLEAN isWepRequired(void); 

#endif /* _MACMGMTAP_H_ */
