/*******************************************************************************************
 *
* File: mlme.h
 *
*        Header file for AP MLME State Machines
*        Copy Right(2003) Marvell Semiconductors, Inc.
 *
 *
* Author(s):    Binh Tran & Jiaqi Shen
* Date:         2003-06-16
*
*******************************************************************************************/

#ifndef MAC_MLME_
#define MAC_MLME_

#define EVENT_NUM_MAX   64
#define ASSOC_TIMEOUT  5 /*500ms */
#define AUTH_TIMEOUT  5 /*500ms */

#include "hsm.h"
#include "timer.h"
#include "wltypes.h"


#define MLME_SUCCESS	0
#define MLME_INPROCESS 1
#define MLME_FAILURE	-1

/* For Association Services */
/* For Station */
typedef struct AssocSrvSta
{
	Hsm super;
	State Assoc_Srv_Sta;
	State Assoc_Idle, Wait_Assoc_Rsp, Wait_ReAssoc_Rsp;
	Timer timer;
	unsigned char *userdata_p;
}AssocSrvSta;

/* For AP */
typedef struct AssocSrvAp
{
	Hsm super;
	State Assoc_Srv_Ap;
	State Assoc_Idle, Wait_Assoc_Rsp, Wait_ReAssoc_Rsp;
	Timer timer;
	unsigned char *userdata_p;
}AssocSrvAp;


/* For Authentication Services */
enum arAlg_t
{
    open_system,
    shared_key
};

/* For Station*/
typedef struct AuthRspSrvSta
{
	Hsm super;
	State Auth_Rsp_Srv_Sta;
	State Auth_Rsp_Idle, Wait_Chal_Rsp;
	Timer timer;
	unsigned char *userdata_p;
}AuthRspSrvSta;

typedef struct AuthReqSrvSta
{
	Hsm super;
	State Auth_Req_Srv_Sta;
	State Auth_Req_Idle, Wait_Auth_Seq2, Wait_Auth_Seq4;
	Timer timer;
	unsigned char *userdata_p;
}AuthReqSrvSta;


typedef struct PowerSaveMonitor
{
    Hsm super;
    State Power_Save_Monitor, Monitor_Idle;
}PowerSaveMonitor;

/* For AP */
typedef struct AuthRspSrvAp
{
	Hsm super;
	State Auth_Rsp_Srv_Ap;
	State Auth_Rsp_Idle, Wait_Chal_Rsp;
	Timer timer;
	unsigned char *userdata_p;
}AuthRspSrvAp;

typedef struct AuthReqSrvAp
{
	Hsm super;
	State Auth_Req_Srv_Ap;
	State Auth_Req_Idle, Wait_Auth_Seq2, Wait_Auth_Seq4;
    Timer timer;
    unsigned char *userdata_p;
}AuthReqSrvAp;

typedef struct AuthRspSrvApMsg
{
    UINT8       rspMac[8];
    UINT16       arAlg;
	UINT16		arAlg_in;
    UINT8       arSeq;
    UINT8       arSeq2;
    UINT8       dot11Privacy;
    UINT8       *mgtMsg;
}AuthRspSrvApMsg;

/* For Synchronization Services */
enum apOpMode_t
{
    independent,
    infrastructure
};

/* For AP */
typedef struct SyncSrvAp
{
	Hsm super;
	State Sync_Srv_Ap;
	State No_Bss, Bss, Sta_Active;
    Timer timer;
}SyncSrvAp;

typedef struct SyncSrvApMsg
{
    UINT8       opMode;
    UINT8       *mgtMsg;
}SyncSrvApMsg;

/* For STA */

typedef struct SyncSrvSta
{
	Hsm super;
	State Sync_Srv_Sta;
    State Act_Recv, No_Bss, AP_Active, IBss_Active, Pas_Listen, Bss,IBss_Idle;
    State Join_Wait_Beacon, Act_Listen, Wait_Csw_done, Wait_Hop_Bss;
    State Init_Wait_Probe_Delay, Wait_Probe_Delay;
}SyncSrvSta;

/* MLME State Machine Events */
enum MacMgtEvents
{
    AssocReq, 
    AssocRsp, 
    ATim, 
    AuthEven, 
    AuthOdd, 
    Beacon, 
    Cfend, 
    Cls2err, 
    Cls3err, 
    DeAuth, 
    DisAssoc, 
    DsResponse,
    Mlme_Confirms, 
    Mlme_Indications, 
    Mlme_Requests, 
    MlmeAssoc_Cnfm,
    MlmeAssoc_Ind,
    MlmeAssoc_Req, 
    MlmeAuth_Cnfm, 
    MlmeAuth_Ind, 
    MlmeAuth_Req, 
    MlmeDeAuth_Cnfm, 
    MlmeDeAuth_Ind, 
    MlmeDeAuth_Req, 
    MlmeDisAssoc_Cnfm, 
    MlmeDisAssoc_Ind, 
    MlmeDisAssoc_Req, 
    MlmeJoin_Cnfm, 
    MlmeJoin_Req, 
    MlmePwMgt_Cnfm, 
    MlmePwMgt_Req, 
    MlmeReAssoc_Cnfm, 
    MlmeReAssoc_Req, 
    MlmeScan_Cnfm, 
    MlmeScan_Req, 
    MlmeStart_Cnfm, 
    MlmeStart_Req, 
    MmgtConfirmSignals, 
    MmgtIndicationSignals, 
    MmgtRequestSignals, 
    ProbeReq,
    ProbeRsp, 
    PsIndicate,
    PsmDone,
    ReAssocReq, 
    ReAssocRsp, 
    ResetMAC,
    Send, 
    Sent, 
    Sst, 
    StaState, 
    SwDone,
    Tatim,
    Tbcn,
    Tmocp,
    Tpdly,
    Tscan,
    Xport 
};


#endif /* MAC_MLME_ */
