#ifndef _TRACE_H_
#define _TRACE_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    trace.h
 * \brief   Interface for syslog-like trace facility.
 *
 *          This trace facility assume that a TCP/IP stack exists.
 */

/* #define ENABLE_SYSLOG   1 */

#ifdef ENABLE_SYSLOG
extern void syslog(int, const char *, ... );

#define SYSLOG_PORT  514
extern unsigned char syslogOn;
extern unsigned int slHost;
#define SetSyslog(x)    (syslogOn = x)
#define SetSyslogIP(x)  (slHost = x)

#define LOG_EMERG   0   /* kernel panic */
#define LOG ALERT   1   /* condition needing immediate attention */
#define LOG_CRIT    2   /* critical conditions */
#define LOG_ERR     3   /* errors */
#define LOG_WARNING 4   /* warning messages */
#define LOG_NOTICE  5   /* not an error, but may need attention */
#define LOG_INFO    6   /* informational messages */
#define LOG_DEBUG   7   /* when debugging a system */
#else
#define LOCAL_TRACE_BUFFER  1
#ifndef EAGLE_SIZE_OF_CMD_BUFFER
#define EAGLE_SIZE_OF_CMD_BUFFER    0x00000c00  /* 3K, see driver dsdef.h */
#endif

#define TRACE_TAG   "TTT"
#define TRACE_TAG_SIZE  strlen(TRACE_TAG)
#define MAX_TRACE_MSGS  8

extern int check_trace(char *cmd, int size);
#endif

#define MAX_TRACE_LINE  128

#define isdigit(c)  (c >= '0' && c <= '9')
#define ROUND32(x) ( ((long)(x)&0x3)? ((long)(x)&0xffffffc0)+4 : (long)(x) )

extern int debug;
extern void DBGPRINT(const char *format, ... );

#endif  /* _TRACE_H_ */

