#if !defined(_WL_DEVICE_H_)
#define _WL_DEVICE_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    wl_mib.h
 * \brief   This file contains the MIB structure definitions based on IEEE 802.11 specification.
 *
*/


/*============================================================================= */
/*                    MARVELL DEVICE I/O STRUCTURES (IEEE 802.11) */
/*============================================================================= */

/*-----------------------------*/
/* Device Configuration Table */
/*-----------------------------*/
typedef enum _BUF_OWNERSHIP
{
    OWNER_USER = 0,
    OWNER_DEVICE
} BUF_OWNERSHIP;

typedef struct _BUF_DESC
{
    UINT32 owner;       /* buffer ownership */
    PVOID  control;     /* control header */
    UINT8* buf;
    UINT32 dataSize;
    _BUF_DESC * next;
} BUF_DESC;

typedef struct _WL_DEVICE_CONFIG
{
    UINT32 recvBufferSize;     /* buffer size: collection of buffers contains multiple packets */
    UINT8* recvBuffer;         /* BIG buffer */

    /* service routines that the device user fills in */
    BOOLEAN (*data_ready)(UINT32 QosId, UINT32 pktNum);

    /* service routines that the device driver fills in */

    /* user provids descriptors without data buffer, and device provides data buffer in the descriptor, */
    /* read buffer is released by user marking ownership to the device */
    BOOLEAN (*read)(BUF_DESC * desc, UINT32 QosId, UINT32 *pktNum);

    /* user provider data buffer in the descriptor */
    /* write buffer is released by device marking ownership to the user */
    BOOLEAN (*write)(BUF_DESC * desc, UINT32 QosId, UINT32 *pktNum);

    BOOLEAN (*ioctl)(UINT32 code, UINT32 param1, PVOID param2);
}
WL_DEVICE_CONFIG;

#endif /* _WL_DEVICE_H_ */

