#if !defined(_WL_DRIVER_H_)
#define _WL_DRIVER_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    wl_mib.h
 * \brief   This file contains the MIB structure definitions based on IEEE 802.11 specification.
 *
*/


/*============================================================================= */
/*                    MARVELL DEVICE I/O STRUCTURES (IEEE 802.11) */
/*============================================================================= */

/*-----------------------------*/
/* Command and Status          */
/*-----------------------------*/
#define OWNER_DEVICE    0x80000000
#define OWNER_HOST      0x00000000

/*-----------------------------*/
/* Device Configuration Table */
/*-----------------------------*/

/*-----------------------------*/
/* Macros for device io        */
/*-----------------------------*/
#define SET_OWNER_DEVICE(bp) ((bp)->command |= OWNER_DEVICE)
#define SET_OWNER_HOST(bp) ((bp)->command &= ~OWNER_DEVICE)
#define IS_OWNER_DEVICE(bp) ((bp)->command & OWNER_DEVICE)
#define IS_OWNER_HOST(bp) (!IS_OWNER_DEVICE(bp))

/* structure passed in to driver init routines.
 * device driver should provide following routines.
 * 
 * Returns the size of global driver specific space needed for the driver to run.
 * BOOLEAN xxx_getPrivateSpaceReq(UINT32 * privateSize)
 *
 * BOOLEAN xxx_init(WL_DEVICE_CONFIG *)
 */
typedef struct _WL_DEVICE_CONFIG
{
    /* buffers are all provided thru PKTQ/BUFMGT service */
    QHANDLE handle;
    /* service routines that the device user fills in */
    BOOLEAN (*data_ready)(UINT32 QosId, UINT32 pktNum);
    BOOLEAN (*tx_done)(UINT32 QosId, UINT32 pktNum);
    BOOLEAN (*evt_notify)(UINT32 evtId, PVOID evtData);

    /* service routines that the device driver fills in */

    /* 
     * user provids descriptors without data buffer, and device provides data buffer in the descriptor, 
     * read buffer is released by user marking ownership to the device.
     *
     * Read operation is non-blocking service as the data has been transferred before driver
     * call data_ready routine.
     */
    BOOLEAN (*read)(IO_BUF_DESC * desc, UINT32 *QosId, UINT32 *pktNum);

    /* user provider data buffer in the descriptor
     * write buffer is released by device marking ownership to the user
     * Write operation is non-blocking service as the driver will queue the data for transfer.
     * Driver will call tx_done after at least one packet has been transferred.
     */
    BOOLEAN (*write)(IO_BUF_DESC *desc, UINT32 *QosId, UINT32 *pktNum);

    BOOLEAN (*ioctl)(UINT32 code, UINT32 param1, PVOID param2);
} WL_DEVICE_CONFIG;

#endif /* _WL_DRIVER_H_ */

