#if !defined(_CF_H_)
#define _CF_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */


/*!
 * \file    cf.h
 * \brief   This file contains the Compact Flash host interface unit registers for the 88W8100 hardware.
 *
 */


/*--------------------------------------------------------------*/
/*  CF/PCCard Host Interface (_PMI_) registers                  */
/*--------------------------------------------------------------*/
/* Host Control Registers (HCR) Offset */
/* */
#define WL_CF_HCR_HOST_STATUS                    ((WL_HIU_CFG_BASE)|0x00000000)
#define WL_CF_HCR_CARD_INT_CAUSE                 ((WL_HIU_CFG_BASE)|0x00000002)
#define WL_CF_HCR_HOST_INT_MASK                  ((WL_HIU_CFG_BASE)|0x00000004)
#define WL_CF_HCR_READ_BASE_LOW                  ((WL_HIU_CFG_BASE)|0x00000008)
#define WL_CF_HCR_READ_BASE_HIGH                 ((WL_HIU_CFG_BASE)|0x0000000A)
#define WL_CF_HCR_WRITE_BASE_LOW                 ((WL_HIU_CFG_BASE)|0x0000000C)
#define WL_CF_HCR_WRITE_BASE_HIGH                ((WL_HIU_CFG_BASE)|0x0000000E)
#define WL_CF_HCR_IO_READ_PORT                   ((WL_HIU_CFG_BASE)|0x00000010)
#define WL_CF_HCR_IO_CMD_READ_PORT               ((WL_HIU_CFG_BASE)|0x00000012)
#define WL_CF_HCR_IO_WRITE_LEN                   ((WL_HIU_CFG_BASE)|0x00000014)
#define WL_CF_HCR_IO_WRITE_PORT                  ((WL_HIU_CFG_BASE)|0x00000016)
#define WL_CF_HCR_IO_CMD_WRITE_LEN               ((WL_HIU_CFG_BASE)|0x00000018)
#define WL_CF_HCR_IO_CMD_WRITE_PORT              ((WL_HIU_CFG_BASE)|0x0000001A)

#define WL_CF_HCR_SCRATCH_PORT                   ((WL_HIU_CFG_BASE)|0x0000003F)
#define WL_CF_HCR_CIS_ADDR_PORT                  ((WL_HIU_CFG_BASE)|0x00000046)
#define WL_CF_HCR_CIS_DATA_PORT                  ((WL_HIU_CFG_BASE)|0x00000048)
#define WL_CF_HCR_IO_GBUS_REG_READ               ((WL_HIU_CFG_BASE)|0x0000004C)
#define WL_CF_HCR_IO_GBUS_REG_WRITE              ((WL_HIU_CFG_BASE)|0x0000004E)

/* Card Control Registers (CCR) Offset */
/* */
#define WL_CF_CCR_PRODUCT_ID                     ((WL_HIU_CFG_BASE)|0x0000001C)
#define WL_CF_CCR_CARD_STATUS                    ((WL_HIU_CFG_BASE)|0x00000020)
#define WL_CF_CCR_HOST_INT_CAUSE                 ((WL_HIU_CFG_BASE)|0x00000022)
#define WL_CF_CCR_IO_READ_LEN                    ((WL_HIU_CFG_BASE)|0x00000024)
#define WL_CF_CCR_SQ_READ_BASE_LOW               ((WL_HIU_CFG_BASE)|0x00000028)
#define WL_CF_CCR_SQ_READ_BASE_HIGH              ((WL_HIU_CFG_BASE)|0x0000002A)
#define WL_CF_CCR_SQ_WRITE_BASE_LOW              ((WL_HIU_CFG_BASE)|0x0000002C)
#define WL_CF_CCR_SQ_WRITE_BASE_HIGH             ((WL_HIU_CFG_BASE)|0x0000002E)
#define WL_CF_CCR_IO_CMD_READ_LEN                ((WL_HIU_CFG_BASE)|0x00000030)
#define WL_CF_CCR_SQ_CMD_READ_BASE_LOW           ((WL_HIU_CFG_BASE)|0x00000034)
#define WL_CF_CCR_SQ_CMD_READ_BASE_HIGH          ((WL_HIU_CFG_BASE)|0x00000036)
#define WL_CF_CCR_SQ_CMD_WRITE_BASE_LOW          ((WL_HIU_CFG_BASE)|0x00000038)
#define WL_CF_CCR_SQ_CMD_WRITE_BASE_HIGH         ((WL_HIU_CFG_BASE)|0x0000003A)
#define WL_CF_CCR_CFG_REG_BASE_ADR               ((WL_HIU_CFG_BASE)|0x0000003C)
#define WL_CF_CCR_CARD_CFG                       ((WL_HIU_CFG_BASE)|0x0000003E)
#define WL_CF_CCR_SCRATCH_PORT                   ((WL_HIU_CFG_BASE)|0x0000003F)
#define WL_CF_CCR_TX_FRAME_SEQ_NUM               ((WL_HIU_CFG_BASE)|0x00000040)
#define WL_CF_CCR_TX_FRAME_STATUS                ((WL_HIU_CFG_BASE)|0x00000042)
#define WL_CF_CCR_CARD_INT_MASK                  ((WL_HIU_CFG_BASE)|0x00000044)
#define WL_CF_CCR_CIS_ADR_PORT                   ((WL_HIU_CFG_BASE)|0x00000046)
#define WL_CF_CCR_CIS_DATA_PORT                  ((WL_HIU_CFG_BASE)|0x00000048)

/* Bit definition for WL_CF_HCR_HOST_STATUS (Host Status) */
#define HCR_HS_TxDnLdOvr                   (1<<0)
#define HCR_HS_RxUpLdOvr                   (1<<1)
#define HCR_HS_CmdDnLdOvr                  (1<<2)
#define HCR_HS_PwrDown                     (1<<3)
#define HCR_HS_HstEvent                    (1<<4)
#define HCR_HS_FlushDataFifo               (1<<5)
#define HCR_HS_MASK                        0xFF3F

/* Bit definition for WL_CF_HCR_CARD_INT_CAUSE (Card Interrupt Cause) */
#define HCR_CIC_TxDnLdOvr                  (1<<0)
#define HCR_CIC_RxUpLdOvr                  (1<<1)
#define HCR_CIC_CmdDnLdOvr                 (1<<2)
#define HCR_CIC_PwrDown                    (1<<3)
#define HCR_CIC_HstEvent                   (1<<4)
#define HCR_CIC_MASK                       0x001F

/* Bit definition for WL_CF_HCR_HOST_INT_MASK (Host Interrupt Mask) */
#define HCR_HIM_TxDnLdRdy                  (1<<0)
#define HCR_HIM_RxUpLdRdy                  (1<<1)
#define HCR_HIM_CmdDnLdRdy                 (1<<2)
#define HCR_HIM_CmdRspRdy                  (1<<3)
#define HCR_HIM_CardEvent                  (1<<4)
#define HCR_HIM_MASK                       0x001F

/* Bit definition for WL_CF_CCR_CARD_STATUS (Card Status) */
#define CCR_CS_TxDnLdRdy                   (1<<0)
#define CCR_CS_RxUpLdRdy                   (1<<1)
#define CCR_CS_CmdDnLdRdy                  (1<<2)
#define CCR_CS_CmdRspRdy                   (1<<3)
#define CCR_CS_CardEvent                   (1<<4)
#define CCR_CS_PwrDwn                      (1<<8)
#define CCR_CS_MACErr                      (1<<9)
#define CCR_CS_802_11LnkUp                 (1<<10)
#define CCR_CS_10btLnkUp                   (1<<11)
#define CCR_CS_100btLnkUp                  (1<<12)
#define CCR_CS_LnkStsChg                   (1<<13)
#define CCR_CS_TxFrmStsChg                 (1<<14)
#define CCR_CS_ReadCISRdy                  (1<<15)
#define CCR_CS_MASK                        0xFF1F

/* Bit definition for WL_CF_CCR_HOST_INT_CAUSE (Host Interrupt Cause) */
#define CCR_HIC_TxDnLdRdy                  (1<<0)
#define CCR_HIC_RxUpLdRdy                  (1<<1)
#define CCR_HIC_CmdDnLdRdy                 (1<<2)
#define CCR_HIC_CmdRspRdy                  (1<<3)
#define CCR_HIC_CardEvent                  (1<<4)
#define CCR_HIC_MASK                       0x001F

/* Bit definition for WL_CF_CCR_TX_FRAME_STATUS (Tx Frame Status) */
#define CCR_TFS_SqDmaOvr                   (1<<0)
#define CCR_TFS_TxFrmSent                  (1<<1)
#define CCR_TFS_RtyLmtExcd                 (1<<2)
#define CCR_TFS_TimeOut                    (1<<3)
#define CCR_TFS_MASK                       0x000F

/* Bit definition for WL_CF_CCR_CARD_INT_MASK (Card Interrupt Mask) */
#define CCR_CIM_TxDnLdOvr                  (1<<0)
#define CCR_CIM_RxUpLdOvr                  (1<<1)
#define CCR_CIM_CmdDnLdOvr                 (1<<2)
#define CCR_CIM_PwrDown                    (1<<3)
#define CCR_CIM_HstEvent                   (1<<4)
#define CCR_CIM_MASK                       0x001F

/* Define Host to card event code used by WL_CF_HCR_SCRATCH_PORT */
#define HCR_SP_NONE                        0x00
#define HCR_SP_HOST_INIT_IN_PROGRESS       0x01
#define HCR_SP_HOST_INIT_FINISHED          0x02
#define HCR_SP_HOST_HALT_IN_PROGRESS       0x81

/* Memory */
#define CF_CIS_SQ_BASE_OFFSET		0xC0000000

#define CF_SQ_DATA_RD_LOW			0x0800	/*2K */
#define CF_SQ_DATA_RD_HIGH		   0xC000
#define CF_SQ_DATA_WR_LOW			0x1000	/*4K */
#define CF_SQ_DATA_WR_HIGH		   0xC000
#define CF_SQ_CMD_RD_LOW			0x1800	/*6K */
#define CF_SQ_CMD_RD_HIGH			0xC000
#define CF_SQ_CMD_WR_LOW			0x2000	/*8K */
#define CF_SQ_CMD_WR_HIGH			0xC000

#define CF_CIS_ADDR_PORT_BASE		0x00

#endif /* _CF_H_ */

