/******************** (c) Marvell Semiconductor, Inc., 2003 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/wlandrv/src/hal/w8310/etherpriv.h,v 1.15 2003/06/25 17:18:09 cfan Exp $
*
* Purpose:
*    This file contains low level definitions of ethernet driver.
*
* Public Procedures:
*    None.
*
* Notes:
*
*****************************************************************************/

#ifndef _ETHERPRIV_H_
#define _ETHERPRIV_H_
#include "wltypes.h"
#define  LE

/* Number of ethernet ports. */
#define NUM_OF_ETHERNET_PORTS 1

#define NUM_TX_QUEUES 2
#define NUM_RX_QUEUES 1

/****************************************************/
/*          Ethernet Register Definitions			*/
/****************************************************/

/* Port status register bits offset in register */
#define LINK_UP_BIT     (1<<3)

/* this macros are used to enable access to SDMA_CONFIGURATION */
#define SINGLE_FRAME_MODE               (1<<1)
#define RECEIVE_FIFO_THRESHOLD          1
#define BURST_SIZE                      (3<<12)

/* These macros describes the Ethernet Port Configuration Register bits		  */
#define	ETH_PROMISCUOUS_MODE	(1<<0 ) /* Enable promiscuous mode 			  */
#define REJECT_BROADCAST		(1<<1 ) /* Reject Broadcast Mode 			  */
#define PASS_BAD_FRAMES			(1<<2 ) /* Pass Bad Frames 					  */
#define PORT_ENABLE				(1<<7 ) /* Port enable 						  */
#define FORCE_COLLISION			(1<<10) /* Force Collision  	  			  */
#define ETH_HASH_SIZE_500B		(1<<12) /* Hash table size 0 - 8K, 1 - 0.5k   */
#define ETH_HASH_FUNCTION_1		(1<<13) /* Hash function 1 or function 0 	  */
#define ETH_HASH_PASS_NOT_FOUND (1<<14) /* Hash function operation mode 	  */
#define DUPLEX_MODE_FULL    	(1<<15) /* Valid when duplex auto-neg disabled*/
#define ACCS					(1<<31) /* Accelerate Slot Time 	  		  */


/* These macros describes the Ethernet Port Configuration Extend Register bits*/
#define IGMP_ENABLE			(1<<0 ) /* IGMP Packets Capture Enable 			  */
#define SPAN_ENABLE			(1<<1 ) /* Spanning Tree Packets Capture Enable   */
#define PARTITION_ENABLE	(1<<2 ) /* Enable partition 					  */
#define RX_PRIO_OVERRIDE	(1<<8 ) /* Override Priority Rx on this Port  	  */
#define AUTO_NEG_DUPLEX_DIS	(1<<9 ) /* Auto-negotiation for Duplex Mode 	  */
#define AUTO_NEG_FLOW_DIS	(1<<10) /* Auto-neg for 802.3x Flow-control 	  */
#define FORCE_LINK_NO		(1<<11) /* Force Link Pass 						  */
#define FLOW_CTRL_DISABLE	(1<<12) /* Enable Flow-Control Mode 			  */
#define MFL_1518_BYTES		(0<<14) /* Maximum packet allowed for reception   */
#define MFL_1536_BYTES		(1<<14) /* Maximum packet allowed for reception   */
#define MFL_2048_BYTES		(2<<14) /* Maximum packet allowed for reception   */
#define MFL_64K_BYTES		(3<<14) /* Maximum packet allowed for reception   */
#define MIB_CLEAR_MODE		(1<<16) /* MIB Counters Clear Mode 				  */
#define PORT_SPEED_100		(1<<18) /* Port Speed 							  */
#define AUTO_NEG_SPEED_DIS	(1<<19) /* Enable Auto-negotiation for Speed 	  */
#define RMII_ENABLE			(1<<20) /* RMII enable 							  */
#define DSCP_ENABLE			(1<<21) /* DSCP enable 							  */
#define TWO_BYTE_STUFF_MODE (1<<28) /* Two byte stuff mode.                   */

/* SDMA Configuration Register (SDCR) */
#define SDCR_RETX_COUNT_UNL (0<<0)
#define SDCR_RETX_COUNT_15  (0xf<<2)
#define SDCR_BLMR_LITTLE    (1<<6)
#define SDCR_BLMT_LITTLE    (1<<7)
#define SDCR_POVR           (1<<8)  /* Enable PCI Override */
#define SDCR_RIFB           (1<<9)  /* Receive Interrupt on Frame Boundaries  */
#define SDCR_BSZ_1x64       (0<<12) /* maximum burst size of SDMA transactions*/
#define SDCR_BSZ_2x64       (1<<12) /* maximum burst size of SDMA transactions*/
#define SDCR_BSZ_4x64       (2<<12) /* maximum burst size of SDMA transactions*/
#define SDCR_BSZ_8x64       (3<<12) /* maximum burst size of SDMA transactions*/


/* Describes Ethernet SDMA Command register options. */
#define ETH_ENABLE_RX_DMA   (1<<7 )
#define ETH_ABORT_RECEIVE   (1<<15)
#define ETH_STOP_TX_HIGH    (1<<16)
#define ETH_STOP_TX_LOW     (1<<17)
#define ETH_START_TX_HIGH   (1<<23)
#define ETH_START_TX_LOW    (1<<24)
#define ETH_ABORT_TRANSMIT  (1<<31)

/* These macros describes the Ethernet SMI Register bits */
#define SMI_WRITE_OPCODE (0<<26) /* 0 - Write, 1 - Read  					  */
#define SMI_READ_OPCODE	 (1<<26) /* 0 - Write, 1 - Read  					  */
#define SMI_READ_VALID 	 (1<<27) /* Indicates completion of Read operation    */
#define SMI_BUSY         (1<<28) /* Indicates that an operation is in progress*/

#define SMI_OP_READ       1
#define SMI_OP_WRITE      0
#define SMI_TIME_OUT      10000 /* in 100ms units */

#define SMI_OP_CODE_BIT_READ                    1
#define SMI_OP_CODE_BIT_WRITE                   0


/* SDMA command status fields macros */
#define ETH_UNDERRUN_ERROR               (1<<6)
#define ETH_ERROR_SUMMARY                (1<<15)
#define ETH_LAST_DESC                    (1<<16)
#define ETH_FIRST_DESC                   (1<<17)
#define ETH_ZERO_PADDING	             (1<<18)
#define ETH_GEN_CRC                      (1<<22)
#define ETH_ENABLE_INTERRUPT             (1<<23)
#define ETH_BUFFER_OWNED_BY_DMA          (1<<31)

/* Ethernet Interrupt cause register bit defines. */

#define ETH_CAUSE_RX_BUFFER_RETURN  (1<<0)  /* Rx Buffer Return             */
#define ETH_CAUSE_RSRVD_L01         (1<<1)
#define ETH_CAUSE_TX_BUFFER_HIGH    (1<<2)  /* Tx Buffer for High prio Q    */
#define ETH_CAUSE_TX_BUFFER_LOW     (1<<3)  /* Tx Buffer for low  prio Q    */
#define ETH_CAUSE_RSRVD_L04         (1<<4)
#define ETH_CAUSE_RSRVD_L05         (1<<5)
#define ETH_CAUSE_TX_END_HIGH       (1<<6)  /* Tx End for High Prio Q       */
#define ETH_CAUSE_TX_END_LOW        (1<<7)  /* Tx End for Low  Prio Q       */
#define ETH_CAUSE_RX_ERROR          (1<<8)  /* Rx Resource Err              */ 
#define ETH_CAUSE_RSRVD_L09         (1<<9)
#define ETH_CAUSE_TX_ERROR_HIGH     (1<<10) /* Tx Resource Err for High Prio Q */
#define ETH_CAUSE_TX_ERROR_LOW      (1<<11) /* Tx Resource Err for Low Prio Q  */
#define ETH_CAUSE_RX_OVERRUN        (1<<12) /* Rx Overrun                   */
#define ETH_CAUSE_TX_UNDERRUN       (1<<13) /* Tx Underrun                  */
#define ETH_CAUSE_RSRVD_L14         (1<<14)
#define ETH_CAUSE_RSRVD_L15         (1<<15)
#define ETH_CAUSE_RX_BUFFER_QUEUE_0 (1<<16) /* Rx Buffer Ret in Prio Q[0]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_1 (1<<17) /* Rx Buffer Ret in Prio Q[1]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_2 (1<<18) /* Rx Buffer Ret in Prio Q[2]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_3 (1<<19) /* Rx Buffer Ret in Prio Q[3]   */
#define ETH_CAUSE_RX_ERROR_QUEUE_0  (1<<20) /* Rx Resource Err in Prio Q[0] */
#define ETH_CAUSE_RX_ERROR_QUEUE_1  (1<<21) /* Rx Resource Err in Prio Q[1] */
#define ETH_CAUSE_RX_ERROR_QUEUE_2  (1<<22) /* Rx Resource Err in Prio Q[2] */
#define ETH_CAUSE_RX_ERROR_QUEUE_3  (1<<23) /* Rx Resource Err in Prio Q[3] */
#define ETH_CAUSE_RSRVD_L24         (1<<24)
#define ETH_CAUSE_RSRVD_L25         (1<<25)
#define ETH_CAUSE_RSRVD_L26         (1<<26)
#define ETH_CAUSE_RSRVD_L27         (1<<27)
#define ETH_CAUSE_MII_PHY_STC       (1<<28) /* MII PHY Status Change */
#define ETH_CAUSE_SMI_DONE          (1<<29) /* SMI Command Done      */
#define ETH_CAUSE_RSRVD_L30         (1<<30)
#define ETH_CAUSE_INT_SUM           (1<<31) /* Ethernet Interrupt Summary */

#ifdef _EAGLEHW_H_ 
#define GT_REG_READ(x,y)   W81_READ_REGS32(x, *(y))
#define GT_REG_WRITE       W81_WRITE_REGS32
#else
#define GT_REG_READ(x,y)   ((*(y)) = (*(volatile unsigned long *)(x)))
#define GT_REG_WRITE(x,y)  ((*(volatile unsigned long *)(x)) = y)
#endif

#define 		TIME_OUT					10  /* 1/6 SEC */

#ifdef BE /* Big Endian */
#define SHORT_SWAP(X) (X)
#define WORD_SWAP(X) (X)
#define LONG_SWAP(X) ((l64)(X))

#else    /* Little Endian */

#define SHORT_SWAP(X) ((X <<8 ) | (X >> 8))

#define WORD_SWAP(X) (((X)&0xff)<<24)+      \
                    (((X)&0xff00)<<8)+      \
                    (((X)&0xff0000)>>8)+    \
                    (((X)&0xff000000)>>24)

#define LONG_SWAP(X) ( (l64) (((X)&0xffULL)<<56)+               \
                            (((X)&0xff00ULL)<<40)+              \
                            (((X)&0xff0000ULL)<<24)+            \
                            (((X)&0xff000000ULL)<<8)+           \
                            (((X)&0xff00000000ULL)>>8)+         \
                            (((X)&0xff0000000000ULL)>>24)+      \
                            (((X)&0xff000000000000ULL)>>40)+    \
                            (((X)&0xff00000000000000ULL)>>56))
#endif

#define     PHY_ADD_REG_VALUE    0x1ee


#define MAX_NUMBER_OF_ETHERNET_PORTS            2

#define MRR_REG_VALUE                           0x00fc0000


#define LL_QUEUE_PRIORITY                       1
#define L_QUEUE_PRIORITY                        2
#define H_QUEUE_PRIORITY                        3
#define HH_QUEUE_PRIORITY                       4


/* Ethernet port register offset addresses. */

#define ETHERNET0_CAUSE_REGISTER        (0x0450)
#define ETHERNET0_MASK_REGISTER         (0x0458)
#define ETHERNET1_CAUSE_REGISTER        (0x0850)
#define ETHERNET1_MASK_REGISTER         (0x0858)
#define SDMA_CAUSE_REGISTER             (0x103A10)
#define SDMA_MASK_REGISTER              (0x103A90)


/* Ethernet Ports */
#define ETHER_PHY_ADDR_REG              (0x80008000)
#define ETHER_SMI_REG                   (0x80008010)

/* Ethernet0 Ports */
#define ETHER0_PORT_CONFIG_REG          (0x0400)
#define ETHER0_PORT_CONFIG_EXT_REG      (0x0408)
#define ETHER0_PORT_COMM_REG            (0x0410)
#define ETHER0_PORT_STATUS_REG          (0x0418)
#define ETHER0_SER_PARAM_REG            (0x0420)
#define ETHER0_HASH_TBL_PTR_REG         (0x0428)
#define ETHER0_FLOW_CNTRL_SRC_ADDR_L    (0x0430)
#define ETHER0_FLOW_CNTRL_SRC_ADDR_H    (0x0438)
#define ETHER0_SDMA_CONFIG_REG          (0x0440)
#define ETHER0_SDMA_COMM_REG            (0x0448)
#define ETHER0_INT_CAUSE_REG            (0x0450)
#define ETHER0_INT_MASK_REG             (0x0458)

#define ETHER0_TOS_PRIORITY_0_LOW       (0x0460)
#define ETHER0_TOS_PRIORITY_0_HIGH      (0x0464)
#define ETHER0_TOS_PRIORITY_1_LOW       (0x0468)
#define ETHER0_TOS_PRIORITY_1_HIGH      (0x046c)
#define ETHER0_VLAN_TO_PRIORITY         (0x0470)

#define ETHER0_1ST_RX_DESC_PTR0         (0x0480)
#define ETHER0_1ST_RX_DESC_PTR1         (0x0484)
#define ETHER0_1ST_RX_DESC_PTR2         (0x0488)
#define ETHER0_1ST_RX_DESC_PTR3         (0x048C)
#define ETHER0_CURR_RX_DESC_PTR0        (0x04A0)
#define ETHER0_CURR_RX_DESC_PTR1        (0x04A4)
#define ETHER0_CURR_RX_DESC_PTR2        (0x04A8)
#define ETHER0_CURR_RX_DESC_PTR3        (0x04AC)
#define ETHER0_CURR_TX_DESC_PTR0        (0x04E0)
#define ETHER0_CURR_TX_DESC_PTR1        (0x04E4)
#define ETHER0_MIB_COUNT_BASE           (0x0500)

#define ETHER_MIB_BYTES_RXED                          0x0500
#define ETHER_MIB_BYTES_TXED                          0x0504
#define ETHER_MIB_PACKETS_RXED                        0x0508
#define ETHER_MIB_PACKETS_TXED                        0x050C
#define ETHER_MIB_TOTAL_BYTES_RXED                    0x0510
#define ETHER_MIB_TOTAL_PACKETS_RXED                  0x0514
#define ETHER_MIB_BROADCAST_PACKETS_RXED              0x0518
#define ETHER_MIB_MULTICAST_PACKETS_RXED              0x051C
#define ETHER_MIB_PACKETS_WITH_CRC_ERR_RXED           0x0520
#define ETHER_MIB_OVERSIZE_PACKETS_RXD                0x0524
#define ETHER_MIB_UNDERSIZE_PACKETS_WITH_CRC_ERR_RXED 0x0528
#define ETHER_MIB_OVERSIZE_PACKET_CRC_ERR_RXED        0x052C
#define ETHER_MIB_COLLISIONS_DETECTED_RXED            0x0530
#define ETHER_MIB_LATE_COLLISIONS_DETECTED_RXED       0x0534
#define ETHER_MIB_64BYTE_PACKETS_RXED_AND_TXED        0x0538
#define ETHER_MIB_65_127_PACKETS_RXED_AND_TXED        0x053C
#define ETHER_MIB_128_255_PACKETS_RXED_AND_TXED       0x0540
#define ETHER_MIB_256_511_PACKETS_RXED_AND_TXED       0x0544
#define ETHER_MIB_512_1023_PACKETS_RXED_AND_TXED      0x0548
#define ETHER_MIB_1024_MAX_PACKETS_RXED_AND_TXED      0x054C
#define ETHER_MIB_RX_ERRORS_DETECTED                  0x0550
#define ETHER_MIB_DROPPED_PACKETS                     0x0554
#define ETHER_MIB_MULTICAST_PACKETS_TXED              0x0558
#define ETHER_MIB_BROADCAST_PACKETS_TXED              0x055C
#define ETHER_MIB_UNDERSIZE_PACKETS_RXED              0x0560


/* Ethernet1 Ports */
#define ETHER1_PORT_CONFIG_REG          (0x0800)
#define ETHER1_PORT_CONFIG_EXT_REG      (0x0808)
#define ETHER1_PORT_COMM_REG            (0x0810)
#define ETHER1_PORT_STATUS_REG          (0x0818)
#define ETHER1_SER_PARAM_REG            (0x0820)
#define ETHER1_HASH_TBL_PTR_REG         (0x0828)
#define ETHER1_FLOW_CNTRL_SRC_ADDR_L    (0x0830)
#define ETHER1_FLOW_CNTRL_SRC_ADDR_H    (0x0838)
#define ETHER1_SDMA_CONFIG_REG          (0x0840)
#define ETHER1_SDMA_COMM_REG            (0x0848)
#define ETHER1_INT_CAUSE_REG            (0x0850)
#define ETHER1_INT_MASK_REG             (0x0858)
#define ETHER1_1ST_RX_DESC_PTR0         (0x0880)
#define ETHER1_1ST_RX_DESC_PTR1         (0x0884)
#define ETHER1_1ST_RX_DESC_PTR2         (0x0888)
#define ETHER1_1ST_RX_DESC_PTR3         (0x088C)
#define ETHER1_CURR_RX_DESC_PTR0        (0x08A0)
#define ETHER1_CURR_RX_DESC_PTR1        (0x08A4)
#define ETHER1_CURR_RX_DESC_PTR2        (0x08A8)
#define ETHER1_CURR_RX_DESC_PTR3        (0x08AC)
#define ETHER1_CURR_TX_DESC_PTR0        (0x08E0)
#define ETHER1_CURR_TX_DESC_PTR1        (0x08E4)
#define ETHER1_MIB_COUNT_BASE           (0x0900)



#define NUMBER_OF_ETHERNET_RX_PRIO              4
#define NUMBER_OF_ETHERNET_TX_PRIO              2

#define PHY_ADD0                                0x1C
#define PHY_ADD1                                0x1D

/* this macros are used to enable access to ETHERNET_SDCMR */
#define START_TX_HIGH       1<<23
#define START_TX_LOW        1<<24
#define ENABLE_RX_DMA       1<<7
#define ABORT_RECEIVE       1<<15
#define STOP_TX_HIGH        1<<16
#define STOP_TX_LOW         1<<17
#define ABORT_TRANSMIT      1<<31

/* address and hash values. */
#define ADDRESS_TABLE_ALIGNMENT     0x8
#define ADDRESS_TABLE_SIZE          (16*1024)
#define MAC_ENTRY_SIZE              8
#define MAC_ADDRESS_STRING_SIZE     12

#define RX_MATCHED_FRAMES           1
#define DISCARD_MATCHED_FRAMES      0

#define SKIP_ADDR_TABLE_ENTRY       1
#define DONOT_SKIP_ADDR_TABLE_ENTRY 0

/* hash table functions. */

#define NIBBLE_SWAPPING_16_BIT(X) (((X&0xf)<<4) | ((X&0xf0)>>4) |\
        ((X&0xf00)<<4) | ((X&0xf000)>>4))
        
#define NIBBLE_SWAPPING_32_BIT(X) (((X&0xf)<<4) | ((X&0xf0)>>4) |\
        ((X&0xf00)<<4) | ((X&0xf000)>>4) | ((X&0xf0000)<<4) | ((X&0xf00000)>>4)\
         | ((X&0xf000000)<<4) | ((X&0xf0000000)>>4))
         
#define GT_NIBBLE(X) ((X&0x1)<<3)+((X&0x2)<<1)+((X&0x4)>>1)+((X&0x8)>>3)

#define FLIP_6_BITS(X) (((X&0x1)<<5)+((X&0x2)<<3)+((X&0x4)<<1)+\
        ((X&0x8)>>1)+((X&0x10)>>3)+((X&0x20)>>5))
        
#define FLIP_9_BITS(X) (((X&0x1)<<8)+((X&0x2)<<6)+((X&0x4)<<4)+\
        ((X&0x8)<<2)+((X&0x10)<<0)+((X&0x20)>>2)+((X&0x40)>>4)+\
        ((X&0x80)>>6)+((X&0x100)>>8))

#define HASH_DEFAULT_MODE           14
#define HASH_MODE                   13
#define HASH_SIZE                   12
#define PROMISCUOUS_MODE            0

#define VALID                       1
#define SKIP                        1<<1
#define SKIP_BIT                    1
#define HOP_NUMBER                  12
#define EIGHT_K                     0x8000
#define HALF_K                      0x8000/16


/* Port definitions. */
#define NT 0x20 /* Number of Transmit buffers */
#define NR 0x40 /* Number of Receive buffers */
 
#define MAX_BUFF_SIZE           1536
#define BUFF_ALIGNMENT          16 /* Ethernet Buffer alignment            */
#define BUFF_DESC_ALIGNMENT     16 /* Ethernet Buffer descriptor alignment */
  
#define ETH_DEV_INITIALIZED     1


typedef struct mibCounters
{
    UINT32    byteReceived;
    UINT32    byteSent;
    UINT32    framesReceived;
    UINT32    framesSent;
    UINT32    totalByteReceived;
    UINT32    totalFramesReceived;
    UINT32    broadcastFramesReceived;
    UINT32    multicastFramesReceived;
    UINT32    cRCError;
    UINT32    oversizeFrames;
    UINT32    fragments;
    UINT32    jabber;
    UINT32    collision;
    UINT32    lateCollision;
    UINT32    frames64;
    UINT32    frames65_127;
    UINT32    frames128_255;
    UINT32    frames256_511;
    UINT32    frames512_1023;
    UINT32    frames1024_MaxSize;
    UINT32    macRxError;
    UINT32    droppedFrames;
    UINT32    outMulticastFrames;
    UINT32    outBroadcastFrames;
    UINT32    undersizeFrames;
}
STRUCT_MIB_COUNTERS;

typedef struct ETH_DESC 
{
    volatile UINT32          command_status;
    volatile UINT32          bytecount_reserved;
    volatile UINT8           *buff_pointer;
    volatile struct ETH_DESC *next_desc;
}ETH_DESC;

typedef struct ETH_TX_DESC 
{
  volatile UINT32             command_status;
  volatile UINT32             bytecount_reserved;
  volatile UINT8              *buff_pointer;
  volatile struct ETH_TX_DESC *next_desc;
}ETH_TX_DESC;

typedef struct ETH_RX_DESC 
{
  volatile UINT32              command_status;
  volatile UINT32              buff_size_byte_count;
  volatile UINT8               *buff_pointer;
  volatile struct ETH_RX_DESC  *next_desc;
}ETH_RX_DESC;

typedef struct ethernet_priv {
    UINT32          ethernetDriverStatus;
    UINT32          ethernetRegisterBaseAddress;
    UINT32          addressTableHashBase;
    UINT32          addressTableHashSize;
    UINT32          addressTableHashMode;
    ETH_TX_DESC     *eth_tx_queue[NUM_TX_QUEUES];
    ETH_RX_DESC     *eth_rx_queue[NUM_RX_QUEUES];
    UINT32          TDN_ETH[NUM_TX_QUEUES];
    UINT32          TDN_ETH_TMP[NUM_TX_QUEUES];
    UINT32          RDN_ETH[NUM_RX_QUEUES];
    UINT32          ethernet_irq;
    NET_DEV_STATS   stats;
    UINT8           port;
}ETHERNET_PRIV;


typedef PACK_START struct rx8023_DataFrame_t
{
    IEEEtypes_8023_Frame_t DataFrame;
}PACK_END rx8023_DataFrame_t;


typedef PACK_START struct llc_snap_hdr
{
    UINT8  Dsap;
    UINT8  Ssap;
    UINT8  Control;
    UINT8  Org[3];
    UINT16 Type;
}
PACK_END llc_snap_hdr_t;



/* functions definitions */

#endif
