#if !defined(_HAL_PHY_H_)
#define _HAL_PHY_H_

/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*
*
* Filename: hal_phy.h
* Description: This file contains all the registers settings related to
*              the PHY layer of Condor+Peregrine. 
*
******************************************************************************
*/
#define PHY_REG_VER 0x11032003


/*===================================================
 *  Type Definitions   
 *===================================================
 */
typedef struct _PHY_REGS
{
    unsigned char reg;
    unsigned char val;
} PHY_REGS;

typedef struct _SOC2PHY_REGS
{
    unsigned long reg;
    unsigned long val;
} SOC2PHY_REGS;

typedef struct _ChnlSelRegs_t {
	unsigned char r3c;
	unsigned char r3b;
	unsigned char r3a;
} ChnlSelRegs_t;

/*-------------------------------------------------------*/
/* [RF Section]                                          */
/* Peregrine initial register settings                   */
/*-------------------------------------------------------*/
// Version A2 Peregrine
PHY_REGS rfInitRegs[] = {
   {0x02, 0x00},
   {0x02, 0x07},
   {0x07, 0xc8},
   {0x09, 0x18},
   {0x0c, 0x01},     /* Register 0c - TX EVM related change */
   {0x0d, 0xd2},
   {0x0e, 0x12},
   {0x0f, 0x0b},
   {0x13, 0x25},     /* Initial Gain in Peregrine to 54dB */
   {0x14, 0x05},
   {0x18, 0x22},
   {0x19, 0x00},
   {0x1e, 0xcf},
   {0x1f, 0xce},
   {0x20, 0x18},
   {0x3d, 0x9c},     /* Set the reference clock to 40mhz */
   {0x3e, 0x00},
   {0xff, 0xff}      /* end of list */
};

// Version B1 Peregrine
PHY_REGS rfInitRegs_B1[] = {
   {0x02, 0x00},
   {0x02, 0x07},
   {0x06, 0xC0},     // Continuing sample power control
   {0x0a, 0x7B},
   {0x0b, 0x04},
   {0x0c, 0x01},     /* Register 0c - TX EVM related change */
   {0x0d, 0xd2},
   {0x0e, 0x12},
   {0x13, 0x25},     /* Initial Gain in Peregrine to 54dB */
   {0x14, 0x05},
   {0x18, 0x22},
   {0x19, 0x00},
   {0x1e, 0xcf},
   {0x1f, 0xce},
   {0x20, 0x18},
   {0x23, 0x5C},
   {0x3a, 0xd7},     /* Set the reference clock to 40mhz */
   {0x3b, 0xeb},
   {0x3c, 0x00},
   {0x49, 0x1b},
   {0xff, 0xff}      /* end of list */
};

/*-------------------------------------------------------*/
/* SOC settings                                          */
/*-------------------------------------------------------*/
SOC2PHY_REGS soc2phyInitRegs[] = { 
   {0x8000a240, 0x3000},      /* TX_PE Rising edge */
   {0x8000a244, 0x50142020},  /* TX_PE Falling edge */
   {0x8000a660, 0x28},        /* Clock divisor (UFactor) */
   {WL_CIU_CONTROL,0x7136},
   {0xffffffff, 0xffffffff}
};


/*-------------------------------------------------------*/
/* [BBP Section]                                         */
/* Condor Baseband initial register settion              */
/*-------------------------------------------------------*/
#if defined(AP_PGRINE)
PHY_REGS bbpInitRegs[] = {
   {0x00, 0x00},     /* BBP_PAGE_REG_0 to page 0 */
   {0x03, 0xff},     /* BBP_SW_RESET soft reset */
   {0x03, 0x00},
   {0x07, 0x88},     /* Register 07 is programmed to 11g mode */
   {0x17, 0x00},
   {0x29, 0xe0},
   {0x2b, 0x28},     /* AGC Gain Target */
   {0x2c, 0x36},     /* AGC initial gain to 54dB */
   {0x2f, 0x78},
   {0x37, 0xa0},
   {0x38, 0xa0},
   {0x3f, 0x30},     /* TX and RX antennas set to Antenna-0 */
   {0x40, 0x10},
   {0x4c, 0x4f},
   {0x00, 0x80},     /* BBP_PAGE_REG_0 set to page 1 */
   {0x83, 0xd7},
   {0x87, 0x94},
   {0xa4, 0x22},     /* TX_PE latency in baseband set to 1us */
   //{0xc3, 0x90},   /* 90: Disable DC loop */
   {0xce, 0x59},
   {0xd6, 0x54},
   {0xe4, 0x7f},
   {0x00, 0x00},     /* BBP_PAGE_REG_0 to page 0 */
   {0xff, 0xff}      /* end of list */
};
#endif


/*-------------------------------------------------------*/
/* [Board Specific Section]                              */
/* Settings for CB32p                                    */
/*-------------------------------------------------------*/
// TBD


/*-------------------------------------------------------*/
/* RF channel settings                                   */
/* Peregrine rev A and B                                 */
/*-------------------------------------------------------*/
ChnlSelRegs_t rfch_A20Mhz_Regs[] = {
	{0x0e, 0x7a, 0xe1},	//Channel 1
	{0x0e, 0xae, 0x14},	//Channel 2
	{0x0e, 0xe1, 0x48},	//Channel 3
	{0x0f, 0x14, 0x7b},	//Channel 4
	{0x0f, 0x47, 0xae},	//Channel 5
	{0x0f, 0x7a, 0xe1},	//Channel 6
	{0x0f, 0xae, 0x14},	//Channel 7
	{0x0f, 0xe1, 0x48},	//Channel 8
	{0x00, 0x14, 0x7b},	//Channel 9
	{0x00, 0x47, 0xae},	//Channel 10
	{0x00, 0x7a, 0xe1},	//Channel 11
	{0x00, 0xae, 0x14},	//Channel 12
	{0x00, 0xe1, 0x48},	//Channel 13
	{0x01, 0x5c, 0x29} 	//Channel 14
};

ChnlSelRegs_t rfch_A40Mhz_Regs[] = {
	{0x0e, 0x3d, 0x71},	//Channel 1
	{0x0e, 0x57, 0x0a},	//Channel 2
	{0x0e, 0x70, 0xa4},	//Channel 3
	{0x0e, 0x8a, 0x3d},	//Channel 4
	{0x0e, 0xa3, 0xd7},	//Channel 5
	{0x0e, 0xbd, 0x71},	//Channel 6
	{0x0e, 0xd7, 0x0a},	//Channel 7
	{0x0e, 0xf0, 0xa4},	//Channel 8
	{0x0f, 0x0a, 0x3d},	//Channel 9
	{0x0f, 0x23, 0xd7},	//Channel 10
	{0x0f, 0x3d, 0x71},	//Channel 11
	{0x0f, 0x57, 0x0a},	//Channel 12
	{0x0f, 0x70, 0xa4},	//Channel 13
	{0x0f, 0xae, 0x14} 	//Channel 14
};

ChnlSelRegs_t rfch_A44Mhz_Regs[] = {
	{0x0f, 0xda, 0xc3},	//Channel 1
	{0x0f, 0xf2, 0x09},	//Channel 2
	{0x00, 0x09, 0x4f},	//Channel 3
	{0x00, 0x20, 0x95},	//Channel 4
	{0x00, 0x37, 0xdb},	//Channel 5
	{0x00, 0x4f, 0x21},	//Channel 6
	{0x00, 0x66, 0x66},	//Channel 7
	{0x00, 0x7d, 0xac},	//Channel 8
	{0x00, 0x94, 0xf2},	//Channel 9
	{0x00, 0xac, 0x38},	//Channel 10
	{0x00, 0xc3, 0x7e},	//Channel 11
	{0x00, 0xda, 0xc3},	//Channel 12
	{0x00, 0xf2, 0x09},	//Channel 13
	{0x01, 0x29, 0xe4} 	//Channel 14
};

ChnlSelRegs_t rfch_B20Mhz_Regs[] = {
	{0x01, 0xe7, 0xae},	//Channel 1
	{0x04, 0xea, 0xe1},	//Channel 2
	{0x08, 0xee, 0x14},	//Channel 3
	{0x0B, 0xf1, 0x47},	//Channel 4
	{0x0E, 0xf4, 0x7a},	//Channel 5
	{0x01, 0xf7, 0xae},	//Channel 6
	{0x04, 0xfa, 0xe1},	//Channel 7
	{0x08, 0xfe, 0x14},	//Channel 8
	{0x0B, 0x01, 0x47},	//Channel 9
	{0x0E, 0x04, 0x7a},	//Channel 10
	{0x01, 0x07, 0xae},	//Channel 11
	{0x04, 0x0a, 0xe1},	//Channel 12
	{0x08, 0x0e, 0x14},	//Channel 13
	{0x09, 0x15, 0xc2} 	//Channel 14
};

ChnlSelRegs_t rfch_B40Mhz_Regs[] = {
	{0x01, 0xe3, 0xd7},	//Channel 1
	{0x0A, 0xe5, 0x70},	//Channel 2
	{0x04, 0xe7, 0x0a},	//Channel 3
	{0x0D, 0xe8, 0xa3},	//Channel 4
	{0x07, 0xea, 0x3d},	//Channel 5
	{0x01, 0xeb, 0xd7},	//Channel 6
	{0x0A, 0xed, 0x70},	//Channel 7
	{0x04, 0xef, 0x0a},	//Channel 8
	{0x0D, 0xf0, 0xa3},	//Channel 9
	{0x07, 0xf2, 0x3d},	//Channel 10
	{0x01, 0xf3, 0xd7},	//Channel 11
	{0x0A, 0xf5, 0x70},	//Channel 12
	{0x04, 0xf7, 0x0a},	//Channel 13
	{0x04, 0xfa, 0xe1} 	//Channel 14
};

ChnlSelRegs_t rfch_B44Mhz_Regs[] = {
	{0x03, 0xfd, 0xac},	//Channel 1
	{0x09, 0xff, 0x20},	//Channel 2
	{0x0F, 0x00, 0x94},	//Channel 3
	{0x05, 0x02, 0x09},	//Channel 4
	{0x0B, 0x03, 0x7d},	//Channel 5
	{0x01, 0x04, 0xf2},	//Channel 6
	{0x06, 0x06, 0x66},	//Channel 7
	{0x0C, 0x07, 0xda},	//Channel 8
	{0x02, 0x09, 0x4f},	//Channel 9
	{0x08, 0x0a, 0xc3},	//Channel 10
	{0x0E, 0x0c, 0x37},	//Channel 11
	{0x03, 0x0d, 0xac},	//Channel 12
	{0x09, 0x0f, 0x20},	//Channel 13
	{0x04, 0x12, 0x9e} 	//Channel 14
};

#ifdef CHANNEL14_SUPPORT
PHY_REGS rfCh14Regs[2][2]={
  {
    {0x39, 0x1f},
    {0xff, 0xff} },
  {
    {0x39, 0x10},
    {0xff, 0xff} }
};

PHY_REGS bbpCh14Regs[2][18]={
  {
    {0x59, 0x80},
    {0x5a, 0xbe},
    {0x5b, 0xbb},
    {0x5c, 0xb8},
    {0x5d, 0xb4},
    {0x5e, 0xb3},
    {0x61, 0xb6},
    {0x62, 0x80},
    {0x63, 0x84},
    {0x64, 0x8b},
    {0x65, 0x93},
    {0x66, 0x9b},
    {0x67, 0xa4},
    {0x68, 0xab},
    {0x6a, 0x70},
    {0x6b, 0x72},
    {0x6c, 0x40},
    {0xff, 0xff} },
  {
    {0x59, 0xbb},
    {0x5a, 0x87},
    {0x5b, 0x8e},
    {0x5c, 0x94},
    {0x5d, 0x8e},
    {0x5e, 0x83},
    {0x61, 0xb0},
    {0x62, 0xa0},
    {0x63, 0xb5},
    {0x64, 0xb7},
    {0x65, 0xbd},
    {0x66, 0x88},
    {0x67, 0x95},
    {0x68, 0xa2},
    {0x6a, 0x6b},
    {0x6b, 0x6f},
    {0x6c, 0x40},
    {0xff, 0xff} }
};
#endif

#endif /* _HAL_PHY_H_ */

