#if !defined(_PCI_H_)
#define _PCI_H_ 
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file  pci.h
 * \brief Definitions for PCI HAL
 */

/*--------------------------------------------------------------*/
/*  PCI Host Interface (_PMI_) registers                        */
/*--------------------------------------------------------------*/

/* Macro to compute address of register in PCI Host Interface */
#define PHI_REG_ADDR(offset) ((WL_HIU_CFG_BASE) | offset)

#define WL_PHI_PCICFGARRD     PHI_REG_ADDR(0x0CF8)
#define WL_PHI_PCICFGDATA     PHI_REG_ADDR(0x0CFC)

#define WL_PHI_CONTROL     PHI_REG_ADDR(0xC00)
#define WL_PHI_BANK_SIZE   PHI_REG_ADDR(0xC04)
#define WL_PHI_BAR0_RMAP   PHI_REG_ADDR(0xC08)
#define WL_PHI_BAR1_RMAP   PHI_REG_ADDR(0xC0C)
#define WL_PHI_GEN_PTR     PHI_REG_ADDR(0xC10)
#define WL_PHI_INTCODE     PHI_REG_ADDR(0xC14)
#define WL_PHI_ITR         PHI_REG_ADDR(0xC18)
#define WL_PHI_ISR         PHI_REG_ADDR(0xC1C) /*interrupt status */
#define WL_PHI_IMR         PHI_REG_ADDR(0xC20) /*interrupt mask */
#define WL_PHI_RSR         PHI_REG_ADDR(0xC24)
#define WL_PHI_SMR         PHI_REG_ADDR(0xC28) /*interrupt status mask */
#define WL_PHI_HOST_ITR    PHI_REG_ADDR(0xC2C)
#define WL_PHI_HOST_ISR    PHI_REG_ADDR(0xC30) /*interrupt status */
#define WL_PHI_HOST_IMR    PHI_REG_ADDR(0xC34) /*interrupt mask */
#define WL_PHI_HOST_RSR    PHI_REG_ADDR(0xC38)
#define WL_PHI_HOST_SMR    PHI_REG_ADDR(0xC3C)

#define PPA_SIZE        32
#define DPA_SIZE        32
#define WCB_ARRAY_SIZE  32*8
#define FW_WCB_SIZE    32

#define SIGNATURE      0xF0F1F2F4
#define FW_PRESENT     0xF4F2F1F0
#define FW_PRESENT_WORD     0xC000bffc

/* Bit definitions for W81_PHI_ISR */
#define ISR_PPA_RDY         (1<<0) /* Host to ARM will be 24 - 31 */
#define ISR_DOORBELL        (1<<1)
#define ISR_RESET           (1<<26)
#define ISR_RETRYEXPIRED    (1<<27)

/* Bit definitions for W81_PHI_IMR */
#define IMR_PPA_RDY         (1<<0)
#define IMR_DOORBELL        (1<<1)
#define IMR_RESET           (1<<26)
#define PIU_MASK            (1<<0)     /* 1 = Mask PIU interrupt to ARM */
#define IMR_RETRYEXPIRED    (1<<27)

/* */
/* new register add in the rev4 chip, write 0 to this register to re-enable */
/* host to arm interrupt */
/* */
/* Bit definitions for W81_PHI_ICR */
#define ICR_PPA_RDY         (1<<0)
#define ICR_DOORBELL        (1<<1)
#define ICR_RESET           (1<<26)
#define ICR_RETRYEXPIRED    (1<<27)


/* Bit definitions for W81_PHI_RSR */
#define RSR_PPA_RDY         (1<<0)
#define RSR_DOORBELL        (1<<1)
#define RSR_RESET           (1<<26)
#define RSR_RETRYEXPIRED    (1<<27)

/* Bit definitions for W81_PHI_HOST_ISR */
#define ISR_TXDONE          (1<<0)
#define ISR_RXDONE          (1<<1)
#define ISR_OPCDONE         (1<<2)
#define ISR_MACEVENTS       (1<<3)

/* Bit definitions for W81_PHI_HOST_IMR */
#define IMR_TXDONE          (1<<0)
#define IMR_RXDONE          (1<<1)
#define IMR_OPCDONE         (1<<2)
#define IMR_MACEVENTS       (1<<3)
/* new in rev4 chip */
#define IMR_INT_DISABLE     (1<<16)
#define IMR_HOST_MASK       (1<<0)     /* 1 = mask pci interrupt to host */

/* Bit definitions for W81_PHI_HOST_RSR */
#define RSR_TXDONE          (1<<0)
#define RSR_RXDONE          (1<<1)
#define RSR_OPCDONE         (1<<2)
#define RSR_MACEVENTS       (1<<3)

#endif /* _PCI_H_ */

